// EMData.C - Steve Ludtke  6/97
// This is the heart of the EMAN project. It does just about all of the
// hard stuff. It reads and writes EM images in various formats, does FFTs,
// filters, etc...
#include <stdio.h>
#ifdef _WIN32
#include <UnixToWin.h>
#else
#include <unistd.h>
#endif
#include "limits.h"
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "List.h"
#include "mrc.h"
#include "imagic.h"
#include "emim.h"
#include "gatan.h"
#include "util.h"
#include "EMData.h"
#include "EMPif.h"
#include "EMVtk.h"
#include "EMIcos.h"
#include "GatanDM3.h"
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include "portable_fileio.h"
#include <tiffio.h>

//#define DEBUG
//#define DEBUG2


// subtract 2 angles (radians)
inline float asub(float x,float y) {
  float r;
  r=fabs(x-y);
  if (r>PI) r=2.0*PI-r;
  return r;
}

// subtract 2 angles (radians)
inline float asub2(float x,float y) {
  float r;

  r=fmod(fabs(x-y),(float)PI);
  if (r>PI/2.0) r=PI-r;
  return r;
}

float acmp(float a0,float a1,float a2) {
  if (a0<=a1 && a1<=a2) return 1;
  if (a0<=a1+PI && a1+PI<a2) return 1;
  if (a0<=a1+2.0*PI && a1+2.0*PI<a2) return 1;
  if (a0<=a1-PI && a1-PI<a2) return 1;
  if (a0<=a1-2.0*PI && a1-2.0*PI<a2) return 1;
  return 0;
}



//#define PI	3.141592654		// silly, but sometimes there are problems with M_PI

// This is used to define the width of the gaussian for 5x5x5 interpolation
// in 3d slices
#define I5G	(10.4/(PI*PI))	// used for 5x5x5 interpolation
#define I4G	(8.8/(PI*PI))	// used for 4 interpolation
#define I3G	(6.4/(PI*PI))	// used for 3 and 5x5x5 interpolation
#define I2G	(4.0/(PI*PI))	// 2 interpolation


List *EMData::ralfp=NULL;
int EMData::rali=0;
int EMData::ralo=0;
int EMData::rals=0;

rfftwnd_plan EMData::plan[6] = { NULL,NULL,NULL,NULL,NULL,NULL };
int EMData::dimplan[6] = { 0,0,0,0,0,0 };
int EMData::fwplan[6] = { 0,0,0,0,0,0 };
int EMData::nplan[6][3] = { 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0, 0 , 0 , 0 , 0 , 0 , 0 };
int EMData::plans=0;

EMData::EMData() {
  //int i;

  nx=ny=nz=0;
  rdata=NULL;
  dx=dy=dz=0;
  euler.setAngle(0,0,0);
  min=max=mean=sigma=0;
  pixel=4.667f;
  //for (i=0; i<200; i++) hist[i]=0;
  //histmin=histmax=0;
  mrch=NULL;
  imagich=NULL;
  spiderH=NULL;
  path[0]=0;
  pathnum=0;
  name[0]=0;
  parent=NULL;
  fft=NULL;
  rocount=0;
  rfp=NULL;
  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  supp=NULL;
  ufloat=ufloat2=0;
  updProgress=NULL;
  unum=unum2=unum3=0;

  if (!ralfp) {
    ralfp=new List;
    rali=ralo=rals=0;
  }
}

EMData::~EMData() {
#if 1
  if (rdata) Sfree(rdata,1);
  if (mrch) Sfree(mrch,1);
  if (spiderH) free(spiderH);
  if (imagich) Sfree(imagich,1);
  if (fft) delete fft;
  if (rfp) delete rfp;
  if (supp) free(supp);
#endif
}

void *EMData::operator new(size_t size) {
  return Smalloc(sizeof(EMData),1);
}

void EMData::operator delete(void *ptr) {
  Sfree(ptr,1);
}

int EMData::swapout() {
  char s[MAXPATH];
  FILE *out;

  if (flags&EMDATA_SWAPPED) return 0;

  tmp=random();
  sprintf(s,"/usr/tmp/tmp.%d",tmp);
  out=fopen(s,"w");
  if (!out) { error("ERROR: Cannot Swap",ERR_CRIT); return -1; }
  if(fwrite(rdata,sizeof(float)*nx,ny*nz,out)!=ny*nz) {
    error("ERROR: Swap space full!",ERR_CRIT); 
    fclose(out);
    unlink(s);
    return -2;
  }
  fclose(out);
  Sfree(rdata,1);
  flags|=EMDATA_SWAPPED;
  return 0;
}

int EMData::swapin() {
  char s[MAXPATH];
  FILE *in;

  if (!(flags&EMDATA_SWAPPED)) return 0;

  sprintf(s,"/usr/tmp/tmp.%d",tmp);
  in=fopen(s,"r");
  if (!in) { error("ERROR: Swap file missing!",ERR_CRIT); return -1; }
  rdata=(float *)Smalloc(nx*ny*nz*sizeof(float),1);
  if (!rdata) { error("Cannot swap in, not enough memory!",ERR_CRIT); return -1; }
  if(fread(rdata,sizeof(float)*nx,ny*nz,in)!=ny*nz) {
    error("ERROR: Swap file incomplete!",ERR_CRIT); 
    return -1;
  }
  flags&=~EMDATA_SWAPPED;
  return 0;
}


const char *VALIDFMT[] = { "gif","pgm","ppm","png","pnm","rgb","tga","bmp" };
const int NVALIDFMT = 8;// was 10;

// general creation

// returns number of images in a file, -1 if none
// type 1=spider, 2=imagic, 4=PGM/gif/etc, 5=LST file, 6=PIF file,
// 7=DM3 file,8=TIFF, 9=VTK, 0=MRC/other
int fileCount(const char *filespec,int *type) {
  int i,l;
  char *s,*v;
  FILE *in;

  if (!filespec) { printf("NULL filecount()\n"); return -1; }
  l=strlen(filespec);
  s=(char *)malloc(l+1);
  v=(char *)&i;
  strcpy(s,filespec);

  // LST file test
  in=fopen(filespec,"r");
  if (!in) return 0;
  char ll[MAXPATHLEN+10];
  fgets(ll,MAXPATHLEN,in);
  if (strncmp(ll,"#LST",4)==0) {
    for (i=0; fgets(ll,MAXPATHLEN,in)!=NULL; i++) if (ll[0]=='#') i--;
    fclose(in);
    if (type) *type=5;
    return i;
  }
  fclose(in);

  // SPIDER file test
  //if (!strcmp(filespec+l-4,".DAT") || !strcmp(filespec+l-4,".dat") ||
  //	!strcmp(filespec+l-4,".spi") ) 
  {
    SPIDERH *sh=readSpiHed(filespec);
    if (sh) {
      if ((sh->type==1.0 || sh->type==3.0) && sh->nslice<65536 &&
	  sh->ny<65536) {
	if (type) *type=1;
	if (sh->istack==2) { int r=sh->maxim; free(sh); return r; }
	free(sh);
	return 1;
      }
      free(sh);
    }
  }

  // IMAGIC file test
  if (!strcmp(filespec+l-4,".hed") || !strcmp(filespec+l-4,".img")) {
    s[l-3]=0;
    strcat(s,"img");
    if (access(s,R_OK)) i=0; else i=1;
    s[l-3]=0;
    strcat(s,"hed");
    if (!access(s,R_OK) && i) {		//ok, both an img and a hed, probably an IMAGIC file
      i=0;
      in=fopen(s,"r");
      fread(&i,4,1,in);
      fread(&i,4,1,in);
      fclose (in);
#ifdef MSB_FIRST
      if (v[0]!=0||v[1]!=0) swapint(v,1);
#else
      if (v[0]==0&&v[1]==0) swapint(v,1);
#endif
      i++;
      if (type) *type=2;
      return i;
    }
  }
  strcpy(s,filespec);

  // check for EMIM
  in=fopen(s,"r");
  if (!in) return 0;
  if (fread(&i,4,1,in)!=1) { fclose(in); return 0; }
  if (!strncmp(v,"EMIM",4)) {
    fread(&i,4,1,in);
    if (i==2) l=0; else l=1;
    fread(&i,4,1,in);
    fclose(in);
    if (l) swapint(v,1);
    return i;
  }
  fclose(in);

  // check for DM3

  Gatan::DM3Reader dm3_file(filespec);
  if (dm3_file.init() == 0) {
    if (type) {
      *type = 7;
    }
    return 1;
  }

 
  // check for TIFF (###)
  if (!strcmp(filespec+l-4,"tiff")||!strcmp(filespec+l-3,"tif")
      ||!strcmp(filespec+l-3,"TIF")||!strcmp(filespec+l-4,"TIFF")) {
    if (type) {
      *type=8; //
    }
    return 1;// only single image suppor currently
  }  

 
  // check for PIF    
  EMPif pif_file(filespec);

  if (pif_file.is_valid_header()) {
    int n_pif_img = pif_file.get_nimg();
    
    if (n_pif_img > 0) {
      if (type) {
	*type = 6;
      }
      return n_pif_img;
    }
  }

  // check for VTK
  EMVtk vtkfile(filespec);
  if(vtkfile.init() == 0) {
      if (type) {
	  *type = 9;
      }
      return 1;
  }

 
  // check for standard types like GIF, etc.
  char EXT[5]= {0,0,0,0,0};
  if (filespec[l-4]=='.') strncpy(EXT,filespec+l-3,3);
  else if (filespec[l-5]=='.') strncpy(EXT,filespec+l-4,4);
  for (i=0; i<strlen(EXT); i++) EXT[i]=tolower(EXT[i]);
  for (i=0; i<NVALIDFMT; i++) if (!strcmp(EXT,VALIDFMT[i])) break;

  if (type && i!=NVALIDFMT) *type=4;
  else if (type) *type=0;

  // Other image types are 1 image only
  return 1;
}

// reads a set of filtered images with an automatically generated filename
// This is really inefficient, but for now it will do...
List *readImagesExt(const char *filespec,const char *xt,int n0,int n1,int nodata)	{
  int i,n,in=0;
  List *rtn = new List();
  EMData *data;
  char *fsp2=(char *)malloc(strlen(filespec)+7);

  if (!xt) return readImages(filespec,n0,n1,nodata);

  strcpy(fsp2,filespec);
  strcpy(fsp2+strlen(fsp2)-4,xt);
  strcat(fsp2,filespec+strlen(filespec)-4);

  n=fileCount(fsp2);
  if (n==0) {		// so we're looking at a lst file, and must modify the filename in the lst
    in=1;
    n=fileCount(filespec);
    if (n==0) { free(fsp2); return NULL; }
  }
  if (n0<0) n0=0;
  if (n1>=n || n1<0) n1=n-1;

  for (i=n0; i<=n1; i++) {
    data=new EMData();
    if (in) {
      if (data->readLST(filespec,i,nodata,xt)<0) { delete data; continue; }
    }
    else {
      if (data->readImage(fsp2,i,nodata)<0) { delete data; continue; }
    }

    rtn->addObject(data);
  }
  free (fsp2);
  return rtn;
}

// reads a set of images
// This is really inefficient, but for now it will do...
List *readImages(const char *filespec,int n0,int n1,int nodata)	{
  int i,n;
  List *rtn = new List();
  EMData *data;

  n=fileCount(filespec);
  if (n==0) return NULL;
  if (n0<0) n0=0;
  if (n1>=n || n1<0) n1=n-1;

  for (i=n0; i<=n1; i++) {
    data=new EMData();
    if (data->readImage(filespec,i,nodata)<0) { delete data; continue; }
    rtn->addObject(data);
  }
  return rtn;
}

int EMData::readImage(const char *filespec, int n,int nodata) {			// reads a single image, 0 on sucess
  int i,l,r;
  FILE *in;

  if (!filespec) { printf("NULL readimage()\n"); return(-1); }
  l=strlen(filespec);

  // LST file test
  in=fopen(filespec,"r");
  if (!in) return -1;
  char ll[10];
  fgets(ll,9,in);
  fclose(in);
  if (strncmp(ll,"#LST",4)==0) {
    r=readLST(filespec,n,nodata);
    if (r) { setSize(10,10,1); zero(); }
    return r;
  }

  // SPIDER file test
  //if (!strcmp(filespec+l-4,".DAT") || !strcmp(filespec+l-4,".dat") ||
  //	!strcmp(filespec+l-4,".spi") ) {
  {
    SPIDERH *sh=readSpiHed(filespec);
    if (sh) {
      if ((sh->type==1.0 || sh->type==3.0) && sh->nslice<65536 &&
	  sh->ny<65536) {
	free(sh); 
	r=readSPIDER(filespec,n,nodata);
	if (r) { setSize(10,10,1); zero(); }
	return r;
      }
      free(sh);
    }
  }

  // Gatan file format test, MUST have dm2 
  if (!strcmp(filespec+l-4,".dm2") || !strcmp(filespec+l-4,".DM2") ) {
    struct gatanH hed;
	
    in=fopen(filespec,"r");
    if (!in) { error(ERR_ERROR,"Could not read Gatan file"); return -1; }
    fread(&hed,sizeof(gatanH),1,in);
    fclose(in);

    //printf("Gatan %hd %hd %hd %hd\n",hed.nx,hed.ny,hed.type,hed.len);
    if (hed.len<=4 && hed.len>0 && hed.type>0 && hed.type<8) {
      r=readGatan(filespec,nodata);
      if (r) { setSize(10,10,1); zero(); }
      return r;		
    }
    swapshort((char *)&hed,7);
    if (hed.len<=4 && hed.len>0 && hed.type>0 && hed.type<8) {
      r=readGatan(filespec,nodata);
      if (r) { setSize(10,10,1); zero(); }
      return r;		
    }	
  }

  char *s=(char *)malloc(l+1);
  char *v=(char *)&i;
  strcpy(s,filespec);
  // IMAGIC file test
  if (!strcmp(filespec+l-4,".hed") || !strcmp(filespec+l-4,".img")) {
    s[l-3]=0;
    strcat(s,"img");
    if (access(s,R_OK)) i=0; else i=1;
    s[l-3]=0;
    strcat(s,"hed");
    if (!access(s,R_OK) && i) {		//ok, both an img and a hed, probably an IMAGIC file
      i=0;
      in=fopen(s,"r");
      fread(&i,4,1,in);
      fread(&i,4,1,in);
      fclose (in);
#ifdef MSB_FIRST
      if (v[0]!=0) swapint(v,1);
#else
      if (v[0]==0) swapint(v,1);
#endif
      if (n>i) return(-1);
      if (n<0) {
	r=readIMAGIC3d(s,nodata);
	if (r) { setSize(10,10,1); zero(); }
	return r;
      }	
      r=readIMAGIC(s,n,nodata);
      if (r) { setSize(10,10,1); zero(); }
      return r;
    }
  }
  strcpy(s,filespec);

  // SAL file test
  if (!strcmp(filespec+l-4,".hdr") || !strcmp(filespec+l-4,".img") || 
      !strcmp(filespec+l-4,".HDR") || !strcmp(filespec+l-4,".IMG")) {
    s[l-3]=0;
    strcat(s,"img");
    if (access(s,R_OK)) i=0; else i=1;
    if (!i) {
      s[l-3]=0;
      strcat(s,"IMG");
      if (access(s,R_OK)) i=0; else i=2;
    }
	
    s[l-3]=0;
    if (i==2) strcat(s,"HDR");
    else strcat(s,"hdr");
    if (!access(s,R_OK) && i) {		//ok, both an img and a hdr, probably a SAL file
      i=0;
      in=fopen(s,"r");
      fread(&i,4,1,in);
      fread(&i,4,1,in);
      fclose (in);
#ifdef MSB_FIRST
      if (v[0]!=0) swapint(v,1);
#else
      if (v[0]==0) swapint(v,1);
#endif
      if (n>i) return(-1);
      r=readSAL(s,nodata);
      if (r) { setSize(10,10,1); zero(); }
      return r;
    }
  }
  strcpy(s,filespec);

  char EXT[5]= {0,0,0,0,0};
  if (filespec[l-4]=='.') strncpy(EXT,filespec+l-3,3);
  else if (filespec[l-5]=='.') strncpy(EXT,filespec+l-4,4);
  for (i=0; i<strlen(EXT); i++) EXT[i]=tolower(EXT[i]);
  for (i=0; i<NVALIDFMT; i++) 
    if (!strcmp(EXT,VALIDFMT[i])) break;

  //printf("EXT= '%s'   %d\n",EXT,i);
  if (i!=NVALIDFMT) {
    r=readPGM(filespec,nodata);
    if (r) { setSize(10,10,1); zero(); }
    return r;
  }

  // check for EMIM
  strcpy(s,filespec);
  in=fopen(s,"r");
  if (!in) return -2;
  if (fread(&i,4,1,in)!=1) { fclose(in); return -2; }
  if (!strncmp(v,"EMIM",4)) {
    fread(&i,4,1,in);
    if (i==2) l=0; else l=1;
    fread(&i,4,1,in);
    fclose(in);
    if (l) swapint(v,1);
    if (n>i) return(-1);
    return readEMIM(filespec,n,nodata);
  }
  fclose(in);

  //check for ICOS
  EMIcos icos_file(filespec);
  if (icos_file.is_valid_header()) {
    return readICOS(filespec,nodata);
  }

  // check for DM3
  Gatan::DM3Reader dm3_file(filespec);
  if (dm3_file.init() == 0) {
    int err = readDM3(filespec, nodata);
    if (err) {
      error("failed in readDM3()", ERR_ERROR);
      setSize(10, 10, 1);
      zero();
    }
    return err;
  }

    
  // check for PIF
  EMPif pif_file(filespec);

  if (pif_file.is_valid_header()) {
    int n_pif_img = pif_file.get_nimg();
#ifdef DEBUG
    printf("readImage: nimg = %d, which_img = %d\n", n_pif_img, n);
#endif
	
    if (n_pif_img > 0) {
      if (n > n_pif_img) {
	return -1;
      }
	    
      int err = readPIF(filespec, n, nodata);
	    
      if (err) {
	error("failed in readPIF()", ERR_ERROR);
	setSize(10, 10, 1);
	zero();
      }
      return err;
    }
  }

  // check for VTK
  EMVtk vtkfile(filespec);
  if (vtkfile.init() == 0) {
    int vtk_err = readVTK(filespec, nodata);
    if (vtk_err) {
      error("failed in readVTK()", ERR_ERROR);
      setSize(10, 10, 1);
      zero();
    }
    return vtk_err;
  }


    
  // check for TIFF image 
  if (!strcmp(filespec+l-4,"tiff")||!strcmp(filespec+l-3,"tif")||
      !strcmp(filespec+l-3,"TIF")||!strcmp(filespec+l-4,"TIFF")) {
    r=readTIFF(filespec,nodata);
    if (r) { setSize(10,10,1); zero(); }
    return 0;
  }
   

  // Ok, at this point we assume MRC
  r=readMRC(filespec,nodata,n);
  if (r) { setSize(10,10,1); zero(); }
  return r;
}

void EMData::serialIn(int filedes) {
  readsock(filedes,(char *)this,sizeof(EMData));

  //imagich=(imagicH*)malloc(sizeof(imagicH));
  //readsock(filedes,(char *)imagich,sizeof(imagicH));
  if(!(flags&EMDATA_NODATA)) {
    rdata=(float *)malloc(nx*ny*nz*sizeof(float));
    getData();
    readsock(filedes,(char *)rdata,nx*ny*nz*sizeof(float));
    doneData();
  }

  supp=NULL;
  mrch=NULL;
  imagich=NULL;
  spiderH=NULL;
  parent=NULL;
  rfp=NULL;
  fft=NULL;
}

void EMData::serialOut(int filedes) {
  writesock(filedes,(char *)this,sizeof(EMData));
  //writesock(filedes,(char *)imagich,sizeof(imagicH));
  if(!(flags&EMDATA_NODATA)) writesock(filedes,(char *)rdata,nx*ny*nz*sizeof(float));
}

EMData *EMData::copy(int withfft,int withparent) {		// returns an exact copy of itself
  EMData *ret;
  float *ndata;

  getDataRO();
  ret=new EMData;
  ret->setSize(nx,ny,nz);
  ret->setTAlign(dx,dy,dz);
  *ret->getEuler()=euler;
  ret->setNImg(nimg);
  ret->setPixel(pixel);
  ndata=ret->getData();
  memcpy(ndata,rdata,nx*ny*nz*sizeof(float));
  ret->doneData();
  //ret->calcHist(histmin,histmax);
  ret->update();
  if (withparent) ret->setParent(this); else ret->setParent(NULL);
  ret->setName(name);
  ret->setPath(path);
  ret->setPathN(pathnum);
  ret->unum=unum;
  ret->fft=ret->rfp=NULL;
  ret->flags=flags&(EMDATA_COMPLEX|EMDATA_RI|EMDATA_HASCTF);
  ret->update();
  if (fft && withfft) ret->fft=fft->copy();
  memcpy(ret->ctf,ctf,11*sizeof(float));
  doneData();
  return ret;
}

EMData *EMData::copyHead() {		// returns an exact copy of itself
  EMData *ret;
  float *ndata;

  ret=new EMData;
  ret->setSize(nx,ny,nz);
  ret->setTAlign(dx,dy,dz);
  *ret->getEuler()=euler;
  ret->setPixel(pixel);
  ret->update();
  ret->setParent(this);
  ret->setName(name);
  ret->setPath(path);
  ret->setPathN(pathnum);
  ret->flags=flags&(EMDATA_COMPLEX|EMDATA_RI|EMDATA_HASCTF);
  ret->fft=ret->rfp=NULL;
  ret->setNImg(nimg);
  memcpy(ret->ctf,ctf,11*sizeof(float));
  ret->update();

  return ret;
}

void EMData::insertClip(EMData *block,int x0, int y0) {
  float *in,*out;
  int i;

  if (x0<0||y0<0||x0+block->xSize()>xSize()||y0+block->ySize()>ySize()) error(ERR_EXIT,"insertClip outside destination image not supported");
  in=block->getDataRO();
  out=getData();

  for (i=0; i<block->ySize(); i++) {
    memcpy(out+x0+(y0+i)*xSize(),in+i*block->xSize(),block->xSize()*sizeof(float));
  }

  block->doneData();
  doneData();
}

EMData *EMData::clip(int x0,int y0,int w,int h)	{	// clip. 0 Pads if larger than data
  EMData *ret;
  int i,j,ys0,ys1,yd0,xs0,xs1,xd0;
  float *src,*dst;

  getDataRO();
  if (nz!=1) { error("Tried to 2d clip 3d image",ERR_ERROR); return NULL; }
  ret=new EMData;
  ret->setSize(w,h,1);
  memset(dst=ret->getData(),0,w*h*sizeof(float));
  ys0=y0<0?0:y0;
  ys1=y0+h>ny?ny:y0+h;
  yd0=y0<0?-y0:0;
  xs0=x0<0?0:x0;
  xs1=x0+w>nx?nx:x0+w;
  xd0=x0<0?-x0:0;
  src=rdata+ys0*nx+xs0;
  dst+=yd0*w+xd0;
  for (i=ys0; i<ys1; i++,src+=nx,dst+=w) {
    memcpy(dst,src,(xs1-xs0)*sizeof(float));
  }
  ret->doneData();
  ret->setPixel(pixel);
  //ret->calcHist(histmin,histmax);
  ret->update();
  ret->setParent(NULL);
  ret->setName(name);
  ret->setPath(path);
  ret->setPathN(pathnum);
  *ret->getEuler()=euler;
  ret->setNImg(nimg);
  doneData();
  return ret;
}

EMData *EMData::clip(int x0,int y0,int z0,int w,int h,int d) {		// clip. Pads if larger than data
  EMData *ret;
  int i,j,jj,ys0,ys1,yd0,xs0,xs1,xd0,zs0,zs1,zd0;
  float *src,*dst,*d2;

  getDataRO();
  ret=new EMData;
  ret->setSize(w,h,d);
  memset(dst=ret->getData(),0,w*h*d*sizeof(float));
  zs0=z0<0?0:z0;
  zs1=z0+d>nz?nz:z0+d;
  zd0=z0<0?-z0:0;
  ys0=y0<0?0:y0;
  ys1=y0+h>ny?ny:y0+h;
  yd0=y0<0?-y0:0;
  xs0=x0<0?0:x0;
  xs1=x0+w>nx?nx:x0+w;
  xd0=x0<0?-x0:0;
  for (j=zs0,jj=zd0; j<zs1; j++,jj++) {
    src=rdata+j*nx*ny+ys0*nx+xs0;
    d2=dst+jj*w*h+yd0*w+xd0;
    for (i=ys0; i<ys1; i++,src+=nx,d2+=w) {
      memcpy(d2,src,(xs1-xs0)*sizeof(float));
    }
  }
  doneData();
  ret->doneData();
  ret->setPixel(pixel);
  //ret->calcHist(histmin,histmax);
  ret->update();
  ret->setParent(NULL);
  ret->setName(name);
  ret->setPath(path);
  ret->setPathN(pathnum);
  return ret;
}


// read routines for specific formats
int EMData::readSAL(const char *filespec,int nodata)		//scans-a-lot
{
  FILE *in;
  char s[81],s2[10],*t;
  int i,j,rl=512,bl,sm=0;
  unsigned char *cdata;
  short *sdata,sw;

  setPath(filespec);
  if (strrchr(filespec,'/')==NULL) setName("SAL File");
  else setName(strrchr(filespec,'/')+1);

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  nx=ny=-1;
  nz=1;

  pixel=4.6667f;
  sprintf((char *)(filespec+strlen(filespec)-4),".HDR");
  if (access(filespec,R_OK)) sprintf((char *)(filespec+strlen(filespec)-4),".hdr");

  i=0;
  if ((in=fopen(filespec,"r"))==NULL) { sprintf(s,"Cannot open %s\n",filespec); error(s,ERR_ERROR); return -1; }
  while (fgets(s,80,in)!=NULL) {
    if (s[0]==0) continue;
    if (strncmp(s+1,"NXP",3)==0) {
      sscanf(strchr(s,'=')+1," %d",&nx);
    }
    else if (strncmp(s+1,"NYP",3)==0) {
      sscanf(strchr(s,'=')+1," %d",&ny);
    }
    else if (strncmp(s+1,"AXSCAN",6)==0) {
      t=strrchr(s,'\'');
      if (t[-1]=='Y') i=1;
    }
    else if (strncmp(s+1,"FILE REC",8)==0) {
      sscanf(strchr(s,'=')+1," %d",&rl);
    }
    else if (strncmp(s+1,"SCAN TYPE",9)==0) {
      sscanf(strchr(s,'\'')+1," %s",s2);
      if (s2[0]=='R') sm=1;
    }
    else if (strncmp(s+1,"DELTAX",6)==0) {
      sscanf(strchr(s,'=')+1," %f",&pixel);
      pixel/=3.0;
    }
  }	
  fclose(in);

  if (i) { i=nx; nx=ny; ny=i; }
  sprintf(s,"%s is %dx%d",filespec,nx,ny);
  error(s,ERR_INFO);
  if (nodata) flags|=EMDATA_NODATA;
  setSize(nx,ny,1);
  cdata=(unsigned char *)rdata;
  sdata=(short *)rdata;
  if (nx==-1 || ny==-1) { error("Problem reading header.",ERR_ERROR); return -2; }

  sprintf((char *)(filespec+strlen(filespec)-4),".IMG");
  if (access(filespec,R_OK)) sprintf((char *)(filespec+strlen(filespec)-4),".img");

  if ((in=fopen(filespec,"r"))==NULL) { printf("Cannot open %s\n",filespec); return -3; }

  if (!nodata) {
    bl=(((nx*sizeof(short)-1)/rl)+1)*rl;
    //printf("Block length=%d x %d\n",bl,ny);
    for (j=0; j<ny; j++) {
      if ((i=fread(&cdata[j*nx*nz*2],bl,nz,in))!=nz) {
	sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	error(s,ERR_WARN);
      }
      //	printf("Line %d is %d bytes\n",j,(((nx*sizeof(short)-1)/rl)+1)*rl);
    }

    // This reverses odd numbered rows in RASTER scan mode
    if (sm) {
      //	printf("Flipping data\n");
      for (i=1; i<ny; i+=2) {
	for (j=0; j<nx/2; j++) {
	  sw=sdata[i*nx+j];
	  sdata[i*nx+j]=sdata[i*nx+nx-j-1];
	  sdata[i*nx+nx-j-1]=sw;
	}
      }
    }

    // Make the data float
    for (i=nx*ny*nz-1; i>=0; i--)  
      rdata[i]=((float)((cdata[i*2+1]*256.0)+cdata[i*2])); 
  }

  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  update();
  //calcHist(Min(),Max());
  fclose(in);
  return 0;
}

int EMData::readPGM(const char *fsp,int nodata)		//portable greymap (and other pix formats)
{
  FILE *in;
  int i,j,l,pipe=0;
  float f,a,p,div=0;
  char s[MAXPATH],*m2,w;
  unsigned char *cdata,*cdata2,c;

#ifdef DEBUG
  printf("PGM open '%s'\n",fsp);
#endif

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  in=fopen(fsp,"r");
  if (in==NULL) { sprintf(s,"Cannot open file %s\n",fsp); zero(); return -1; }
  if (getc(in)!='P'||getc(in)!='5') {
    fclose(in);
    pipe=1;
    m2=(char *)(fsp+strlen(fsp)-4);
    s[0]=0;
    sprintf(s,"convert %s pgm:-",fsp);

    in=popen(s,"r");
    if (getc(in)!='P'||getc(in)!='5') {
      pclose(in);
      error("Sorry, I do not recognize this file type.",ERR_ERROR);
      zero();
      return -2;
    }
  }

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  nx=ny=0;
  nz=1;
  getc(in);
  while (fgets(s,100,in)!=NULL && s[0]=='#') ;
  sscanf(s," %d %d  %f",&nx,&ny,&div);
  if (div!=255.0 && div!=0) printf("Warning, this could be an invalid pgm (%1.0f).\n",div);
  while (getc(in)!=10);
  if (nx<=0||ny<=0) { error("Invalid file\n",ERR_ERROR); zero(); return -1; }

#ifdef DEBUG
  printf("Image is %dx%d\n",nx,ny);
#endif

  if (nodata) flags|=EMDATA_NODATA;
  setSize(nx,ny,1);

  if (!nodata) {
    cdata=(unsigned char *)rdata;

    for (j=0; j<ny; j++) {
      if ((i=fread(&cdata[(ny-j-1)*nx*nz],nx,nz,in))!=nz) {
	printf("Incomplete data read %d/%d blocks\n",i,ny*nz);
	zero();
	return -1;
      }
    }
    if (div<1.0 || div>=65536.0) div=255.0;
    for (i=nx*ny*nz-1; i>=0; i--) rdata[i]=(float)cdata[i]/div;

    //for (i=nx*ny*nz-1; i>=0; i--) if (rdata[i]>1.0||rdata[i]<0) printf("%d,%g\n",i,rdata[i]);
  }

  if (pipe) pclose(in); else fclose(in);
  pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  update();
  //calcHist(Min(),Max());
  return 0;
}

int EMData::readICOS(const char *fsp,int nodata)
{
  FILE *in;
  int i,j,l,pipe=0;
  int ord=0;	// 1 if the file order is MSB first
  int mord=0; // 1 if the machine order is MSB first
  float f,a,p;
  char s[800],*m2,w;
  unsigned char *cdata,c,t;
  icosH *icosh=new icosH;

#ifdef DEBUG
  printf("ICOS open '%s'\n",fsp);
#endif

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;

  if (fsp==NULL) { in=stdin; }
  else {
    if (strcmp(&fsp[strlen(fsp)-3],".gz")==0 || strcmp(&fsp[strlen(fsp)-2],".Z")==0) {
      sprintf(s,"zcat %s",fsp);
      in=popen(s,"r");
      pipe=1;
    }
    if (strcmp(&fsp[strlen(fsp)-3],".bz")==0 || strcmp(&fsp[strlen(fsp)-4],".bz2")==0) {
      sprintf(s,"bzcat %s",fsp);
      in=popen(s,"r");
      pipe=1;
    }
    else in=fopen(fsp,"r");
    if (in==NULL) {
      error("Cannot open file",ERR_ERROR);
      zero();
      return -1;
    }

    setPath(fsp);
  }

  if (!fread(icosh,sizeof(icosH),1,in)) { setSize(10,10,1); zero(); return -1; }

  // This swaps the byte order in the header on appropriate machines
#ifdef MSB_FIRST
  mord=1;
#else
  mord=0;
#endif

  m2=(char *)icosh;
  if (m2[84]==0 && m2[85]==0) ord=1;
  if (mord ^ ord) {
    for (i=84; i<84+5*4; i+=4) {
      w=m2[i]; m2[i]=m2[i+3]; m2[i+3]=w;
      w=m2[i+1]; m2[i+1]=m2[i+2]; m2[i+2]=w;
    }
  }

  if (icosh->ny>0xffff || icosh->nz>0xffff ) {
    setSize(10,10,1); printf("Invalid ICOS file\n"); zero(); return -1;
  }

  //printf("ord=%d mord=%d\n",ord,mord);

  if (icosh->ny==1) { printf("This appears to be a 1d file. I only read 2d files.\n"); zero(); return -4; }

  nx=icosh->nx;
  ny=icosh->ny;
  nz=icosh->nz;
  delete icosh;

  setName(icosh->title);
  setSize(nx,ny,nz);

  //printf("in readICOS: (nx,ny,nz)=(%d,%d,%d)\tmin=%f\tmax=%f\n",nx,ny,nz,icosh->min,icosh->max);

  if (nodata) flags|=EMDATA_NODATA;

  if (!nodata) {

    portable_fseek(in,sizeof(icosH),SEEK_SET);
	
    getData();
    cdata=(unsigned char *)rdata;
    flags&=~EMDATA_COMPLEX;
    for (j=0; j<ny*nz; j++) {
      portable_fseek(in,4,SEEK_CUR);
      if ((i=fread(&rdata[j*nx],nx*sizeof(float),1,in))!=1) {
	sprintf(s,"Incomplete data read %d/%d blocks\n",j,ny*nz);
	error(s,ERR_WARN);
      }
      portable_fseek(in,4,SEEK_CUR);
    }
    // swap order if machine and file have different order
    if (ord+mord==1) {
      for (i=nx*ny*nz*4-4; i>=0; i-=4) {
	c=cdata[i]; cdata[i]=cdata[i+3]; cdata[i+3]=c;
	c=cdata[i+1]; cdata[i+1]=cdata[i+2]; cdata[i+2]=c;
      }
    }
  }
  if(fsp!=NULL){
    if (pipe) pclose(in); else fclose(in);
  }

  pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  doneData();
  update();
  return 0;
}
 
int EMData::readDM3(const char* fsp, int nodata)
{
  flags = EMDATA_NEEDUPD | EMDATA_NEEDHIST;
    
  if (strrchr(fsp,'/') == NULL)
    setName(fsp);
  else
    setName(strrchr(fsp,'/') + 1);

  setPath(fsp);

  Gatan::DM3Reader dm3_file(fsp);

  int err = 0;
    
  if ((err = dm3_file.init()) == 0) {
    err = dm3_file.read();
  }

  if (err) {
    setSize(10, 10, 1);
    zero();
    return -1;
  }

  if (nodata) {
    flags |= EMDATA_NODATA;
  }

  setSize(dm3_file.get_nx(), dm3_file.get_ny(), 1);

  sprintf(name, "\nExp#=%d,C=%d,F='%s',Bin=%d,Z=%4.2f,ExpT=%4.2f,AB=%d,M=%5.0f",
	  dm3_file.get_exposure_num(), dm3_file.get_camera_x(), 
	  dm3_file.get_frame_type(), dm3_file.get_binning_x(), dm3_file.get_zoom_factor(), 
	  dm3_file.get_exposure_time(), dm3_file.get_anti_blooming(),
	  dm3_file.get_magnification());
    
  if (!nodata) {
    getData();
    flags &= ~EMDATA_COMPLEX;
    dm3_file.read_data(&rdata);
  }
    
  pixel = 4.66667;
  parent = NULL;
  pathnum = 0;
  dx = dy = dz = 0;
  setRAlign(0, 0, 0);
  atime = time(0);
  doneData();
  update();    
    
  return 0;
}
    
int EMData::writePIF(const char* filename) {
  FILE *out;

  if (!filename) {
    out = stdout;
  }
  else {
    out = fopen(filename, "wb");
    if (!out) {
      error("Could not write PIF", ERR_ERROR);
      return -1;
    }
    file_lock_wait(out);
  }

  setPath(filename);
  pathnum = 0;
    
  time_t t0 = time(0);
  struct tm* t = localtime(&t0);
    
  PifFileHeader pfh;
  memset(&pfh, 0, sizeof(PifFileHeader));
  pfh.magic[0] = PIF_MAGIC_NUM;
  pfh.magic[1] = PIF_MAGIC_NUM;
  sprintf(pfh.scalefactor, "1.0");
  pfh.nimg = 1;
  sprintf(pfh.program, "EMAN %d/%02d/%02d", t->tm_mon,t->tm_mday,t->tm_year);
  pfh.htype = 1;
  pfh.nx = nx;
  pfh.ny = ny;
  pfh.nz = nz;
  pfh.mode = PIF_FLOAT_INT;
    

  PifImageHeader pih;
  memset(&pih, 0, sizeof(PifImageHeader));
    
  pih.nx = nx;
  pih.ny = ny;
  pih.nz = nz;
  pih.mode = PIF_FLOAT;
  pih.xlen = pixel;
  pih.ylen = pixel;
  pih.zlen = pixel;
  pih.alpha = 90;
  pih.beta = 90;
  pih.gamma = 90;
  pih.mapc=1;
  pih.mapr=2;
  pih.maps=3;
  pih.min = Min();
  pih.max = Max();
  pih.mean = Mean();
  pih.sigma = Sigma();
    
  sprintf(pih.time, "%d/%02d/%02d %d:%02d",
	  t->tm_mon,t->tm_mday,t->tm_year,t->tm_hour,t->tm_min);
    
  fwrite(&pfh, sizeof(PifFileHeader), 1, out);
  fwrite(&pih, sizeof(PifImageHeader), 1, out);
    
  int* buf = new int[nx];
#ifdef DEBUG
  double min = 1 < 16;
  double max = -min;
  double sum = 0;
#endif    
  for (int i = 0; i < nz; i++) {
    for (int j = 0; j < ny; j++) {
      for (int k = 0; k < nx; k++) {
#ifdef DEBUG
	float f = rdata[i*nx*ny + j*nx + k];

	if (f > max) {
	  max = f;
	}
	if (f < min) {
	  min = f;
	}
	sum += f;
#endif
	buf[k] = (int) rdata[i*nx*ny + j*nx + k];		
      }
	    
      fwrite(buf, sizeof(int) * nx, 1, out);
    }
  }
#ifdef DEBUG
  printf("writePif: min = %f, max = %f, mean = %f\n", min, max, sum/(nx*ny*nz));
#endif
  delete [] buf;
  buf = 0;
  fclose(out);
}
    
int EMData::readVTK(const char* fsp, int nodata) {
  flags = EMDATA_NEEDUPD | EMDATA_NEEDHIST;
    
  if (strrchr(fsp,'/') == NULL)
    setName(fsp);
  else
    setName(strrchr(fsp,'/') + 1);

  setPath(fsp);

  EMVtk vtkfile(fsp);
  int bad_vtk = vtkfile.init();
  if (bad_vtk) {
    fprintf(stderr, "Error: not a VTK file: '%s'\n", fsp);
    setSize(10, 10, 1);
    zero();
    return -1;
  }

  if (nodata) {
    flags |= EMDATA_NODATA;
  }

  setSize(vtkfile.get_nx(), vtkfile.get_ny(), vtkfile.get_nz());
    
  if (!nodata) {
    getData();
    flags &= ~EMDATA_COMPLEX;
    if (vtkfile.read_data(&rdata) != 0) {
      fprintf(stderr, "Error in read VTK file: '%s'\n", fsp);
      doneData();
      return -1;
    }
    doneData();
  }
    
  pixel = 4.66667;
  parent = NULL;
  pathnum = 0;
  dx = dy = dz = 0;
  setRAlign(0, 0, 0);
  atime = time(0);
  update();    
  return 0;
}

int EMData::readPIF(const char *fsp, int which_img, int nodata) {
#ifdef DEBUG
  if (which_img < 0) {
    printf("PIF open '%s': 3d image\n", fsp);
  }
  else {
    printf("PIF open '%s': %dth image\n", fsp, which_img);
  }
#endif
  flags = EMDATA_NEEDUPD | EMDATA_NEEDHIST;
    
  if (strrchr(fsp,'/') == NULL)
    setName(fsp);
  else
    setName(strrchr(fsp,'/') + 1);

  setPath(fsp);

  EMPif pif_file(fsp);
  int bad_pif = 0;
    
  if (!pif_file.is_valid_header()) {
    printf("Error: not a PIF file: '%s'\n", fsp);
    bad_pif = 1;
  }

  if (!pif_file.is_valid_mode()) {
    printf("Error: invalid PIF mode: '%d'\n", pif_file.get_mode());
    bad_pif = 1;
  }

  if (bad_pif) {
    setSize(10, 10, 1);
    zero();
    return -1;
  }
    
  if (!pif_file.get_ny() ==  1) {
    printf("This appears to be a 1d file. I only read 2d files.\n");
    zero();
    return -4;
  }
    
#ifdef DEBUG
  pif_file.dump();
#endif
    
  if (nodata) {
    flags |= EMDATA_NODATA;
  }
    
  setSize(pif_file.get_nx(), pif_file.get_ny(), pif_file.get_nz());
   	
  if (!nodata) {
    getData();

    if (pif_file.is_complex_mode()) {
      flags|=EMDATA_COMPLEX|EMDATA_RI;
    }
    else {
      flags &= ~EMDATA_COMPLEX;
    }

    bool is_3d = false;
    if (which_img < 0) {
      is_3d = true;
      which_img = 0;
    }
	
    if (pif_file.read_data(which_img, &rdata, is_3d) != 0) {
      printf("Error in read PIF %dth image\n", which_img);
      return -1;
    }
	
  }
    
  pixel = 4.66667;
  parent = NULL;
  pathnum = 0;
  dx = dy = dz = 0;
  setRAlign(0, 0, 0);
  atime = time(0);
  doneData();
  update();    
    
  return 0;
}

int EMData::readTIFF(const char *fsp,int nodata){

  //TIFF read using libtiff. Currently, handle 8 or 16-bits gray scale images
  //This version gives correct image orientation
  //htet khant
 
  TIFF *in;
  int stripSize, stripMax, stripCount,i,j,k;
  uint16 photo;//,bitspersample;
  uint32 width,height;
  unsigned long bufferSize,imageOffset,result,count;
  unsigned char *cdata;
  float t;

#ifdef DEBUG
  printf("TIFF open '%s'\n",fsp);
#endif

  flags = EMDATA_NEEDUPD | EMDATA_NEEDHIST;
  if (strrchr(fsp,'/') == NULL)
    setName(fsp);
  else
    setName(strrchr(fsp,'/') + 1);
  setPath(fsp);
  nx=ny=0;
  bitspersample=0;
  nz=1;

  //open TIFF image
  if((in=TIFFOpen(fsp,"r"))==NULL){
    fprintf(stderr,"Error: Could not open the image !!!\n");return(-1);}
  //check bits per sample
  if(TIFFGetField(in,TIFFTAG_BITSPERSAMPLE,&bitspersample)==0){
    fprintf(stderr,"Error: Undefined bits per sample !!!\n");return (-1);}
  //image width
  if(TIFFGetField(in,TIFFTAG_IMAGEWIDTH,&width)==0){
    fprintf(stderr,"Error: Undefined image width !!!\n");return(-1);}    
  //image height
  if(TIFFGetField(in,TIFFTAG_IMAGELENGTH,&height)==0){
    fprintf(stderr,"Error: Undefined image height !!!\n");return(-1);}
  //getting strip size and possible multiple strips
  stripSize=TIFFStripSize(in);
  stripMax=TIFFNumberOfStrips(in);
  imageOffset=0;
  //allocating buffer size
  bufferSize= stripMax*stripSize;
  if((cdata=(unsigned char*)_TIFFmalloc(bufferSize))==NULL){
    fprintf(stderr,"Error: Could not allocate enough memory\n");
    return(-1);
  }
  //reading image as multiple strips
  //(***) CAUTION: the last strips may have empty values
  for (stripCount=0;stripCount<stripMax;stripCount++){
    if((result=TIFFReadEncodedStrip(in,stripCount,cdata+imageOffset,stripSize))==-1){
      fprintf(stderr,"Error reading stripped image\n");return(-1);}
    imageOffset+=result;
  }
  //check photometric interpretations
  //if(TIFFGetField(in,TIFFTAG_PHOTOMETRIC, &photo)==0){
  //fprintf(stderr,"Error:Invalid photometric interpretation\n");
  //return(-1);
  //}
  //    if(photo!=PHOTOMETRIC_MINISWHITE){
  //printf("Correcting photometric interpretation\n");
  //for(count=0;count<bufferSize;count++)
  //    cdata[count]=~cdata[count];
  //}
  //Just sending some of the tag info to screen
  //cout<<"image width :"<<width<<"    image height: "<<height<<endl;
  //cout<<" stripSize :"<<stripSize<<"  stripMax :"<<stripMax<<endl;
  //cout<<"Orientation "<<orientation<<endl;

  nx=width;
  ny=height;

#ifdef DEBUG
  printf("Image is %dx%d\n",nx,ny);
#endif

  if (nodata) {
    flags |= EMDATA_NODATA;
  }
  setSize(nx, ny, nz);
  
  if (!nodata) {
    getData();
    //convert 8 or 16-bits TIFF images to float and  correct image orientation 
    for (i=1; i<ny/2; i++) {
      for (j=0; j<nx; j++) {
	if(bitspersample==8){rdata[i*nx+j]=(float) ((unsigned char*)cdata)[i*nx+j];}
	else if(bitspersample==16){rdata[i*nx+j] = (float)((unsigned short*)cdata)[i*nx+j];}
	else { fprintf(stderr,"BAILING OUT:Allow only 8- or 16-bits image\n");doneData();return(-1);}
	t=rdata[i*nx+j];
	rdata[i*nx+j]=rdata[(ny-i)*nx+j];
	rdata[(ny-i)*nx+j]=t;
      }
    }

    for (i=1; i<ny; i++) {
      for (j=0; j<nx; j++) {
	rdata[i*nx+j] = (float)((unsigned short*)cdata)[i*nx+j];
	t=rdata[i*nx+j];
	rdata[i*nx+j]=rdata[(ny-i)*nx+j];
	rdata[(ny-i)*nx+j]=t;
      }
    }

  }

  //cdata=0;
  pixel = 4.66667;
  parent = NULL;
  pathnum = 0;
  dx = dy = dz = 0;
  setRAlign(0, 0, 0);
  atime = time(0);
  doneData();
  update();
  TIFFClose(in);
  return 0;
}


int EMData::readMRC(const char *fsp,int nodata, int n) {
  FILE *in;
  int i,j,l,pipe=0;
  int ord=0;	// 1 if the file order is MSB first
  int mord=0; // 1 if the machine order is MSB first
  float f,a,p;
  char s[800],*m2,w;
  unsigned char *cdata,*cdata2,c,t;
  short *sdata;
  static char *str[] = { "char", "short", "float", "short complex", "float complex" };
  static char *str2[] = { "" , "byte reversed" , "" };
  static char *str3[] = { "" , "compressed" };

#ifdef DEBUG
  printf("MRC open '%s'\n",fsp);
#endif

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  if (!mrch) mrch=(mrcH *)Smalloc(sizeof(mrcH),1);

  if (fsp==NULL) { in=stdin; }
  else {
    if (strcmp(&fsp[strlen(fsp)-3],".gz")==0 || strcmp(&fsp[strlen(fsp)-2],".Z")==0) {
      sprintf(s,"zcat %s",fsp);
      in=popen(s,"rb");
      pipe=1;
    }
    if (strcmp(&fsp[strlen(fsp)-3],".bz")==0 || strcmp(&fsp[strlen(fsp)-4],".bz2")==0) {
      sprintf(s,"bzcat %s",fsp);
      in=popen(s,"rb");
      pipe=1;
    }
    else in=fopen(fsp,"r");
    if (in==NULL) {
      error("Cannot open file",ERR_ERROR);
      zero();
      return -1;
    }

    //if (strrchr(fsp,'/')==NULL) setName(fsp);
    //else setName(strrchr(fsp,'/')+1);
    setPath(fsp);
  }

  if (!fread(mrch,sizeof(mrcH),1,in)) { setSize(10,10,1); zero(); return -1; }

  mrch->labels[0][79]=0;
  setName(mrch->labels[0]);
  if (name[0]=='!' && name[1]=='-') {
    i=sscanf(name+2," %f %f %f %f %f %f %f %f %f %f %f",
	     ctf,ctf+1,ctf+2,ctf+3,ctf+4,ctf+5,ctf+6,ctf+7,ctf+8,ctf+9,ctf+10); 
    if (i==11) flags|=EMDATA_HASCTF;
    else printf("Incomplete CTF info: %s\n",name);
  }
  if (name[0]=='!' && name[1]=='$') {
    sscanf(name+2," %f,%f",ctf,ctf+10);
  }

  // This swaps the byte order in the header on appropriate machines
#ifdef MSB_FIRST
  mord=1;
#else
  mord=0;
#endif

  m2=(char *)mrch;
  if (m2[0]==0 && m2[1]==0) ord=1;
  if (mord ^ ord) {
    for (i=0; i<(27+MRC_USER)*4; i+=4) {
      w=m2[i]; m2[i]=m2[i+3]; m2[i+3]=w;
      w=m2[i+1]; m2[i+1]=m2[i+2]; m2[i+2]=w;
    }
  }

  if (mrch->ny>0xffff || mrch->nz>0xffff || mrch->mode>4 || mrch->nlabl>11) {
    setSize(10,10,1); printf("Invalid MRC file\n"); zero(); return -1;
  }

  //printf("ord=%d mord=%d\n",ord,mord);

  if (mrch->ny==1) { printf("This appears to be a 1d file. I only read 2d files.\n"); zero(); return -4; }
  //if (mrch->nz!=1) { printf("This appears to be a 3d file. I only read 2d files.\n"); return 0; }

  int startz;
  if (mrch->mode==MODE_float_COMPLEX || mrch->mode==MODE_short_COMPLEX) mrch->nx*=2;
  nx=mrch->nx;
  ny=mrch->ny;
  if (n>=0 && n< mrch->nz) {
    nz=1;
    startz=n;
  }
  else {
    if(n!=-1) printf("WARNING from readMRC: section n=%d is out of valid range [0,%d] and thus ignored, whole map will be read in\n",n,mrch->nz-1); 	
    nz=mrch->nz;
    startz=0;
  }

  if (mrch->nz==1) sprintf(s,"Read a %s %s %dx%d 2D %s MRC File\n",
			   str2[ord+mord],str3[pipe],nx,ny,str[mrch->mode]);
  else sprintf(s,"Read a %dx%dx%d 3D MRC File\n",nx,ny,nz);

  if (mrch->nxstart!=0||mrch->nystart!=0||mrch->nzstart!=0) printf("WARNING: nx/y/z start not zero\n");

  //if (header.mapc!=1 || header.mapr!=2 || header.maps!=3) printf("WARNING: map not 1,2,3\n");


  if (mrch->xlen==0) mrch->xlen=1.0;
  if (mrch->ylen==0) mrch->ylen=1.0;
  if (mrch->zlen==0) mrch->zlen=1.0;

  if (nodata) flags|=EMDATA_NODATA;

  // This reads 2d&3d arrays
  //if (header.nz==1) {
  setSize(nx,ny,nz);

  if (!nodata) {

    portable_fseek(in,sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
	
    getData();
    cdata=(unsigned char *)rdata;
    sdata=(short *)rdata;

    switch(mrch->mode) {
    case MODE_char:
      portable_fseek(in,startz*nx*ny,SEEK_CUR);
      flags&=~EMDATA_COMPLEX;
      for (j=0; j<ny; j++) {
	if (updProgress && updProgress(j,ny,"Reading MRC file (8 bit)")) break;
	if ((i=fread(&cdata[j*nx*nz],nx,nz,in))!=nz) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
      for (i=nx*ny*nz-1; i>=0; i--) rdata[i]=(float)cdata[i]/100.0-1.28;
      if (updProgress) updProgress(0,0,NULL);
      break;
    case MODE_short:
      portable_fseek(in,startz*nx*ny*sizeof(short),SEEK_CUR);
      flags&=~EMDATA_COMPLEX;
#ifdef DEBUG
      printf("Short file read\n");
#endif	
      for (j=0; j<ny; j++) {
	if (updProgress && updProgress(j,ny,"Reading MRC file (16 bit)")) break;
	if ((i=fread(&cdata[j*nx*nz*sizeof(short)],nx*sizeof(short),nz,in))!=nz) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	  error(s,ERR_WARN);
	  //printf ("ferror is %d", ferror(in));
	  //printf ("feof is %d", feof(in));
	}
			
		
      }
      if ((ord+mord)&1) {
	for (i=nx*ny*nz-1; i>=0; i--) {
	  t=cdata[i*2];
	  cdata[i*2]=cdata[i*2+1];
	  cdata[i*2+1]=t;
	  rdata[i]=(float)(sdata[i]);
	} 
      }
      else {
	for (i=nx*ny*nz-1; i>=0; i--)  {
	  rdata[i]=(float)(sdata[i]); 
	}
      }
      if (updProgress) updProgress(0,0,NULL);

#ifdef DEBUG
      printf("Short read complete\n");
#endif
      break;
    case MODE_float:
      portable_fseek(in,startz*nx*ny*sizeof(float),SEEK_CUR);
      cdata=(unsigned char *)rdata;
      flags&=~EMDATA_COMPLEX;
      for (j=0; j<ny; j++) {
	if (updProgress && updProgress(j,ny,"Reading MRC file (float)")) break;
	if ((i=fread(&rdata[j*nx*nz],nx*sizeof(float),nz,in))!=nz) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
      // swap order if machine and file have different order
      if (ord+mord==1) {
	for (i=nx*ny*nz*4-4; i>=0; i-=4) {
	  c=cdata[i]; cdata[i]=cdata[i+3]; cdata[i+3]=c;
	  c=cdata[i+1]; cdata[i+1]=cdata[i+2]; cdata[i+2]=c;
	}
      }
      if (updProgress) updProgress(0,0,NULL);
      break;
    case MODE_short_COMPLEX:
      portable_fseek(in,startz*nx*ny*sizeof(short),SEEK_CUR);
      cdata=(unsigned char *)rdata;
      flags|=EMDATA_COMPLEX|EMDATA_RI;
      for (j=0; j<ny; j++) {
	if (updProgress && updProgress(j,ny,"Reading MRC file (16 bit complex)")) break;
	if ((i=fread(&cdata[j*nx*nz*2],nx*sizeof(short),nz,in))!=nz) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
      if (ord) {
	for (i=nx*ny*nz-1; i>=0; i--) {
	  rdata[i]=(float)((cdata[i*2]<<8)+cdata[i*2+1]); 
	}
      }
      else {
	for (i=nx*ny*nz-1; i>=0; i--) {
	  rdata[i]=(float)((cdata[i*2+1]<<8)+cdata[i*2]); 
	}
      }
      if (updProgress) updProgress(0,0,NULL);
      break;
    case MODE_float_COMPLEX:
      portable_fseek(in,startz*nx*ny*sizeof(float),SEEK_CUR);
      cdata=(unsigned char *)rdata;
      flags|=EMDATA_COMPLEX|EMDATA_RI;
      clearerr(in);
      for (j=0; j<ny; j++) {
	if (updProgress && updProgress(j,ny,"Reading MRC file (float complex)")) break;
	if ((i=fread(&rdata[j*nx*nz],nx*sizeof(float),nz,in))!=nz) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	  error(s,ERR_ERROR);
	}
      }
      if (ord+mord==1) {
	for (i=nx*ny*nz*4-4; i>=0; i-=4) {
	  c=cdata[i]; cdata[i]=cdata[i+3]; cdata[i+3]=c;
	  c=cdata[i+1]; cdata[i+1]=cdata[i+2]; cdata[i+2]=c;
	}
      }
      if (updProgress) updProgress(0,0,NULL);
      if (ferror(in)) { zero(); return -1; }
      break;
    default: 
      printf("Bad MRC file\n"); 
      zero(); 
      return -1;
    }
  }

  update();
  if(fsp!=NULL){
    if (pipe) pclose(in); else fclose(in);
  }

  //setup pixel if possible
  if (mrch->xlen > 10.0) pixel=mrch->xlen/nx;
  else pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  doneData();
  update();
  //calcHist(Min(),Max());
  // this is to filp the phase for conforming to MRC convention
  if (isComplex()) {
    ap2ri();
    getData();
    for(i=0;i<nx*ny*nz;i+=2)rdata[i+1]*=-1;
    doneData();
  }
  return 0;
}

// NOTE: This works ONLY for real 2d images. Also the area MUST reside within the image
int EMData::readMRCArea(const char *fsp,int x0,int y0,int w,int h)
{
  FILE *in;
  int i,j,l,pipe=0;
  int ord=0;	// 1 if the file order is MSB first
  int mord=0; // 1 if the machine order is MSB first
  float f,a,p;
  char s[800],*m2,ww;
  unsigned char *cdata,*cdata2,c,t;
  short *sdata;
  static char *str[] = { "char", "short", "float", "short complex", "float complex" };
  static char *str2[] = { "" , "byte reversed" , "" };
  static char *str3[] = { "" , "compressed" };

#ifdef DEBUG
  printf("MRC open '%s'\n",fsp);
#endif

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  if (!mrch) mrch=(mrcH *)Smalloc(sizeof(mrcH),1);

  if (strcmp(&fsp[strlen(fsp)-3],".gz")==0 || strcmp(&fsp[strlen(fsp)-2],".Z")==0) {
    sprintf(s,"zcat %s",fsp);
    in=popen(s,"r");
    pipe=1;
  }
  else in=fopen(fsp,"r");
  if (in==NULL) {
    error("Cannot open file (area)",ERR_ERROR);
    zero();
    return -1;
  }

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  if (!fread(mrch,sizeof(mrcH),1,in)) { zero(); return -1; }

  // This swaps the byte order in the header on appropriate machines
#ifdef MSB_FIRST
  mord=1;
#else
  mord=0;
#endif

  m2=(char *)mrch;
  if (m2[0]==0 && m2[1]==0) ord=1;
  if (mord ^ ord) {
    for (i=0; i<(27+MRC_USER)*4; i+=4) {
      ww=m2[i]; m2[i]=m2[i+3]; m2[i+3]=ww;
      ww=m2[i+1]; m2[i+1]=m2[i+2]; m2[i+2]=ww;
    }
  }

  //printf("ord=%d mord=%d\n",ord,mord);

  if (mrch->ny==1) { printf("This appears to be a 1d file. I only read 2d files.\n"); zero(); return 0; }
  //if (mrch->nz!=1) { printf("This appears to be a 3d file. I only read 2d files.\n"); return 0; }

  if (mrch->mode==MODE_float_COMPLEX || mrch->mode==MODE_short_COMPLEX) {
    printf("ERROR: cannot read a region of a complex image\n");
    zero();
    return -1; 
  }
  if (mrch->nz!=1) { printf("ERROR: cannot read a region from volume data\n"); zero(); return -1; }
  if (x0>mrch->nx || y0>mrch->ny || x0+w<0 || y0+h<0) {
    if (pipe) pclose(in); else fclose(in);
    setSize(w,h,1);
    zero();
    error(ERR_WARN,"Error: MRC box read outside image area");
    return 0;
  }
  // Not the most efficient way, but functional
  if (x0<0 || x0+w>mrch->nx || y0<0 ||y0+h>mrch->ny) { 
    /*	printf("ERROR: MRC box size extends beyond edge of image\n");
	zero();
	return -1;*/
    if (pipe) pclose(in); else fclose(in);
    int x0a=x0,y0a=y0,wa=w,ha=h;
    if (x0a<0) { x0a=0; wa+=x0; }
    if (y0a<0) { y0a=0; ha+=y0; }
    if (x0a+wa>=mrch->nx) wa=mrch->nx-x0a;
    if (y0a+ha>=mrch->ny) ha=mrch->ny-y0a;
    EMData tmp;
    tmp.readMRCArea(fsp,x0a,y0a,wa,ha);
    setSize(w,h,1);
    zero();
    int x0i=0,y0i=0;
    if (x0<0) x0i=-x0;
    if (y0<0) y0i=-y0;
    insertClip(&tmp,x0i,y0i);
    return 0;
  } 

  nx=w;
  ny=h;
  nz=1;

  if (mrch->nz==1) sprintf(s,"Read a %s %s %dx%d 2D %s MRC File\n",
			   str2[ord+mord],str3[pipe],nx,ny,str[mrch->mode]);
  else sprintf(s,"Read a %dx%dx%d 3D MRC File\n",nx,ny,nz);

  if (mrch->nxstart!=0||mrch->nystart!=0||mrch->nzstart!=0) printf("WARNING: nx/y/z start not zero\n");

  //if (header.mapc!=1 || header.mapr!=2 || header.maps!=3) printf("WARNING: map not 1,2,3\n");


  if (mrch->xlen==0) mrch->xlen=1.0;
  if (mrch->ylen==0) mrch->ylen=1.0;
  if (mrch->zlen==0) mrch->zlen=1.0;

  // This reads 2d&3d arrays
  //if (header.nz==1) {
  setSize(nx,ny,nz);

  getData();
  cdata=(unsigned char *)rdata;
  sdata=(short *)rdata;

  switch(mrch->mode) {
  case MODE_char:
    flags&=~EMDATA_COMPLEX;
    for (j=0; j<ny; j++) {
      portable_fseek(in,(j+y0)*mrch->nx+x0+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
      if ((i=fread(&cdata[j*nx*nz],nx,nz,in))!=nz) {
	sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	error(s,ERR_WARN);
      }
    }
    for (i=nx*ny*nz-1; i>=0; i--) rdata[i]=(float)cdata[i]/100.0;
    break;
  case MODE_short:
    flags&=~EMDATA_COMPLEX;
#ifdef DEBUG
    printf("Short file read\n");
#endif	
    for (j=0; j<ny; j++) {
      portable_fseek(in,((j+y0)*mrch->nx+x0)*sizeof(short)+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
      if ((i=fread(&cdata[j*nx*nz*sizeof(short)],nx*sizeof(short),nz,in))!=nz) {
	sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	error(s,ERR_WARN);
      }
    }
    if ((ord+mord)&1) {
      for (i=nx*ny*nz-1; i>=0; i--) {
	t=cdata[i*2];
	cdata[i*2]=cdata[i*2+1];
	cdata[i*2+1]=t;
	rdata[i]=(float)(sdata[i]);
      } 
    }
    else {
      for (i=nx*ny*nz-1; i>=0; i--)  {
	rdata[i]=(float)(sdata[i]); 
      }
    }

#ifdef DEBUG
    printf("Short read complete\n");
#endif
    break;
  case MODE_float:
    cdata=(unsigned char *)rdata;
    flags&=~EMDATA_COMPLEX;
    for (j=0; j<ny; j++) {
      portable_fseek(in,((j+y0)*mrch->nx+x0)*sizeof(float)+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
      if ((i=fread(&rdata[j*nx*nz],nx*sizeof(float),nz,in))!=nz) {
	sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
	error(s,ERR_WARN);
      }
    }
    // swap order if machine and file have different order
    if (ord+mord==1) {
      for (i=nx*ny*nz*4-4; i>=0; i-=4) {
	c=cdata[i]; cdata[i]=cdata[i+3]; cdata[i+3]=c;
	c=cdata[i+1]; cdata[i+1]=cdata[i+2]; cdata[i+2]=c;
      }
    }
    break;
  default: printf("Bad MRC file\n"); zero(); return -1;
  }

  mrch->nx=nx;
  mrch->ny=ny;
  mrch->nz=nz;

  update();
  if (pipe) pclose(in); else fclose(in);

  pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  doneData();
  update();
  //calcHist(Min(),Max());
  return 0;
}

// NOTE: This works ONLY for real 3d images. Also the area MUST reside within the image
int EMData::readMRCArea(const char *fsp,int x0,int y0,int z0,int xl,int yl,int zl)
{
  FILE *in;
  int i,j,k,l,pipe=0;
  int ord=0;	// 1 if the file order is MSB first
  int mord=0; // 1 if the machine order is MSB first
  float f,a,p;
  char s[800],*m2,ww;
  unsigned char *cdata,*cdata2,c,t;
  short *sdata;
  static char *str[] = { "char", "short", "float", "short complex", "float complex" };
  static char *str2[] = { "" , "byte reversed" , "" };
  static char *str3[] = { "" , "compressed" };

#ifdef DEBUG
  printf("MRC open '%s'\n",fsp);
#endif

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  if (!mrch) mrch=(mrcH *)Smalloc(sizeof(mrcH),1);

  if (strcmp(&fsp[strlen(fsp)-3],".gz")==0 || strcmp(&fsp[strlen(fsp)-2],".Z")==0) {
    sprintf(s,"zcat %s",fsp);
    in=popen(s,"r");
    pipe=1;
  }
  else in=fopen(fsp,"r");
  if (in==NULL) {
    error("Cannot open file (area)",ERR_ERROR);
    zero();
    return -1;
  }

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  if (!fread(mrch,sizeof(mrcH),1,in)) { zero(); return -1; }

  // This swaps the byte order in the header on appropriate machines
#ifdef MSB_FIRST
  mord=1;
#else
  mord=0;
#endif

  m2=(char *)mrch;
  if (m2[0]==0 && m2[1]==0) ord=1;
  if (mord ^ ord) {
    for (i=0; i<(27+MRC_USER)*4; i+=4) {
      ww=m2[i]; m2[i]=m2[i+3]; m2[i+3]=ww;
      ww=m2[i+1]; m2[i+1]=m2[i+2]; m2[i+2]=ww;
    }
  }

  //printf("ord=%d mord=%d\n",ord,mord);

  if (mrch->ny==1) { printf("This appears to be a 1d file. I only read 2d files.\n"); zero(); return 0; }
  //if (mrch->nz==1) { printf("This appears to be a 2d file. I only read 3d files.\n"); return 0; }

  if (mrch->mode==MODE_float_COMPLEX || mrch->mode==MODE_short_COMPLEX) {
    printf("ERROR: cannot read a region of a complex image\n");
    zero();
    return -1; 
  }
  if (mrch->nz==1) { printf("ERROR: cannot read a chunk from 2D data\n"); zero(); return -1; }
  if (x0>=mrch->nx || y0>=mrch->ny || z0>=mrch->nz || x0<0 || y0<0 || z0<0 || x0+xl<0 || y0+yl<0 || z0+zl<0 || x0+xl>mrch->nx || y0+yl>mrch->ny || z0+zl>mrch->nz) {
    if (pipe) pclose(in); else fclose(in);
    setSize(10,10,1);
    zero();
    error(ERR_WARN,"Error: MRC box read outside image area");
    return 0;
  }

  nx=xl;
  ny=yl;
  nz=zl;

  if (mrch->nz==1) sprintf(s,"Read a %s %s %dx%d 2D %s MRC File\n",
			   str2[ord+mord],str3[pipe],nx,ny,str[mrch->mode]);
  else sprintf(s,"Read a %dx%dx%d 3D MRC File\n",nx,ny,nz);

  if (mrch->nxstart!=0||mrch->nystart!=0||mrch->nzstart!=0) printf("WARNING: nx/y/z start not zero\n");

  //if (header.mapc!=1 || header.mapr!=2 || header.maps!=3) printf("WARNING: map not 1,2,3\n");


  if (mrch->xlen==0) mrch->xlen=1.0;
  if (mrch->ylen==0) mrch->ylen=1.0;
  if (mrch->zlen==0) mrch->zlen=1.0;

  // This reads 2d&3d arrays
  //if (header.nz==1) {
  setSize(nx,ny,nz);

  getData();
  cdata=(unsigned char *)rdata;
  sdata=(short *)rdata;

  switch(mrch->mode) {
  case MODE_char:
    flags&=~EMDATA_COMPLEX;
    for( k=0; k<nz; k++) {
      for (j=0; j<ny; j++) {
	portable_fseek(in,(k+z0)*mrch->nx*mrch->ny+(j+y0)*mrch->nx+x0+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
	if ((i=fread(&cdata[k*nx*ny+j*nx],nx,1,in))!=1) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",k*ny+i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
    }
    for (i=nx*ny*nz-1; i>=0; i--) rdata[i]=(float)cdata[i]/100.0;
    break;
  case MODE_short:
    flags&=~EMDATA_COMPLEX;
#ifdef DEBUG
    printf("Short file read\n");
#endif	
    for( k=0; k<nz; k++) {
      for (j=0; j<ny; j++) {
	portable_fseek(in,((k+z0)*mrch->nx*mrch->ny+(j+y0)*mrch->nx+x0)*sizeof(short)+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
	if ((i=fread(&cdata[(k*nx*ny+j*nx)*sizeof(short)],nx*sizeof(short),1,in))!=1) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",k*ny+i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
    }
    if ((ord+mord)&1) {
      for (i=nx*ny*nz-1; i>=0; i--) {
	t=cdata[i*2];
	cdata[i*2]=cdata[i*2+1];
	cdata[i*2+1]=t;
	rdata[i]=(float)(sdata[i]);
      } 
    }
    else {
      for (i=nx*ny*nz-1; i>=0; i--)  {
	rdata[i]=(float)(sdata[i]); 
      }
    }

#ifdef DEBUG
    printf("Short read complete\n");
#endif
    break;
  case MODE_float:
    cdata=(unsigned char *)rdata;
    flags&=~EMDATA_COMPLEX;
    for( k=0; k<nz; k++) {
      for (j=0; j<ny; j++) {
	portable_fseek(in,((k+z0)*mrch->nx*mrch->ny+(j+y0)*mrch->nx+x0)*sizeof(float)+sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
	if ((i=fread(&rdata[k*nx*ny+j*nx],nx*sizeof(float),1,in))!=1) {
	  sprintf(s,"Incomplete data read %d/%d blocks\n",k*ny+i,ny*nz);
	  error(s,ERR_WARN);
	}
      }
    }
    // swap order if machine and file have different order
    if (ord+mord==1) {
      for (i=nx*ny*nz*4-4; i>=0; i-=4) {
	c=cdata[i]; cdata[i]=cdata[i+3]; cdata[i+3]=c;
	c=cdata[i+1]; cdata[i+1]=cdata[i+2]; cdata[i+2]=c;
      }
    }
    break;
  default: printf("Bad MRC file\n"); zero(); return -1;
  }

  mrch->nx=nx;
  mrch->ny=ny;
  mrch->nz=nz;

  update();
  if (pipe) pclose(in); else fclose(in);

  pixel=1;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  doneData();
  update();
  //calcHist(Min(),Max());
  return 0;
}

// This is reverse engineered to find the data block. It will not work in general cases.
int EMData::readGatan3(const char *fsp, int nodata) {
  struct stat fs;
  if (stat(fsp,&fs)) { error(ERR_CRIT,"Unable to stat gatan3 file"); return 1; }

  FILE *in = fopen(fsp,"r");
  char *raw=(char *)malloc(fs.st_size+1);
  char sw;
  int swap=0;
  fread(raw,fs.st_size,1,in);

  char *ds=memmem(raw,fs.st_size,"Data%%%%",8);
  if (!ds) { error(ERR_CRIT,"No Data block in gatan3 file"); free(raw); return 2; }
  short *dp=(short *)(ds+24);
  char *dim = memmem(ds,fs.st_size-(ds-raw),"Dimensions",10);
  if (!dim) { error(ERR_CRIT,"No Dimensions in gatan3 file"); free(raw); return 2; }
  dim=memmem(dim,fs.st_size-(dim-raw),"%%%%",4);
  unsigned short *xs = (unsigned short *)(dim+12);
  dim=memmem(dim+4,fs.st_size-(dim-raw)-4,"%%%%",4);
  unsigned short *ys = (unsigned short *)(dim+12);

  // Try to guess if the order is swapped (not perfect)
  if ((*xs) * (*ys)*2>fs.st_size||(*xs) * (*ys)*2<fs.st_size/4) {
    *xs = (((*xs)&0xff)<<8)+((*xs)>>8);
    *ys = (((*ys)&0xff)<<8)+((*ys)>>8);
    swap=1;
    printf("Gatan DM3, I think the byte order is swapped (%d x %d)?\n",*xs,*ys);
    if ((*xs) * (*ys)*2>fs.st_size) { error(ERR_ERROR,"That size is bigger than the file!\n"); return -5; }
  }
  setSize(*xs,*ys,1);

  if (swap) {
    swapshort((char *)dp,nx*ny);
  }
  int i,j;
  float *d=getData();
  for (j=0; j<ny; j++) {
    for (i=0; i<nx; i++) {
      d[i+j*nx]=dp[i+(ny-j-1)*nx];
    }
  }
  doneData();

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  free(raw);
  return 0;
}

int EMData::readGatan(const char *fsp, int nodata) {
  FILE *in;
  gatanH hed;
  char swap=0;
  int i;

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  in=fopen(fsp,"r");
  if (!in) { error(ERR_ERROR,"Could not read Gatan file"); return -1; }
  fread(&hed,sizeof(gatanH),1,in);
  if (hed.len>4) { swap=1; swapshort((char *)&hed,7); }	// note this messes up a couple of elements
  if (hed.len>4 || hed.len<0 ||hed.type<0 ||hed.type>7) {
    error(ERR_ERROR,"Doesn't appear to be a Gatan file\n");
    fclose(in);
    return -1;
  }
  if (hed.type==3||hed.type==5) {
    error(ERR_ERROR,"Cannot read complex Gatan files\n");
    fclose(in);
    return -1;
  }

  setSize(hed.nx,hed.ny,1);
  getData();
  if (fread(rdata,hed.nx*hed.len,hed.ny,in)!=hed.ny) error(ERR_WARN,"Data read incomplete in Gatan file\n");
  fclose(in);

  switch(hed.type) {
    // short
  case 1:
    if (swap) swapshort((char *)rdata,hed.nx*hed.ny);
    {
      short *ds=(short *)rdata;
      for (i=hed.nx*hed.ny-1; i>=0; i--) rdata[i]=ds[i];
    }
    break;

    // float
  case 2:
    if (swap) swapint((char *)rdata,hed.nx*hed.ny);
    break;

    // byte
  case 6:
    {
      unsigned char *dc=(unsigned char *)rdata;
      for (i=hed.nx*hed.ny-1; i>=0; i--) rdata[i]=dc[i];
    }
    break;
	
    // long
  case 7:
    if (swap) swapint((char *)rdata,hed.nx*hed.ny);
    {
      long *dl=(long *)rdata;
      for (i=0; i<hed.nx*hed.ny; i++) rdata[i]=dl[i];
    }
    break;
  }
  doneData();

  return 0;
}

int EMData::readIMAGIC(const char *fsp3, int n,int nodata) {
  List *list;
  EMData *tmp;
  int ret;
  imagicH *hed;
  FILE *in,*inh;
  char *fsp,*fsp2,*t;
  float *data;
  short *sdata;
  unsigned char *cdata,jj;
  char *lab[] = { "","Byte reversed ","" };
  int i,j,count,type,ord=0;

  hed=(imagicH *)Smalloc(sizeof(imagicH),1);

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  fsp=(char *)malloc(strlen(fsp3)+5);
  fsp2=(char *)malloc(strlen(fsp3)+5);
  strcpy(fsp,fsp3);
  strcpy(fsp2,fsp3);
  if (strrchr(fsp3,'.')!=NULL) { *(strrchr(fsp,'.'))=0; *(strrchr(fsp2,'.'))=0; }

  strcat(fsp2,".hed");
  strcat(fsp,".img");

  inh=fopen(fsp2,"r");
  in=fopen(fsp,"r");
  if (inh==NULL || in==NULL) { error("ERROR: cannot open imagic file",ERR_ERROR); Sfree(hed,1); zero(); return -1; }

  fread(hed,4,256,inh);
  rewind(inh);

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  // check byte order of image (header must match image)
  t=(char *)&hed->headrec;
#ifdef MSB_FIRST
  if (t[0]==0&&t[1]==0) ord=0; else ord=1;
#else
  if (t[0]==0&&t[1]==0) ord=1; else ord=0;
#endif

  if (strncmp(hed->type,"REAL",4)==0) type=1;
  else if (strncmp(hed->type,"INTG",4)==0) type=0;
  else {
    error("Unsupported imagic type\n",ERR_ERROR);
    fclose(in);
    fclose(inh);
    Sfree(hed,1);
    zero();
    return -1;
  }

  if (ord) { 
    //	fprintf(stderr,"Image %s is in opposite byte order, data byte order will be inverted\n",fsp);
    swapint((char *)hed,14);
    swapint((char *)&hed->ixold,14);
    swapint((char *)&hed->SPACE,207);
  }

  count=hed->count+1;
  if (n>=count) { fclose(in); fclose(inh); free(hed); zero(); return -1; }

  portable_fseek(inh,1024*n,SEEK_SET);
  EMAN_off_t cur_offset = ((EMAN_off_t)n)*hed->nx*hed->ny*(2+2*type);
  portable_fseek(in,cur_offset,SEEK_SET);
  fread(hed,4,256,inh);

  if (ord==1) { 
    swapint((char *)hed,14);
    swapint((char *)&hed->ixold,14);
    swapint((char *)hed->SPACE,207);
  }

  if (nodata) flags|=EMDATA_NODATA;
  setSize(hed->nx,hed->ny,1);
  imagich=hed;

  if (!nodata) {
    data=getData();
    sdata=(short *)data;
    cdata=(unsigned char *)data;

    // read the data (remember to flip y
    for (i=ny-1; i>=0; i--) {
      if (updProgress && updProgress(ny-i-1,ny,"Reading IMAGIC file")) break;
      fread(cdata+i*nx*(2+2*type),nx*(2+2*type),1,in);
    }
    if (updProgress) updProgress(0,0,NULL);

    if (type) {
      if (ord) swapint((char *)data,nx*ny);
    }
    else {
      if (ord) {
	for (i=nx*ny-1; i>=0; i--) { 
	  jj=cdata[i*2]; 
	  cdata[i*2]=cdata[i*2+1];
	  cdata[i*2+1]=jj;
	  data[i]=sdata[i];
	}
      }
      else {
	for (i=nx*ny-1; i>=0; i--) data[i]=sdata[i];
      }
    }

  }

  fclose(in);
  fclose(inh);

  free(fsp);
  free(fsp2);

  pixel=4.66667;
  parent=NULL;
  pathnum=n;
  dx=dy=dz=0;
  if (!goodf(&hed->MRC1[0])) hed->MRC1[0]=0;
  if (!goodf(&hed->MRC1[1])) hed->MRC1[1]=0;
  if (!goodf(&hed->MRC1[2])) hed->MRC1[2]=0;
  setRAlign(hed->MRC1[1],hed->MRC1[2],hed->MRC1[0]);	// inappropriate use, but what the heck
  nimg=hed->MRC2;
  atime=time(0);
  hed->SPACE[0]=0;
  hed->label[79]=0;
  setName(hed->label);
  if (name[0]=='!' && name[1]=='-') {
    i=sscanf(name+2," %f %f %f %f %f %f %f %f %f %f %f",
	     ctf,ctf+1,ctf+2,ctf+3,ctf+4,ctf+5,ctf+6,ctf+7,ctf+8,ctf+9,ctf+10); 
    if (i==11) flags|=EMDATA_HASCTF;
    else printf("Incomplete CTF info: %s\n",name);
    //	printf("IMGCTF '%s' %f %f %f\n",name,ctf[0],ctf[1],ctf[2]);
  }
  if (name[0]=='!' && name[1]=='$') {
    sscanf(name+2," %f,%f",ctf,ctf+10);
  }
  //calcHist(Min(),Max());
  doneData();
  update();
  return(0);
}

int EMData::readIMAGIC3d(const char *fsp3,int nodata)
{
  List *list;
  EMData *tmp;
  int ret;
  imagicH *hed;
  FILE *in,*inh;
  char *fsp,*fsp2,*t;
  float *data;
  short *sdata;
  unsigned char *cdata;
  char *lab[] = { "","Byte reversed ","" };
  int i,j,count,type,ord=0;

  hed=(imagicH *)Smalloc(sizeof(imagicH),1);

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  fsp=(char *)malloc(strlen(fsp3)+5);
  fsp2=(char *)malloc(strlen(fsp3)+5);
  strcpy(fsp,fsp3);
  strcpy(fsp2,fsp3);
  if (strrchr(fsp3,'.')!=NULL) { *(strrchr(fsp,'.'))=0; *(strrchr(fsp2,'.'))=0; }

  strcat(fsp2,".hed");
  strcat(fsp,".img");

  inh=fopen(fsp2,"r");
  in=fopen(fsp,"r");
  if (inh==NULL || in==NULL) { error("ERROR: cannot open imagic file",ERR_ERROR); Sfree(hed,1); zero(); return -1; }

  fread(hed,4,256,inh);
  rewind(inh);

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  // check byte order of image (header must match image)
  t=(char *)&hed->headrec;
#ifdef MSB_FIRST
  if (t[0]==0&&t[1]==0) ord=0; else ord=1;
#else
  if (t[0]==0&&t[1]==0) ord=1; else ord=0;
#endif

  if (strncmp(hed->type,"REAL",4)==0) type=1;
  else if (strncmp(hed->type,"INTG",4)==0) type=0;
  else {
    error("Unknown imagic type\n",ERR_ERROR);
    fclose(in);
    fclose(inh);
    Sfree(hed,1);
    zero();
    return -1;
  }

  if (ord) {
    swapint((char *)hed,14);
    swapint((char *)&hed->ixold,14);
    swapint((char *)&hed->SPACE,207);
  }

  count=hed->count+1;

  if (!(count==hed->nx || (hed->nx==hed->ny && hed->nx==count*2-1))) { return readIMAGIC(fsp3,0,nodata); }	// if it isn't a valid 3d file (cube) return the first 2d image
  //{ printf("Imagic3d invalid %d,%d,%d\n",hed->nx,hed->ny,count); zero(); return(1); }

  if (nodata) flags|=EMDATA_NODATA;
  setSize(hed->nx,hed->ny,count);
  imagich=hed;

  if (!nodata) {
    data=getData();
    sdata=(short *)data;
    cdata=(unsigned char *)data;

    printf("Imagic3d -> %d,%d,%d\n",nx,ny,nz);

    // read the data
    for (j=0; j<nz; j++) {
      if (updProgress && updProgress(j,nx,"Reading IMAGIC 3D file")) break;
      for (i=ny-1; i>=0; i--) {
	fread(cdata+(i+j*ny)*nx*(2+2*type),nx*(2+2*type),1,in);
      }
    }

    if (updProgress) updProgress(0,0,NULL);

    if (type) {
      if (ord) swapint((char *)data,nx*ny*nz);
    }
    else {
      if (ord) {
	for (i=nx*ny*nz-1; i>=0; i--) { 
	  j=cdata[i*2]; 
	  cdata[i*2]=cdata[i*2+1];
	  cdata[i*2+1]=j;
	  data[i]=sdata[i];
	}
      }
      else {
	for (i=nx*ny*nz-1; i>=0; i--) data[i]=sdata[i];
      }
    }
  }

  fclose(in);
  fclose(inh);

  free(fsp);
  free(fsp2);

  pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  setRAlign(0,0,0);
  atime=time(0);
  //calcHist(Min(),Max());
  doneData();
  update();
  return(0);
}

int EMData::readEMIM(const char *filespec, int n,int nodata)
{
  FILE *in;
  emimH1 imh1;
  emimH2 imh2;
  int swap=0;

  in=fopen(filespec,"r");
  if (!in) { error("EMIM: Cannot open emim file for reading",ERR_ERROR); zero(); return -1; }

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;

  // read main header
  if (!fread(&imh1,sizeof(emimH1),1,in)) { error("EMIM: Cannot read EMIM",ERR_ERROR); zero(); return -1; }

  if (strncmp(imh1.magic,"EMIM",4)) { error("EMIM: File is not an EMIM",ERR_ERROR); zero(); return -1; }

  if (imh1.count&0x7f000000) swap=1;	// there are unlikely to be more than 16e6 images in the file

  if (swap) swapint((char *)&imh1,7);		// swap order of main header if necessary

  if (n>=imh1.count) { error("EMIM: Not enough images",ERR_ERROR); zero(); return -2; }

  // move to the image we want
  EMAN_off_t cur_offset = ((EMAN_off_t)imh1.nx)*imh1.ny*imh1.nz*sizeof(float);
  cur_offset = n*(sizeof(emimH2)+cur_offset);
  cur_offset +=  sizeof(emimH1);
  portable_fseek(in,cur_offset,SEEK_SET);

  fread(&imh2,sizeof(emimH2),1,in);

  if (nodata) flags|=EMDATA_NODATA;
  setSize(imh1.nx,imh1.ny,imh1.nz);

  if (!nodata) {
    getData();
    if (fread(rdata,nx*nz*sizeof(float),ny,in)!=ny) { error("EMIM: File truncated?",ERR_ERROR); zero(); return -3; }

    if (swap) {
      swapint((char *)&imh2,10);
      //	swapint((char *)imh2.param,140);
      swapint((char *)rdata,nx*ny*nz);
    }

    doneData();
  }

  dx=imh2.dx;
  dy=imh2.dy;
  dz=imh2.dz;
  setRAlign(imh2.alt,imh2.az,imh2.phi);
  //memcpy(param,imh2.param,25*4*sizeof(float));
  //memcpy(ptype,imh2.ptype,4);

  setName(imh2.name);

  flags=0;
  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  if (imh1.flag&EMIM_COMPLEX) flags|=EMDATA_COMPLEX+EMDATA_RI;

  pixel=imh1.pixel;
  parent=NULL;
  pathnum=n;
  atime=time(0);
  update();
  //calcHist(Min(),Max());
  return 0;
}

// write 3D imagic image (just a set of slices)
int EMData::writeIMAGIC3D(const char *filespec, int nolock) {
  int i;
  for (i=0; i<nz; i++) {
    EMData *d = clip(0,0,i,nx,ny,1);
    d->writeIMAGIC(filespec,i);
    delete d;
  }
  return 0;  
}

// write routines for specific formats
int EMData::writeIMAGIC(const char *filespec, int n,int nolock,bool headonly)	// n=-1 appends 
{
  EMData *tmp;
  int ret;
  imagicH hed;
  FILE *in,*inh;
  char *fsp,*fsp2;
  int i,swap=0,count,size;
  time_t itime;
  struct tm *tm;
  float *cpy;

  setPath(filespec);
  fsp=(char *)Smalloc(strlen(filespec)+5,0);
  fsp2=(char *)Smalloc(strlen(filespec)+5,0);
  strcpy(fsp,filespec);
  if (strlen(fsp)>4 && (strcmp(fsp+strlen(fsp)-4,".img")==0 || strcmp(fsp+strlen(fsp)-4,".hed")==0)) fsp[strlen(fsp)-4]=0;
  strcpy(fsp2,fsp);

  strcat(fsp2,".hed");
  strcat(fsp,".img");

#ifdef FILESERVER
  if (FSHOST) { 
    int sock=sock_open(FSHOST,FSPORT);
    if (sock<0) { printf("%hu,%x\n",FSPORT,FSHOST); error(ERR_EXIT,"No socket to fileserver"); }
    char path[MAXPATHLEN];
    getcwd(path,MAXPATHLEN-1);
    FSCOMMAND com;
    com.datalen=sizeof(EMData)+nx*ny*nz*sizeof(float);
    com.command=FS_IMAGIC;
    com.key=FSKEY;
    com.fileno=n;
    com.filelen=strlen(fsp2);
    com.pathlen=strlen(path);
#ifdef DEBUG2
    printf("IMAGIC: Send file to %x %hd: fileno %d\n",FSHOST,FSPORT,n);
#endif
    writesock(sock,(char *)&com,sizeof(com));
    writesock(sock,(char *)path,strlen(path));
    writesock(sock,(char *)fsp2,strlen(fsp2));
    serialOut(sock);
    close(sock);
    Sfree(fsp,0);
    Sfree(fsp2,0);
    return (0); 
  }
#endif

  inh=fopen(fsp2,"r+");
#ifdef DEBUG2
  if (inh==NULL) printf("Error opening header '%s' (%d)\n",fsp2,errno);
#endif
  if (inh==NULL) inh=fopen(fsp2,"w+");
  if (inh==NULL) { error("Cannot open imagic header.",ERR_ERROR); zero(); return -1; }
  if (!nolock) file_lock_wait(inh);
  if (!headonly){
    in=fopen(fsp,"r+");
    if (in==NULL) in=fopen(fsp,"w+");
    if (in==NULL) { fclose(inh); error("Cannot open imagic file.",ERR_ERROR); zero(); return -1; }
    if (!nolock) file_lock_wait(in);
  }

#ifdef DEBUG2
  printf("IMAGIC: Begin Imagic write '%s' %d\n",fsp2,n);
#endif

  // new file, start from zero
  if (fread(&hed,1024,1,inh)!=1) {
    if (!feof(inh)) { printf("ERROR: fatal IMAGIC file problem ... bye (%d)\n",errno); exit(1); }
    hed.nx=nx;
    hed.ny=ny;
    hed.count=-1;
    hed.headrec=1;
#ifdef DEBUG2
    printf("IMAGIC: new file %s\n",fsp2);
#endif
  }
  rewind(inh);

  // check byte order
  if (hed.headrec>64) {
    swap=1;		// file has the opposite order we do
    swapint((char *)&hed,256);
  }

  // make sure the file has the right image size (all must be same)
  count=hed.count+1;
  size=hed.nx*hed.ny*4;
  if (nx*ny*4!=size) { error("Imagic file wrong size",ERR_ERROR); if (!nolock) { file_unlock(in); file_unlock(inh); } zero(); return -2; }
  if (n==count) n=-1;
  if (n>count) {
#ifdef DEBUG2
    printf("IMAGIC: extend from %d to %d\n",count,n);
#endif
    EMData tmp;
    tmp.setSize(nx,ny,nz);
    tmp.zero();
    tmp.setNImg(-1);
    fclose(inh);
    fclose(in);
    while (n>count) { tmp.writeIMAGIC(filespec,-1,1); count++; }
    Sfree(fsp,0);
    Sfree(fsp2,0);
    return writeIMAGIC(filespec,n,1);
    //	n=-1;
    //	inh=fopen(fsp2,"r+");
    //	in=fopen(fsp,"r+");
  }

  // seek to the correct place in the file
  if (n<0) {
    portable_fseek(inh,0,SEEK_END);
    if(!headonly) portable_fseek(in,0,SEEK_END);
    count++;
    hed.imnum=count;
  }
  else {
    portable_fseek(inh,1024*n,SEEK_SET);	
    if(!headonly) portable_fseek(in,((EMAN_off_t)size)*n,SEEK_SET);
    hed.imnum=n;
  }
  pathnum=hed.imnum;

#ifdef DEBUG2
  printf("IMAGIC: actual write %d (n=%d)\n",n,count);
#endif

  // set up the header
  itime=time(0);
  tm=localtime(&itime);
  hed.error=0;
  hed.headrec=1;
  hed.mday=tm->tm_mday;
  hed.month=tm->tm_mon;
  hed.year=tm->tm_year+1900;
  hed.hour=tm->tm_hour;
  hed.minute=tm->tm_min;
  hed.sec=tm->tm_sec;
  hed.reals=nx*ny;
  hed.pixels=nx*ny;
  hed.ny=ny;
  hed.nx=nx;
  hed.ixold=hed.iyold=0;
  hed.oldav=0;
  if (!(flags&EMDATA_NODATA)) {
    hed.max=Max();
    hed.min=Min();
    hed.avdens=Mean();
    hed.sigma=Sigma();
  }
  hed.MRC1[0]=euler.phi();
  hed.MRC1[1]=euler.alt();
  hed.MRC1[2]=euler.az();
  hed.MRC2=nimg;


  // buffering parameters IMAGIC seems to need

  hed.lbuf=nx;
  hed.inn=1;
  hed.iblp=ny;
  hed.ifb=0;
  hed.lbw=0;
  hed.lbr= -1;
  hed.lastlr= -1;
  hed.lastlw=1;
  hed.num=8; // ???
  hed.nhalf=nx/2;
  hed.ibsd=nx*2;
  hed.ihfl=7;
  hed.lcbr= -1;
  hed.lcbw=1;
  hed.imstr= -1;
  hed.imstw= -1;
  hed.istart=1;
  hed.iend=nx;
  hed.leff=nx;
  hed.linbuf=nx*2;
  hed.ntotbuf= -1;
  hed.icstart=1;
  hed.icend=nx/2;
  memset(&hed.clsrep,0,157*4);

  getDataRO();
  if (swap && !(flags&EMDATA_NODATA)) {
    swapint((char *)&hed,256);
    if(swap){
      cpy=(float *)Smalloc(nx*ny*sizeof(float),0);
      memcpy(cpy,rdata,nx*ny*sizeof(float));
      swapint((char *)cpy,nx*ny);
    }
  }
  else cpy=rdata;

  if (flags&EMDATA_HASCTF) {
    char c[80];
    sprintf(c,"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	    ctf[0],ctf[1],ctf[2],ctf[3],ctf[4],ctf[5],ctf[6],ctf[7],ctf[8],ctf[9],ctf[10]);
    setName(c);		
  }

  if (name) strncpy(hed.label,name,80); else name[0]=0;
  hed.label[79]=0;
  hed.type[0]='R';
  hed.type[1]='E';
  hed.type[2]='A';
  hed.type[3]='L';

  fwrite(&hed,1024,1,inh);

  if(!((flags&EMDATA_NODATA)||headonly)){		
    for (i=ny-1; i>=0; i--) {
      fwrite(&cpy[i*nx],4*nx,1,in);
      if(ferror(in)) {
	fprintf(stderr,"ERROR: writeIMAGIC cannot write to file \"%s\". The disk might be full\n",filespec);
	exit(1);
      }
    }
    if (swap) Sfree(cpy,0);
  }
  doneData();

  portable_fseek(inh,4,SEEK_SET);
  count--;
  if (swap) swapint((char *)&count,1);
  fwrite(&count,4,1,inh);
  //printf("%d records\n",count);
  if (!nolock && !headonly) file_unlock(in);
  if (!nolock) file_unlock(inh);
  if(!headonly) fclose(in);
  fclose(inh);

  Sfree(fsp,0);
  Sfree(fsp2,0);

#ifdef DEBUG2
  printf("IMAGIC: exit write %d\n",n);
#endif
  return 0;
}

int EMData::writeMRC(const char *filespec,int mode,int n) {
  struct tm *t;
  time_t t0=time(0);
  FILE *out;

  if(n>=0 && nz>1) { error("Could not write MRC by section",ERR_ERROR); return -1;}
	
#ifdef FILESERVER
  if (FSHOST) { 
    int sock=sock_open(FSHOST,FSPORT);
    if (sock<0) { printf("%hu,%x\n",FSPORT,FSHOST); error(ERR_EXIT,"No socket to fileserver"); }
    char path[MAXPATHLEN];
    getcwd(path,MAXPATHLEN-1);
    FSCOMMAND com;
    com.datalen=sizeof(EMData)+nx*ny*nz*sizeof(float);
    com.command=FS_MRC;
    com.key=FSKEY;
    com.fileno=n;
    com.filelen=strlen(filespec);
    com.pathlen=strlen(path);
#ifdef DEBUG2
    printf("IMAGIC: Send file to %x %hd: fileno %d\n",FSHOST,FSPORT,n);
#endif
    writesock(sock,(char *)&com,sizeof(com));
    writesock(sock,(char *)path,strlen(path));
    writesock(sock,(char *)filespec,strlen(filespec));
    serialOut(sock);
    close(sock);
    return (0); 
  }
#endif

  if (filespec==NULL) { out=stdout; }
  else {
    if(n>=0 && (out=fopen(filespec,"r+")) ){
      if (!mrch) mrch=(mrcH *)Smalloc(sizeof(mrcH),1);
      if (!fread(mrch,sizeof(mrcH),1,out)) {
	fprintf(stderr,"ERROR: writeMRC cannot write to invalid MRC file %s",filespec); 
	exit(1);
      }

      // This swaps the byte order in the header on appropriate machines
#ifdef MSB_FIRST
      int mord=1, ord=0;
#else
      int mord=0, ord=0;
#endif

      char* m2=(char *)mrch;
      if (m2[0]==0 && m2[1]==0) ord=1;
      if (mord ^ ord) {	
	fprintf(stderr,"ERROR: writeMRC cannot write to existing MRC file %s which is in opposite byte order\n",filespec); 
	exit(1);
      }
      else if (mrch->ny>0xffff || mrch->nz>0xffff || mrch->mode>4 || mrch->nlabl>11) {
	fprintf(stderr,"ERROR: writeMRC cannot write to invalid MRC file %s\n",filespec); 
	exit(1);
      }
      else if(mrch->nx != nx || mrch->ny != ny){
	fprintf(stderr,"ERROR: writeMRC cannot write to existing MRC file %s which is of different size (%dx%d != %dx%d)\n",filespec,nx,ny,mrch->nx,mrch->ny); 
	exit(1);
      }
      else if(mrch->mode != mode){
	fprintf(stderr,"ERROR: writeMRC cannot write to existing MRC file %s which is of different mode\n",filespec); 
	exit(1);
      }
      portable_fseek(out,0,SEEK_SET);
    }
    else {
      out=fopen(filespec,"w");
    }
    if (!out) { 
      fprintf(stderr,"ERROR: writeMRC cannot open %s to write",filespec); 
      exit(1);
    }
    file_lock_wait(out);
  }
  setPath(filespec);
  pathnum=0;

  t=localtime(&t0);

  // this is to filp the phase for conforming to MRC convention
  if (isComplex()) {
    ap2ri();
    getData();
    for(int i=0;i<nx*ny*nz;i+=2)rdata[i+1]*=-1;
    doneData();
    if (mode==MODE_float) mode=MODE_float_COMPLEX;
    if (mode==MODE_short) mode=MODE_short_COMPLEX;
  }
		
  if (!mrch) {
    mrch=(mrcH *)Smalloc(sizeof(mrcH),1);
    memset(mrch,0,sizeof(mrcH));
    mrch->mx=nx-1;
    mrch->my=ny-1;
    mrch->mz=nz-1;
    mrch->xlen=mrch->ylen=mrch->zlen=pixel;
    mrch->alpha=mrch->beta=mrch->gamma=90.0;
    mrch->mapc=1;
    mrch->mapr=2;
    mrch->maps=3;
    mrch->nxstart=mrch->nystart=mrch->nzstart=0;
    mrch->nlabl=1;
    sprintf(&mrch->labels[0][0],"EMAN %d/%02d/%02d %d:%02d %s",
	    t->tm_mon,t->tm_mday,t->tm_year,t->tm_hour,t->tm_min,name);
    mrch->labels[0][79]=0;
    mrch->labels[1][0]=0;
  }
  else if (mrch->nlabl<MRC_NUM_LABELS-1) {
    sprintf(&mrch->labels[mrch->nlabl][0],"EMAN %d/%02d/%02d %d:%02d %1.50s",
	    t->tm_mon,t->tm_mday,t->tm_year,t->tm_hour,t->tm_min,name);
    mrch->labels[mrch->nlabl][79]=0;
    mrch->nlabl++;
  }

  if (flags&EMDATA_HASCTF) {
    sprintf(&mrch->labels[0][0],"!-%1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g %1.3g",
	    ctf[0],ctf[1],ctf[2],ctf[3],ctf[4],ctf[5],ctf[6],ctf[7],ctf[8],ctf[9],ctf[10]);
  }
  if (hasCTFF()) {
    strncpy(&mrch->labels[0][0],name,79);
  }

  mrch->nx=nx;
  if(isComplex())mrch->nx=nx/2;
  mrch->ny=ny;
  if(n<0)mrch->nz=nz;
  else if(n>=0 && n>=mrch->nz) mrch->nz=n+1;
  mrch->ispg=0;
  mrch->nsymbt=0;
  mrch->amin=Min();
  mrch->amax=Max();
  mrch->amean=Mean();
  mrch->mode=mode;

  //for compatibilty to CCP4 format
  if (nz>1) {
#ifdef MSB_FIRST 
    ((char*)mrch)[54]=0;
#else
    ((char*)mrch)[54]=1;
#endif
    mrch->ispg=1;
    mrch->xlen=nx*pixel;
    mrch->ylen=ny*pixel;
    mrch->zlen=nz*pixel;
    mrch->nxstart=-(nx/2);
    mrch->nystart=-(ny/2);
    mrch->nzstart=-(nz/2);
    mrch->mx=nx;
    mrch->my=ny;
    mrch->mz=nz;
    if (nz*2-1==nx && nz*2-1==ny){
      mrch->nzstart=0;
    }
  }
	
  switch (mode) {
  case MODE_float:
  case MODE_float_COMPLEX:
    fwrite(mrch,sizeof(mrcH),1,out);
    if(n>=0) portable_fseek(out,sizeof(float)*nx*ny*n,SEEK_CUR);
    fwrite(getData(),nx*nz,ny*sizeof(float),out);
    if(ferror(out)) {
      fprintf(stderr,"ERROR: writeMRC cannot write to file \"%s\". The disk might be full\n",filespec);
      exit(1);
    }
    break;
  case MODE_short:
  case MODE_short_COMPLEX:
    error(ERR_EXIT,"Unimplemented MRC short write");
    break;
  case MODE_char:
    fwrite(mrch,sizeof(mrcH),1,out);
    if(n>=0) portable_fseek(out,sizeof(unsigned char)*nx*ny*n,SEEK_CUR);
    unsigned char *tmp=(unsigned char *)malloc(nx*ny*nz);
    float ss=Max()-Min(),s0=Min();
    if (ss==0) ss=.000001;
    for (int i=0; i<nx*ny*nz; i++) tmp[i]=(unsigned char)floor(255.998*(rdata[i]-s0)/ss);
    fwrite(tmp,nx*nz,ny,out);
    if(ferror(out)) {
      fprintf(stderr,"ERROR: writeMRC cannot write to file \"%s\". The disk might be full\n",filespec);
      exit(1);
    }
    free(tmp);
    break;
  }
  doneData();
  if(filespec!=NULL){
    file_unlock(out);
    fclose(out);
  }

  if (isComplex()) {
    ap2ri();
    getData();
    for(int i=0;i<nx*ny*nz;i+=2)rdata[i+1]*=-1;
    doneData();
  }
  return 0;
}

int EMData::writeVTK(const char *filespec) {
  struct tm *t;
  time_t t0=time(0);
  FILE *out;


  if(filespec==NULL){ out=stdout; }
  else{
    out=fopen(filespec,"w");
    if (!out) { error("Could not write VTK",ERR_ERROR); return -1; }
    file_lock_wait(out);
  }
  setPath(filespec);
  pathnum=0;

  // VTK files are big-endian
  int swap=1;
  if (((char *)&swap)[0]==0) swap=0;

  t=localtime(&t0);

  fprintf(out,"# vtk DataFile Version 2.0\n");
  fprintf(out,"%s\n",Name());
  fprintf(out,"BINARY\n");
  fprintf(out,"DATASET STRUCTURED_POINTS\nDIMENSIONS %0d %0d %0d\nORIGIN %0d %0d %0d\nSPACING 1 1 1\n",
	  nx,ny,nz,-nx/2,-ny/2,-nz/2);
  fprintf(out,"POINT_DATA %0d\nSCALARS density float 1\nLOOKUP_TABLE default\n",nx*ny*nz);
  if (swap) swapint((char *)rdata,nx*ny*nz);		// unsafe briefly, but what the heck
  fwrite(getDataRO(),nx*nz,ny*sizeof(float),out);
  if (swap) swapint((char *)rdata,nx*ny*nz);
  doneData();

  if(filespec!=NULL){
    file_unlock(out);
    fclose(out);
  }

  return 0;
}


int EMData::writeICOS(const char *filespec) {
  struct tm *t;
  time_t t0=time(0);
  FILE *out;


  if(filespec==NULL){ out=stdout; }
  else{
    out=fopen(filespec,"w");
    if (!out) { error("Could not write ICOS",ERR_ERROR); return -1; }
    file_lock_wait(out);
  }
  setPath(filespec);
  pathnum=0;

  t=localtime(&t0);

  icosH icosh;
  icosh.stamp=icosh.stamp1=72;
  icosh.stamp2=icosh.stamp3=20;

  strncpy(icosh.title,name,71);
  icosh.nx=nx;
  icosh.ny=ny;
  icosh.nz=nz;
  icosh.min=Min();
  icosh.max=Max();
  fwrite(&icosh,1,sizeof(icosH),out);

  float *tmp=new float[nx+2];
  int *td0=(int*)tmp, *td1=(int*)(tmp+nx+1);
  *td0=*td1=nx*sizeof(float);

  float *mydata=getData();
  for (int r=0; r<ny*nz; r++){
    memcpy((void*)(tmp+1),(void*)(mydata+nx*r),nx*sizeof(float));
    fwrite(tmp,1,(nx+2)*sizeof(float),out);
  }	

  delete tmp;
	
  doneData();

  if(filespec!=NULL){
    file_unlock(out);
    fclose(out);
  }
  return 0;
}

int EMData::writeAMIRA(const char *filespec) {
  struct tm *t;
  time_t t0=time(0);
  FILE *out;


  if(filespec==NULL){ out=stdout; }
  else{
    out=fopen(filespec,"w");
    if (!out) { error("Could not write AMIRA",ERR_ERROR); return -1; }
    file_lock_wait(out);
  }
  setPath(filespec);
  pathnum=0;

  t=localtime(&t0);

  fprintf(out,"# AmiraMesh 3D BINARY 2.0\n\n# Dimensions in x-, y-, and z-direction\n");
  fprintf(out,"define Lattice %d %d %d\n\n",nx,ny,nz);
  fprintf(out,"Parameters {\n\tCoordType \"uniform\",\n\t# BoundingBox is xmin xmax ymin ymax zmin zmax\n\t");
  if(nx==ny && nx==nz){
    fprintf(out,"BoundingBox %f %f %f %f %f %f\n}\n\n",-int(nx/2)*pixel,(nx-1-nx/2)*pixel,-int(ny/2)*pixel,(ny-1-ny/2)*pixel,
	    -int(nz/2)*pixel,(nz-1-nz/2)*pixel);
  }
  else if(nx==ny && nx==nx/2*2+1 && nx==2*nz-1) {// the Virus half map
    fprintf(out,"BoundingBox %f %f %f %f %f %f\n}\n\n",-int(nx/2)*pixel,(nx-1-nx/2)*pixel,-int(ny/2)*pixel,(ny-1-ny/2)*pixel,0.0,(nz-1)*pixel);
  }
  else {
    fprintf(out,"BoundingBox %f %f %f %f %f %f\n}\n\n",0.0,(nx-1)*pixel,0.0,(ny-1)*pixel,0.0,(nz-1)*pixel);
  }
  fprintf(out,"Lattice { float ScalarField } = @1\n\n@1\n");

#ifdef MSB_FIRST 
  fwrite(getData(),nx*nz,ny*sizeof(float),out); //AMIRA convention: MSB_FIRST only
#else
  char *tmpdata=new char[4*nx*ny*nz];
  char *cdata=(char*)getData();
  for(int i=0;i<nx*ny*nz;i++){
    tmpdata[4*i]=cdata[4*i+3];
    tmpdata[4*i+1]=cdata[4*i+2];
    tmpdata[4*i+2]=cdata[4*i+1];
    tmpdata[4*i+3]=cdata[4*i];
  }
  fwrite(tmpdata,nx*nz,ny*sizeof(float),out); //AMIRA convention: MSB_FIRST only
  if(ferror(out)) {
    fprintf(stderr,"ERROR: writeAMIRA cannot write to file \"%s\". The disk might be full\n",filespec);
    exit(1);
  }
  delete tmpdata;
#endif
	
  doneData();

  if(filespec!=NULL){
    file_unlock(out);
    fclose(out);
  }

  return 0;
}

int EMData::writeXPLOR(const char *filespec) {
  struct tm *t;
  time_t t0=time(0);
  FILE *out;

  if(filespec==NULL){ out=stdout; }
  else{
    out=fopen(filespec,"w");
    if (!out) { error("Could not write XPLOR",ERR_ERROR); return -1; }
    file_lock_wait(out);
  }
  setPath(filespec);
  pathnum=0;

  t=localtime(&t0);

  fprintf(out,"\n%8d\n\"%s\" written by EMAN at %s",1,filespec,asctime(t));
  int z0=-nz/2,z1=nz%2?nz/2:nz/2-1;
  if(2*nz-1==nx && 2*nz-1==ny) {z0=0; z1=nz-1;}
  fprintf(out,"%8d%8d%8d%8d%8d%8d%8d%8d%8d\n",nx,-nx/2,nx%2?nx/2:nx/2-1,ny,-ny/2,ny%2?ny/2:ny/2-1,nz,z0,z1);
  fprintf(out,"%12.5E%12.5E%12.5E%12.5E%12.5E%12.5E\nZYX\n",nx*pixel,ny*pixel,nz*pixel,90.0,90.0,90.0);

  float *data=getDataRO();
  int i, k;
  for(k=0;k<nz;k++){
    fprintf(out,"%8d\n",k);
    for(i=0;i<nx*ny-6;i+=6) {
      fprintf(out,"%12.5E%12.5E%12.5E%12.5E%12.5E%12.5E\n",data[k*nx*ny+i],data[k*nx*ny+i+1],data[k*nx*ny+i+2],data[k*nx*ny+i+3],data[k*nx*ny+i+4],data[k*nx*ny+i+5]);
    }
    for(i=(nx*ny-1)/6*6;i<nx*ny;i++) 	fprintf(out,"%12.5E",data[k*nx*ny+i]);
    fprintf(out,"\n");
  }
  fprintf(out,"%8d\n",-9999);
  doneData();

  if(filespec!=NULL){
    file_unlock(out);
    fclose(out);
  }

  return 0;
}

int EMData::writePGM(const char *filespec,float gmin,float gmax) {
  FILE *out;
  unsigned char *tmp;

  if (nz!=1) { error("Cannot write 3d image as PGM",ERR_ERROR); return -2; }

  out=fopen(filespec,"w");
  if (!out) { error("Could not write PGM",ERR_ERROR); return -1; }

  tmp=(unsigned char *)malloc(nx*ny);
  renderAmp8(tmp,0,0,nx,ny,nx,1.0,0,255,gmin,gmax);

  fprintf(out,"P5\n%d %d\n255\n",nx,ny);
  fwrite(tmp,nx,ny,out);
  free(tmp);
  fclose(out);

  return 0;
}

int EMData::writeEMIM(const char *filespec, int n)	// n=-1 appends
{
  return -1;
}

// read the initial spider header
SPIDERH *readSpiHed(const char *fsp) {
  int i,swap,imsize;
  SPIDERH *sh;
  FILE *in=fopen(fsp,"r");
  if (!in) return NULL;

  sh=(SPIDERH *)malloc(sizeof(SPIDERH));
  // read initial header
  if (!fread(sh,sizeof(SPIDERH),1,in)) { free(sh); return NULL; }
  fclose (in);

  if (goodf(&sh->nslice) && sh->nslice>0&&sh->nslice<10000.0 && sh->nslice==floor(sh->nslice)) swap=0; else swap=1;
  if (swap) swapint((char *)sh,211);

  // Basic tests to see if the header is valid
  /*for (i=0; i<160; i++) {
    if (!isprint(sh->name[i]) && sh->name[i]!=0 ) break;
    }
    if (i!=160 || sh->nslice!=floor(sh->nslice) || sh->nslice<1.0 ||sh->nslice>8192 ||
    (sh->nslice!=1.0 && sh->nslice!=3.0)) { free(sh); return NULL; }*/
  if (int(sh->nslice)!=sh->nslice||sh->nslice<1.0 ||sh->nslice>8192) 
    { free(sh); return NULL; }

  // a few return values we shove into unused areas
  if (sh->istack!=2) sh->u8[158]=1;	// # images in file
  else sh->u8[158]=sh->maxim;
  sh->u8[159]=swap;					// byte order change necessary ?
  sh->u8[160]=sh->headlen+sh->nx*sh->ny*sh->nslice*sizeof(float); // total image+hed len

  return sh;
}

unsigned int DEBUGHOST = 0;

struct DBWHED { int com, imnum, nx, ny, nz; };

void EMData::writeDebug(int n) {
  if (!DEBUGHOST) return;

  int sock=sock_open(DEBUGHOST,33333);
  if (sock<0) return;

  DBWHED dbh = { 0,n,nx,ny,nz };
  writesock(sock,(char*)&dbh,sizeof(DBWHED));

  writesock(sock,(char*)rdata,nx*ny*nz*sizeof(float));

  close(sock);
}

// read a spider file
int EMData::readSPIDER(const char *fsp, int n, int nodata) {
  FILE *in;
  int swap=0,sz,stack;
  SPIDERH *sh;

  if (!spiderH) spiderH=(SPIDERH *)calloc(sizeof(SPIDERH),1);

  flags=EMDATA_NEEDUPD|EMDATA_NEEDHIST;
  sh=readSpiHed(fsp);
  if (!sh) { zero(); return -1; }
  if (sh->istack==2) stack=sh->maxim-1; else stack=0;
  if (n>stack) { free(sh); zero(); return -1; }
  swap=(int)sh->u8[159];

  in=fopen(fsp,"r");

  if (strrchr(fsp,'/')==NULL) setName(fsp);
  else setName(strrchr(fsp,'/')+1);
  setPath(fsp);

  // read header
  portable_fseek(in,(int)(sh->u8[160]*n+(stack?sh->headlen:0)),SEEK_SET);
  if (!fread(spiderH,sizeof(SPIDERH),1,in)) { free(sh); zero(); return -1; }
  if (swap) swapint((char *)spiderH,211);
  portable_fseek(in,(int)spiderH->headlen-sizeof(SPIDERH),SEEK_CUR);

  setSize(spiderH->nx,spiderH->ny,spiderH->nslice);
  euler.setAngle(spiderH->theta*PI/180,spiderH->phi*PI/180,spiderH->gamma*PI/180,Euler::SPIDER);

  if (!nodata) {
    getData();
    if (fread(rdata,nx*ny,4*nz,in)!=4*nz) { printf("Incomplete SPIDER data read\n"); zero(); return -1; }

    if (swap) swapint((char*)rdata,nx*ny*nz);
    doneData();
  }
  fclose(in);

  pixel=4.66667;
  parent=NULL;
  pathnum=0;
  dx=dy=dz=0;
  //setRAlign(0,0,0);
  atime=time(0);
  spiderH=sh;
  return 0;
}

// write a spider file, if n>nimg in file || n==-1 appends
int EMData::writeSPIDER(const char *fsp, int n,int swap) {

  
    SPIDERH *sh;
    FILE *out=NULL;
    int onr;
    int lenbyt=nx*4;//bytes per record
    int hr=(1024/(lenbyt));//number of records
    if(fmod(1024,lenbyt)!=0) hr =hr+1;
    int hl=hr*lenbyt;//header length
    int irec = nx*nz+hr;//(***)format descriptor says this is unused but it
			//is required for reading the stack images
			//correctly. Otherwise, only the first image
			//can be read. 
			
       
  sh=readSpiHed(fsp);
  if (sh) {
    if (sh->u8[159]) { swap=1; }
    if (sh->nx!=nx || sh->ny!=ny || sh->nslice!=nz) { printf("Wrong image size spider write\n"); exit(1); }
    if (sh->istack!=2) { error(ERR_ERROR,"Cannot write to single-image Spider files"); return -2; }
    out=fopen(fsp,"r+");
    if (!out) return (-4);
    file_lock_wait(out);
    if (n==-1) n=sh->maxim;
    rewind(out);
    onr=sh->maxim;
    if (onr<n+1) sh->maxim=n+1;
    if (swap) swapint((char *)sh,211);
    fwrite(sh,sizeof(SPIDERH),1,out);
    if (swap) swapint((char *)sh,211);
    sh->maxim=onr;
    portable_fseek(out,0,SEEK_END);
  }
  else {
    sh=(SPIDERH *)calloc(hl,1);
    sh->mmvalid=0;
    sh->sigma=-1.0;
    sh->angvalid=0;
    sh->scale=1.0;
    sh->nslice=nz;
    sh->nx=nx;
    sh->ny=ny;
    sh->type=(nz==1?1:3);
    sh->reclen=nx*4;
    sh->headrec=hr;
    sh->headlen=hl;
    sh->inuse=-1.;	//?
    sh->istack=2.;
    sh->maxim=1;
    sh->u1=irec;
    
	
    out=fopen(fsp,"w");
    if (!out) { printf("Cannot open '%s'\n",fsp); return -3; }
    file_lock_wait(out);
    if (swap) swapint((char *)sh,211);
    fwrite(sh,hl,1,out);
    if (swap) swapint((char *)sh,211);

    if (n==-1) n=0;
    sh->maxim=0;
  }

  char *t=(char *)calloc(hl-sizeof(SPIDERH),1);

  if (swap) swapint((char *)rdata,nx*ny*nz);
  while (n>=sh->maxim) { 
    sh->maxim++;
    sh->inuse=1.0;
    sh->imgnum=sh->maxim;

    if (swap) swapint((char *)sh,211);
    fwrite(sh,sizeof(SPIDERH),1,out);
    if (swap) swapint((char *)sh,211);

    fwrite(t,hl-sizeof(SPIDERH),1,out);

    fwrite(rdata,nx*ny,nz*sizeof(float),out);
  }
  if (swap) swapint((char *)rdata,nx*ny*nz);

  free(sh);


  if (!spiderH) {
    spiderH=(SPIDERH *)calloc(sizeof(SPIDERH),1);
    spiderH->scale=1.0;
  }
  setPath(fsp);
  pathnum=n;

  spiderH->nslice=nz;
  spiderH->nslice=nz;
  spiderH->nx=nx;
  spiderH->ny=ny;
  spiderH->type=nz==1?1:3;
  spiderH->mmvalid=1;
  spiderH->max=Max();
  spiderH->min=Min();
  spiderH->mean=Mean();
  spiderH->sigma=Sigma();
  spiderH->reclen=nx*4;
  spiderH->inuse=1.0;
  spiderH->istack=-1;
  spiderH->imgnum=n+1;
  spiderH->headrec=hr;
  spiderH->headlen=hl;
  spiderH->angvalid=1.0;
  spiderH->dx=dx;
  spiderH->dy=dy;
  spiderH->dz=dz;
  spiderH->phi=euler.phiSpider()*180/PI;
  spiderH->theta=euler.thetaSpider()*180/PI;
  spiderH->gamma=euler.gammaSpider()*180/PI;
  spiderH->u1=irec;
  strncpy(spiderH->name,name,79);

  EMAN_off_t cur_offset = (EMAN_off_t) (hl+1.0*n*(hl+nx*ny*nz*sizeof(float)));
  portable_fseek(out,cur_offset,SEEK_SET);

  if (swap) swapint((char *)spiderH,211);
  fwrite(spiderH,sizeof(SPIDERH),1,out);
  if (swap) swapint((char *)spiderH,211);

  fwrite(t,hl-sizeof(SPIDERH),1,out);
  free(t);
  getDataRO();

  if (swap) swapint((char *)rdata,nx*ny*nz);
  fwrite(rdata,nx*ny,nz*sizeof(float),out);
  if (swap) swapint((char *)rdata,nx*ny*nz);

  file_unlock(out);
  fclose(out);
  doneData();
  return 0;
}

int EMData::writeSingleSPIDER(const char *fsp,int swap) {
  int hr=(1+1023/(nx*4));
  int hl=hr*nx*4;
  FILE *out=NULL;

  out=fopen(fsp,"w");
  if (!out) { printf("Cannot open '%s'\n",fsp); return -3; }
  file_lock_wait(out);

  if (!spiderH) {
    spiderH=(SPIDERH *)calloc(sizeof(SPIDERH),1);
    spiderH->mmvalid=0;
    spiderH->sigma=-1.0;
    spiderH->angvalid=0;
    spiderH->scale=1.0;
  }
  setPath(fsp);
  pathnum=0;

  spiderH->istack=0;
  spiderH->nslice=nz;
  spiderH->nx=nx;
  spiderH->ny=ny;
  spiderH->type=nz==1?1:3;
  spiderH->reclen=nx*4;
  spiderH->headrec=(1+1023/(nx*4));
  spiderH->headlen=spiderH->headrec*spiderH->reclen;
  spiderH->inuse=1;
  spiderH->dx=dx;
  spiderH->dy=dy;
  spiderH->dz=dz;
  spiderH->phi=euler.phiSpider()*180/PI;
  spiderH->theta=euler.thetaSpider()*180/PI;
  spiderH->gamma=euler.gammaSpider()*180/PI;
  strncpy(spiderH->name,name,79);

  if (swap) swapint((char *)spiderH,211);
  fwrite(spiderH,sizeof(SPIDERH),1,out);
  if (swap) swapint((char *)spiderH,211);

  char *t=(char *)calloc(hl-sizeof(SPIDERH),1);
  fwrite(t,hl-sizeof(SPIDERH),1,out);
  free(t);

  getDataRO();

  if (swap) swapint((char *)rdata,nx*ny*nz);
  fwrite(rdata,nx*ny,nz*sizeof(float),out);
  if (swap) swapint((char *)rdata,nx*ny*nz);

  file_unlock(out);
  fclose(out);
  doneData();
  return 0;
}

int EMData::readLST(const char *filespec,int n,int nodata,const char *xt) {
  char s[MAXPATHLEN+10],t[MAXPATHLEN+1],*c;
  FILE *in = fopen(filespec,"r");
  if (!in) { printf("'%s' not found\n",filespec); return (-3); }
  int i,nn;

  if (n<0) n=0;
  t[0]=0;
  for (i=0; i<=n; i++) {
    if (!fgets(s,MAXPATHLEN+10,in)) break;
    if (s[0]=='#') i--;
    else sscanf(s," %d %s",&nn,t);	// we scan every line, so the filespec only needs to be present once
  }
  fclose(in);
  if (i<=n) return (-1);
//printf("readlst %s(%d) -> %s(%d)\n",filespec,n,t,nn);

  if (xt) {
    char tmp[6];
    strcpy(tmp,t+strlen(t)-4);
    strcpy(t+strlen(t)-4,xt);
    strcat(t,tmp);
  }

  // Now we must reference the file to the directory it's in if necessary
  strcpy(s,filespec);
  c=strrchr(s,'/');
  if (c && t[0]!='/') {
    c++;
    *c=0;
    strcat(s,t);
    return readImage(s,nn,nodata);
  }

  return readImage(t,nn,nodata);
}

int EMData::writeLST(const char *filespec,int n,const char *reffile, int refn,const char *comment) {
  if (!reffile) reffile=Path();
  if (refn<0) refn=pathnum;

  if (n==-1 || n==0) {
    char s[1024];
    sprintf(s,"%d\t%s",refn,reffile);
    if (comment) sprintf(s+strlen(s),"\t%s\n",comment);
    else strcat(s,"\n");

    FSsend(FS_LST,strlen(s)+1,filespec,s,n);
    return 0;
  }

  printf("Sorry, writeLST(n) not supported yet\n");
  exit(1);
  /*
    char s[MAXPATHLEN+10],t[MAXPATHLEN+1];
    char *tn=tmpnam();
    FILE *in = fopen(filespec,"r");
    FILE *out = fopen(
    if (!in) { printf("'%s' not found\n",filespec); return (-3); }
    int n;

    for (i=0; i<=n; i++) {
    if (!fgets(s,MAXPATHLEN+10,in)) break;
    if (s[0]=='#') i--;
    else sscanf(s," %d %s",&n,t);	// we scan every line, so the filespec only needs to be present once
    }
    if (i<=n) return (-1);
  */
  return 0;
}

// update statistics
void EMData::calcHist(float *hist,int n,float histmin,float histmax,int add) {
  int i,j,k,p0,p1,di;
  float u,v,w,norm=0;
  static int prime[] = { 1,3,7,11,17,23,37,59,127,253,511 } ;

  if (histmin==histmax) { histmin=Min(); histmax=Max(); } 

  if (!add) for (i=0; i<n; i++) hist[i]=0;


  // decide on the sampling based on how big the data set is
  if (nx*ny*nz<300000) p0=p1=0;
  else if (nx*ny*nz<2000000) { p0=2; p1=3; }
  else if (nx*ny*nz<8000000) { p0=4; p1=6; }
  else { p0=7; p1=9; }

  // real images don't need as much sampling
  if (!(flags&EMDATA_COMPLEX)&&p0>0) { p0++; p1++; }

  for (k=p0; k<=p1; k++) {
    if (flags&EMDATA_COMPLEX) di=prime[k]*2; else di=prime[k];
    norm+=nx*ny*nz/(float)di;
    w=(float)n/(histmax-histmin);
    for (i=nx*ny*nz-di; i>=0; i-=di) {
      // This version splits the histogram data up into 2 bins
      /*		v=(rdata[i]-histmin)*w;	
			u=v-floor(v);
			j=(int)floor(v);
			if (j<0||j>n-1) continue;
			hist[j+1]+=u;
			hist[j]+=1.0-u;*/

      // This one bins each pixel in only 1 bin		
      j=(int)round((rdata[i]-histmin)*w);
      if (j<0||j>n-1) continue;
      hist[j]+=1.0;
    }
  }

  for (i=0; i<n; i++) hist[i]=hist[i]/norm;

}

// This normalizes each horizontal line in the image individually
// mode=0 -> sum of the values on a line = 1
void EMData::xNormalize(int mode) {
  int x,y;

  getData();
  for (y=0; y<ySize(); y++) {
    double f=0,m=0,g;
    for (x=0; x<xSize(); x++) { g=rdata[x+y*xSize()]; f+=g; }
    f/=xSize();
    if (f<=0) f=1.0;	// not much else we can do
    for (x=0; x<xSize(); x++) rdata[x+y*xSize()]/=f;
  }

  doneData();
}

// This will shift the zero value of 'this' to the average value inside a passed
// 1/0 mask image. If sigma is 1, it will also adjust the standard deviation of the mean to 1
void EMData::maskNormalize(EMData *mask,int sigmatoo) {
  double av=0,sig=Sigma();
  float *d = getData();
  float *m = mask->getDataRO();
  int n=nx*ny*nz;
  int i,norm=0;

  for (i=0; i<n; i++) if (m[i]) { av+=d[i]; norm++; }
  if (!norm) { doneData(); av=edgeMean();	getData(); }// The passed mask didn't exist
  else av/=(double)norm;

  if (sigmatoo) {
    for (i=0; i<n; i++) d[i]=(d[i]-av)/sig;
  } else {
    for (i=0; i<n; i++) d[i]=(d[i]-av);
  }
  doneData();
  mask->doneData();
};

// This will add a given number of 'shells' around an existing 2/3d mask
// not exceptionally efficient, but effective, avoids a 1 pixel border at the edge of the image
void EMData::addMaskShell(int shells) {
  if (ny==1) error(ERR_EXIT,"Tried to addMaskShell to 1d image");
  float *d=getData(),k;
  if (nz==1) {
    int i,x,y;
	
    for (i=0,k=.99999; i<shells; i++,k-=.00001) {
      for (y=1; y<ny-1; y++) {
	for (x=1; x<nx-1; x++) {
	  int j=x+y*nx;
	  if (!d[j] && (d[j-1]>k || d[j+1]>k || d[j+nx]>k || d[j-nx]>k)) d[j]=k;
	}
      }
    }
  }
  else {
    int i,x,y,z;
	
    for (i=0,k=.99999; i<shells; i++,k-=.00001) {
      for (z=1; z<nz-1; z++) {
	for (y=1; y<ny-1; y++) {
	  for (x=1; x<nx-1; x++) {
	    int j=x+y*nx+z*nx*ny;
	    if (!d[j] && (d[j-1]>k || d[j+1]>k || d[j+nx]>k || d[j-nx]>k || d[j-nx*ny]>k || d[j+nx*ny]>k)) d[j]=k;
	  }
	}
      }
    }
  }

  for (int i=0; i<nx*ny*nz; i++) if (d[i]) d[i]=1.0; else d[i]=0.0;
  doneData();
  return;
}

void EMData::leastSquareNormalizeTo(EMData *to, float low_thresh, float high_thresh, float* scale, float* shift) {
  float *rawp=getData();	// y
  float *refp=to->getDataRO();	// x	
  float sx=0, sy=0, sxx=0, syy=0, s=0;
  float a, b=0, siga, sigb, stt=0;
  int i;
  // ignore the weight
  for(i=0;i<nx*ny*nz;i++){
    if( refp[i]>=low_thresh && refp[i]<=high_thresh && refp[i]!=0 ) { // assume 0s are masked area
      s++;
      sx += refp[i];
      sy += rawp[i];
    }
  }
  float sxos=sx/s, t;
  for(i=0;i<nx*ny*nz;i++){
    if( refp[i]>=low_thresh && refp[i]<=high_thresh && refp[i]!=0 ) {
      t=refp[i]-sxos;
      stt +=t*t;
      b+=t*rawp[i];
    }
  }

  to->doneData();
	
  b/=stt;
  a=(sy-sx*b)/s;
  siga=sqrt((1.0+sy*sy/(s*stt))/s);
  sigb=sqrt(1.0/stt);

  if(scale) *scale=1/b;
  if(shift) *shift=-a/b;

  for(i=0;i<nx*ny*nz;i++){
    rawp[i]=(rawp[i]-a)/b;
  }
  doneData();	
  update();
}

float EMData::Skewness() {
  double mean=Mean();
  double sigma=Sigma();
  getDataRO();

  double k=0;
  for (int i=0; i<nx*ny*nz; i++) k+=pow((rdata[i]-mean)/sigma,3.0);
  doneData();

  return k/(nx*ny*nz);
}

float EMData::Kurtosis() {
  double mean=Mean();
  double sigma=Sigma();
  getDataRO();

  double k=0;
  for (int i=0; i<nx*ny*nz; i++) k+=pow((rdata[i]-mean)/sigma,4.0);
  doneData();

  return k/(nx*ny*nz)-3.0;
}

void EMData::realupdate() {
  int i,j,k,l,di,n=0;
  double u,v,w;
  double m,s,t,m2,s2;

  if (nx==0 || ny==0 || nz==0) { setSize(2,2,2); zero(); }

  flags&=~EMDATA_NEEDUPD;

  max=-1.0e20;
  min=-max;
  m=m2=0;
  s=s2=0;

  // getDataRO();		//dangerous not to, but causes problems

  //for (i=0; i<200; i++) hist[i]=0;
  if ((flags&EMDATA_COMPLEX)&& !(flags&EMDATA_RI)) di=2; else di=1;

  // pass 1, statistics
  for (j=0,i=0; j<nz; j++) {
    for (k=0; k<ny; k++) {
      for (l=0; l<nx; l+=di,i+=di) {
	v=rdata[i];
	//			if (!(finite(v))) printf("wrong number at x,y,z=%d,%d,%d = %f\n",l,k,j,v);
	//	if (v>1e5) { printf(" %d,%d=%f\n",i%nx,i/nx,v); continue; }
	if (v>max) { max=v; maxloc=i; }
	if (v<min) { min=v; minloc=i; }
	m+=v;
	s+=v*v;
	if (v!=0) n++;

	/*			if (k<4 || l<4 || k>ny-5 || l>nx-5) {
				m2+=v;
				s2+=v*v;
				}*/
      }
    }
  }
  sumsq=s*di;
  mean=m*di/(double)(nx*ny*nz);
  sigma=s*di/(double)(nx*ny*nz)-mean*mean;
  if (sigma<0) sigma=0;	// in case of roundoff errors for v small sigma
  //if (fabs(sigma)<1.0e-10) sigma=0;		// could be that a 'zero' is a bit off due to rounding
  sigma=sqrt(sigma);

  if (n==0) n=1;
  mean2=m*di/(double)(n);
  sigma2=s*di/(double)(n)-mean2*mean2;
  if (sigma2<0) sigma2=0;	// in case of roundoff errors for v small sigma
  sigma2=sqrt(sigma2);

  //histmax=max;
  //histmin=min;

  //	printf("!!! %d %g %g %g %g\n",(nx*ny*nz),min,max,(float)m,(float)s);
  //	printf("!!! %d %g %g %g %g\n",(nx*ny*nz),min,max,mean,sigma);

  if (!finite(mean) || !finite(sigma)) {
    char st[120];
    sprintf(st,"ERROR - Bad statistics on this data set! %d %g %g",(nx*ny*nz),min,max);
    //	for (i=0; i<110; i++) printf("%5.3g\t",rdata[i+6050]);
    //	printf("\n");
    error(st,ERR_ERROR);
    zero();
    //	kill(0,16);
    return;
  }

  //printf("ru min %f max %f mean %f sigma %f\n",min,max,mean,sigma);

  //if (histmax-mean>sigma*3.0) histmax=mean+sigma*3.0;
  //if (mean-histmin>sigma*3.0) histmin=mean-sigma*3.0;
  //if (histmax==histmin) histmax+=.0001;
  //if (max==min) max+=.0001;

  // pass 2, histogram
  //calcHist(histmin ,histmax);

  return;
}

// 2d only
float EMData::edgeMean() {
  int i,j,k,di=0;
  float edgemean=0;

  getData();
  // this finds the average around the (1 pixel) edge of the image

  if (nz==1) {
    for (i=0,j=(ny-1)*nx; i<nx; i++,j++) edgemean+=rdata[i]+rdata[j]; 
    for (i=0, j=nx-1; i<nx*ny; i+=nx,j+=nx) edgemean+=rdata[i]+rdata[j];
    edgemean/=(double)(nx*2+ny*2);
  }
  else {
    if(nx==ny && nx==nz*2-1){ // the special icos half maps
      for (j=(nx*ny*(nz-1)); j<nx*ny*nz;j++,di++) edgemean+=rdata[j];
    }
    else {
      for (i=0,j=(nx*ny*(nz-1)); i<nx*ny; i++,j++,di++) edgemean+=rdata[i]+rdata[j];
    }	
    for (k=1; k<nz-1; k++) {
      for (i=0; i<nx; i++,di++) {
	edgemean+=rdata[i+k*nx*ny]+rdata[i+nx*(ny-1)+k*nx*ny];
      }
    }
    for (k=1; k<nz-1; k++) {
      for (i=1; i<ny-1; i++,di++) {
	edgemean+=rdata[i*nx+k*nx*ny]+rdata[i*nx+nx-1+k*nx*ny];
      }
    }
    //	printf("ni=%d\n",di);
    edgemean/=(double)(di*2);
  }

  doneData();

  return edgemean;
}

//circular edgeMean
float EMData::cirMean() {
  static int busy=0;
  static EMData *mask=NULL;
  int i,n;
  float r,*d;

  while (busy) ;
  busy=1;
  if (!mask || mask->xSize()!=nx || mask->ySize()!=ny || mask->zSize()!=nz) {
    if (!mask) mask=new EMData;
    mask->setSize(nx,ny,nz);
    mask->zero();
    mask->applyMask(ny/2-2,7);
    n=0;
    d=mask->getDataRO();
    for (i=0; i<nx*ny*nz; i++) if (d[i]) n++;
    mask->doneData();
    mask->multConst(1.0/(float)n);
    //mask->writeMRC("cir.mrc");
  }
  r=dot(mask);
  busy=0;
  return r;
}

// Tries to locate the statistical center of the data, sort
// of like the mode value of the image. Obviously this will
// work poorly if the data has a bimodal histogram
float EMData::dCenter() {
  float cen=Mean();	//start at the mean value
  double sum,norm,ds=Sigma()/2.0;
  float *d;
  int n=nx*ny*nz,i;

  d=getDataRO();

  while (ds>Sigma()/20.0) {
    sum=norm=0;
    for (i=0; i<n; i++) {
      if (fabs(d[i]-cen)<ds) { sum+=d[i]; norm+=1.0; }
    }
    sum/=norm;
    if (fabs(sum-cen)<Sigma()/1000.0) ds*=.5;
    //	printf("%f\t%f\t%1.0f\n",sum,ds,norm);
    cen=sum;
  }

  doneData();

  return cen;
}

// This calculates sigma above and below the mean and returns the difference
float EMData::sigDiff() {
  float *d = getDataRO();
  double sigup=0,sigdown=0,nup=0,ndown=0,m=Mean();
  int i;

  for (i=0; i<nx*ny*nz; i++) {
    if (d[i]>m) { sigup+=SQR(d[i]-m); nup++; }
    else { sigdown+=SQR(m-d[i]); ndown++; }
  }
  sigup=sqrt(sigup/nup);
  sigdown=sqrt(sigdown/ndown);

  doneData();

  //printf("%f\t%f\t%f\n",sigup,sigdown,Sigma());
  return fabs(sigup-sigdown)/Sigma();
}

// general parameter set/gets
int EMData::setSize(int x,int y, int z) {
  if (waitReady(0)) return -1; 

  if (x<=0) x=1;
  if (y<=0) y=1;
  if (z<=0) z=1;
  if (y==1&&z!=1) { y=z; z=1; }
  if (x==1&&y!=1) { x=y; y=1; }
  nx=x;
  ny=y;
  nz=z;
  if (flags&EMDATA_NODATA) {
    if (rdata) free(rdata);
    rdata=NULL;
  }
  else rdata=(float *)Srealloc(rdata,x*y*z*sizeof(float),1);
  update();
  flags&= ~EMDATA_SWAPPED;
  if (supp) { free(supp); supp=NULL; }
  return 0;
}

// Access to raw data
float *EMData::getData() {	// This returns a pointer to the raw float data
  //if (flags&EMDATA_BUSY || rocount>0 || !rdata) return NULL;
  waitReady(0);
  if (flags&EMDATA_SWAPPED) swapin();
  flags|=EMDATA_BUSY;
  return rdata;
}

// Read only access to raw data
float *EMData::getDataRO()	// This returns a pointer to the raw float data
{
  //if (flags&EMDATA_BUSY || !rdata) return NULL;
  waitReady(1);
  if (flags&EMDATA_SWAPPED) swapin();
  rocount++;
  return rdata;
}

void EMData::doneData()	// MUST be called when the data is no longer being
  // modified. Another getData while the data is
  // out will return NULL
{
  if (flags&EMDATA_BUSY) {
    flags&=(~EMDATA_BUSY);
    update();
  }
  else {
    rocount--;
    if (rocount<0) rocount=0;
  }

  return;
}

int EMData::Busy()			// Returns 1 if data is checked out
{
  if (flags&EMDATA_BUSY || rocount) return 1; else return 0;
}

// Waits for data to become available
int EMData::waitReady(int ro) 
{
  /*
    if (ro) {
    while (flags&EMDATA_BUSY) sleep(1);
    }
    else {
    while ((flags&EMDATA_BUSY) || (rocount)) sleep(1);
    }
  */

  // for debugging only
  if (ro) {
    while (flags&EMDATA_BUSY) {
      printf("Busy 1 (%p)\n",this);
#ifdef ALPHA
      sleep(1);
#else
      usleep(100);
#endif
    }
  }
  else {
    if ((flags&EMDATA_BUSY) || (rocount)) {
      printf("Busy 0.%d (%p)\n",rocount,this);
      while ((flags&EMDATA_BUSY) || (rocount)) {
#ifdef ALPHA
	sleep(1);
#else
	usleep(100);
#endif
      }
      printf("Busy done 0.%d (%p)\n",rocount,this);	
    }
  }

  return 0;
}

// Rendering
int EMData::renderAmp8( unsigned char * data , int x0 , int y0, int ixsize, 
			int iysize, int bpl, float scale, int mingray, int maxgray,float renMin,float renMax) {
  int i,j,k,l,ll,ymin=0,xmin,ymax,lmax,xsize,ysize;
  int dsx,dsy; 	// used for integer scalings
  int remx,remy,br,addi,addr;	// used for non integer scaling
  float x,y,gs,t,rm;
  unsigned char *tmp;

  if (flags&EMDATA_COMPLEX) ri2ap();
  if (renMax<=renMin) renMax=renMin+.01;
  rm=renMin;

  getDataRO();
  scale=1.0/scale;
  ysize=iysize;
  xsize=ixsize;
  if (iysize*scale>ny) ymin=(int)(iysize-ny/scale);
  gs=(maxgray-mingray)/(renMax-renMin);
  if (renMax<renMin) { gs=0;  rm=1.0e20; }
  if (scale==floor(scale)) { dsx=(int)scale; dsy=(int)(scale*nx); } else dsx=-1;
  if (dsx==-1) { remx=remy=0; addi=(int)floor(scale); addr=(int)(100000.0*(scale-floor(scale))); }
  if (x0<0) { xmin=(int)(-x0/scale); xsize-=(int)floor(x0/scale); x0=0; } 
  else { xmin=0; }
  if ((xsize-xmin)*scale>(nx-x0)) xsize=(int)((nx-x0)/scale+xmin);
  if (y0<0) { ymax=(int)(ysize+y0/scale-1); ymin+=(int)floor(y0/scale); y0=0; } else ymax=ysize-1;

  if (xmin<0) xmin=0;
  if (ymin<0) ymin=0;
  if (xsize>ixsize) xsize=ixsize;
  if (ymax>iysize) ymax=iysize;

  lmax=nx*ny-1;

  // complex version very similar to real version
  if (flags&EMDATA_COMPLEX) {
    // integer scalings are fast
    if (dsx!=-1) {
      for (j=ymax,l=y0*nx; j>=ymin; j--) {
	for (i=xmin,ll=x0; i<xsize; i++,ll+=dsx) {
	  if (l+ll>lmax || ll>=nx-2) break;
	  // +2 here is to prevent displaying qx=0 twice
	  if (ll>=nx/2) {
	    if (l>=(ny-scale)*nx) k=2*(ll-nx/2)+2;
	    else k=2*(ll-nx/2)+l+2+nx;
	  }
	  else k=nx*ny-(l+2*ll)-2;
	  t=rdata[k];
	  if (t<=rm) k=mingray;	// renMin, but deals with renMin>renMax
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  data[i+j*bpl]=k;
	}
	l+=dsy;
      }
    }
    // non integer scalings a bit slower
    else {
      remy=10; // a small fixed fraction
      for (j=ymax,l=y0*nx; j>=ymin; j--) {
	br=l;
	remx=10;
	for (i=xmin,ll=x0; i<xsize-1; i++) {
	  if (l+ll>lmax || ll>=nx-2) break;
	  if (ll>=nx/2) {
	    if (l>=(ny*nx-nx)) k=2*(ll-nx/2)+2;
	    else k=2*(ll-nx/2)+l+2+nx;
	  }
	  else k=nx*ny-(l+2*ll)-2;

	  t=rdata[k];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  data[i+j*bpl]=k;
	  ll+=addi;
	  remx+=addr;
	  if (remx>100000) { remx-=100000; ll++; }
	}
	l=br+addi*nx;
	remy+=addr;
	if (remy>100000) { remy-=100000; l+=nx; }
      }
    }
  }
  else {
    // integer scalings are fast
    if (dsx!=-1) {
      for (j=ymax,l=x0+y0*nx; j>=ymin; j--) {
	br=l;
	for (i=xmin; i<xsize; i++,l+=dsx) {
	  if (l>lmax) break;
	  t=rdata[l];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  data[i+j*bpl]=k;
	}
	l=br+dsy;
      }
    }
    // non integer scalings a bit slower
    else {
      remy=10;
      for (j=ymax,l=x0+y0*nx; j>=ymin; j--) {
	br=l;
	remx=10;
	for (i=xmin; i<xsize; i++) {
	  if (l>lmax) break;
	  t=rdata[l];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  data[i+j*bpl]=k;
	  l+=addi;
	  remx+=addr;
	  if (remx>100000) { remx-=100000; l++; }
	}
	l=br+addi*nx;
	remy+=addr;
	if (remy>100000) { remy-=100000; l+=nx; }
      }
    }
  }

  doneData();
  return 1;
}

int EMData::renderAmp24( unsigned char * data , int x0 , int y0, int ixsize, 
			 int iysize, int bpl, float scale, int mingray, int maxgray,
			 float renMin,float renMax,void *ref,void cmap(void *,int ,unsigned char *)) {
  int i,j,k,kk,l,ll,ymin=0,xmin,ymax,lmax,xsize,ysize;
  int dsx,dsy; 	// used for integer scalings
  int remx,remy,br,addi,addr;	// used for non integer scaling
  float x,y,gs,t,rm;
  unsigned char *tmp;
  unsigned char tri[3];

  if (flags&EMDATA_COMPLEX) ri2ap();
  if (renMax<=renMin) renMax=renMin+.01;
  rm=renMin;

  getDataRO();
  scale=1.0/scale;
  ysize=iysize;
  xsize=ixsize;
  if (iysize*scale>ny) ymin=(int)(iysize-ny/scale);
  gs=(maxgray-mingray)/(renMax-renMin);
  if (renMax<renMin) { gs=0;  rm=1.0e20; }
  if (scale==floor(scale)) { dsx=(int)scale; dsy=(int)(scale*nx); } else dsx=-1;
  if (dsx==-1) { remx=remy=0; addi=(int)floor(scale); addr=(int)(100000.0*(scale-floor(scale))); }
  if (x0<0) { xmin=(int)(-x0/scale); xsize-=(int)floor(x0/scale); x0=0; } 
  else { xmin=0; }
  if ((xsize-xmin)*scale>(nx-x0)) xsize=(int)((nx-x0)/scale+xmin);
  if (y0<0) { ymax=(int)(ysize+y0/scale-1); ymin+=(int)floor(y0/scale); y0=0; } else ymax=ysize-1;

  if (xmin<0) xmin=0;
  if (ymin<0) ymin=0;
  if (xsize>ixsize) xsize=ixsize;
  if (ymax>iysize) ymax=iysize;

  lmax=nx*ny-1;

  // complex version very similar to real version
  if (flags&EMDATA_COMPLEX) {
    // integer scalings are fast
    if (dsx!=-1) {
      for (j=ymax,l=y0*nx; j>=ymin; j--) {
	for (i=xmin,ll=x0; i<xsize; i++,ll+=dsx) {
	  if (l+ll>lmax || ll>=nx-2) break;
	  // +2 here is to prevent displaying qx=0 twice
	  if (ll>=nx/2) {
	    if (l>=(ny-scale)*nx) kk=2*(ll-nx/2)+2;
	    else kk=2*(ll-nx/2)+l+2+nx;
	  }
	  else kk=nx*ny-(l+2*ll)-2;
	  t=rdata[kk];
	  if (t<=rm) k=mingray;	// renMin, but deals with renMin>renMax
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  tri[0]=k;
	  cmap(ref,kk,tri);
	  data[i*3+j*bpl]=tri[0];
	  data[i*3+1+j*bpl]=tri[1];
	  data[i*3+2+j*bpl]=tri[2];
	}
	l+=dsy;
      }
    }
    // non integer scalings a bit slower
    else {
      remy=10; // a small fixed fraction
      for (j=ymax,l=y0*nx; j>=ymin; j--) {
	br=l;
	remx=10;
	for (i=xmin,ll=x0; i<xsize-1; i++) {
	  if (l+ll>lmax || ll>=nx-2) break;
	  if (ll>=nx/2) {
	    if (l>=(ny*nx-nx)) kk=2*(ll-nx/2)+2;
	    else kk=2*(ll-nx/2)+l+2+nx;
	  }
	  else kk=nx*ny-(l+2*ll)-2;

	  t=rdata[kk];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  tri[0]=k;
	  cmap(ref,kk,tri);
	  data[i*3+j*bpl]=tri[0];
	  data[i*3+1+j*bpl]=tri[1];
	  data[i*3+2+j*bpl]=tri[2];
	  ll+=addi;
	  remx+=addr;
	  if (remx>100000) { remx-=100000; ll++; }
	}
	l=br+addi*nx;
	remy+=addr;
	if (remy>100000) { remy-=100000; l+=nx; }
      }
    }
  }
  else {
    // integer scalings are fast
    if (dsx!=-1) {
      for (j=ymax,l=x0+y0*nx; j>=ymin; j--) {
	br=l;
	for (i=xmin; i<xsize; i++,l+=dsx) {
	  if (l>lmax) break;
	  t=rdata[l];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  tri[0]=k;
	  cmap(ref,l,tri);
	  data[i*3+j*bpl]=tri[0];
	  data[i*3+1+j*bpl]=tri[1];
	  data[i*3+2+j*bpl]=tri[2];
	}
	l=br+dsy;
      }
    }
    // non integer scalings a bit slower
    else {
      remy=10;
      for (j=ymax,l=x0+y0*nx; j>=ymin; j--) {
	br=l;
	remx=10;
	for (i=xmin; i<xsize; i++) {
	  if (l>lmax) break;
	  t=rdata[l];
	  if (t<=rm) k=mingray;
	  else if (t>=renMax) k=maxgray;
	  else {
	    k=(int)(gs*(t-renMin));
	    k+=mingray;
	  }
	  tri[0]=k;
	  cmap(ref,l,tri);
	  data[i*3+j*bpl]=tri[0];
	  data[i*3+1+j*bpl]=tri[1];
	  data[i*3+2+j*bpl]=tri[2];
	  l+=addi;
	  remx+=addr;
	  if (remx>100000) { remx-=100000; l++; }
	}
	l=br+addi*nx;
	remy+=addr;
	if (remy>100000) { remy-=100000; l+=nx; }
      }
    }
  }

  doneData();
  return 1;
}

int EMData::renderPha24( unsigned char * data , int x , int y, int xsize, int ysize, 
			 int bpl, float scale, float renMin, float renMax)
{
  return -1;
}

//
void EMData::ri2ap() {		// converts real/imaginary to amp/pha (complex only)
  int i;
  float f;
  float *data;

  if (!(flags&EMDATA_COMPLEX) || !(flags&EMDATA_RI)) { /* printf("already ap\n"); */  return; }
  data=getData();

  for (i=0; i<nx*ny*nz; i+=2) {
    f=hypot(data[i],data[i+1]);
    if (data[i]==0 && data[i+1]==0) data[i+1]=0;
    else data[i+1]=atan2(data[i+1],data[i]);
    data[i]=f;
  }

  doneData();
  flags&=~EMDATA_RI;
  update();
}

void EMData::ap2ri() {		// the opposite
  int i;
  float f,*data;

  if (!(flags&EMDATA_COMPLEX) || (flags&EMDATA_RI)) return;
  data=getData();

  for (i=0; i<nx*ny*nz; i+=2) {
    f=data[i]*sin(data[i+1]);
    data[i]=data[i]*cos(data[i+1]);
    data[i+1]=f;
  }

  doneData();
  flags|=EMDATA_RI;
  update();
}

// real images only
void EMData::invert() {
  int i,di=1;
  float f,*data;

  if ((flags&EMDATA_COMPLEX)) return;
  data=getData();

  for (i=0; i<nx*ny*nz; i+=di) {
    data[i]=-data[i];
  }

  doneData();
  update();
}

// least squares fit returns slope, m, and intercept, b
void lsqfit(int n,float *dx,float *dy,float *m,float *b) {

}

// 'noisy' should be noisy, 'this' should be less noisy
// May still work in some cases if this is not true
// least-squares to determine optimal normalization
// invert implies that 'this' is noisy rather than 'noisy'
void EMData::normalizeTo(EMData *noisy,int keepzero,int invert,float *mult,float *add) {
  float ret=0,*dx,*dy;
  int i,n=nx*ny*nz;
  float m,b;

  if (nx!=noisy->xSize() || ny!=noisy->ySize() || nz!=noisy->zSize()) { error(ERR_WARN,"NormalizeTo on different sized images"); return; }

  if (!invert) {
    dx=getDataRO();
    dy=noisy->getDataRO();

    lsqfit(n,dx,dy,&m,&b,1);

    doneData();
    noisy->doneData();
  }

  // If m<0 this implies quite strongly that 'this' is noisy
  // Rather than simply fail, we try inverting the least-squares
  if (invert || m<0) { 
    dx=getDataRO();
    dy=noisy->getDataRO();

    lsqfit(n,dy,dx,&m,&b,1);

    doneData();
    noisy->doneData();

    if (m==0) m=1e-20;
    b=-b/m;
    m=1.0/m;

    if (m<0) { b=0; m=1.0; }	// Ok. Lost cause, give up
  }


  multConst(m);
  if (keepzero) {
    getData();
    for (i=0; i<nx*ny*nz; i++) if (rdata[i]) rdata[i]+=b;
    doneData();
  }
  else addConst(b);

  if (mult) *mult=m;
  if (add) *add=b;
}

void EMData::normalize() {
  int i,di=1;
  float f,*data;
  float nor,s;

  if ((flags&EMDATA_COMPLEX) && !(flags&EMDATA_RI)) di=2;
  s=Sigma();			// need pointer for goodf
  if ( !goodf(&s)) { zero(); return; }
  if (s==0 ) return;
  data=getData();

  // this normalizes to sigma excluding pixels which are exactly zero
  //s=Sigma2();	
  //if (s>0 && goodf(&s)) nor=s;	
  //else nor=Sigma();

  // The above may be risky, we'll return to the standard normalization
  nor=Sigma();

  //printf("%f %f %f\n",nor,Sigma(),Sigma2());

  for (i=0; i<nx*ny*nz; i+=di) {
    data[i]=(data[i]-Mean())/nor;
  }

  doneData();
  update();
}

// real only
void EMData::normalizeMax() {
  int i,di=1;
  float f,*data;
  float sc,d;

  if ( !finite(Sigma())) { zero(); return; }
  if (Sigma()==0 ) return;
  sc=(Max()-Min())/2.0;
  d=(Max()+Min())/2.0;

  data=getData();

  for (i=0; i<nx*ny*nz; i+=di) {
    data[i]=(data[i]-d)/sc;
  }

  doneData();
  update();
}

void EMData::edgeNormalize(int cir) {
  int i,di=1;
  float f,*data,em,s;

  if ((flags&EMDATA_COMPLEX) && !(flags&EMDATA_RI)) di=2;
  if (Sigma()==0 || !finite(Sigma())) return;

  if (cir) em=cirMean();
  else em=edgeMean();
  s=Sigma();
  data=getData();

  for (i=0; i<nx*ny*nz; i+=di) {
    data[i]=(data[i]-em)/s;
  }

  doneData();
  update();
}

// makes plans for both forward and backwards transforms
rfftwnd_plan EMData::makeplan(int dim,int nx,int ny, int nz,int fw,int measure) {
  int i;

  // see if we've got one already
  for (i=0; i<plans; i++) {
    //	printf("%d. %d %d   %d %d   %d %d   %d %d   %d %d\n",i,nplan[i][0],nz,nplan[i][1],ny,nplan[i][2],nx,dimplan[i],dim,fwplan[i],fw);
    if (nplan[i][0]==nz && nplan[i][1]==ny && nplan[i][2]==nx 
	&& dimplan[i]==dim && fwplan[i]==fw) break;
  }
  if (i!=plans) return plan[i];

  //printf("New plan %d %d %d\n",nx,ny,nz);
  while (dimplan[0]==-1) ;	// make sure we don't make 2 plans at once


  if (plan[5]) rfftwnd_destroy_plan(plan[5]);
  for (i=5; i>0; i--) {
    plan[i]=plan[i-1];
    dimplan[i]=dimplan[i-1];
    fwplan[i]=fwplan[i-1];
    nplan[i][0]=nplan[i-1][0];
    nplan[i][1]=nplan[i-1][1];
    nplan[i][2]=nplan[i-1][2];
  }
  dimplan[0]=-1;

  if (measure) measure=FFTW_MEASURE; else measure=FFTW_ESTIMATE;

  nplan[0][0]=nz;
  nplan[0][1]=ny;
  nplan[0][2]=nx;
  fwplan[0]=fw;
  if (fw) plan[0]=rfftwnd_create_plan(dim, nplan[0]+(3-dim),FFTW_REAL_TO_COMPLEX,
				      measure|FFTW_OUT_OF_PLACE|FFTW_USE_WISDOM|FFTW_THREADSAFE);
  else plan[0]=rfftwnd_create_plan(dim, nplan[0]+(3-dim),FFTW_COMPLEX_TO_REAL,
				   measure|FFTW_IN_PLACE|FFTW_USE_WISDOM|FFTW_THREADSAFE);
  dimplan[0]=dim;
  if (plans<6) plans++;
  return plan[0];
}

// vertical correlation function with various relative shifts
EMData *EMData::vertACF(int maxdy) {
  EMData *ret = new EMData;

  if (maxdy<=1) maxdy=ySize()/8;
  ret->setSize(xSize(),maxdy,1);

  int x,y,yy;
  float *d = getDataRO();
  float *dd = ret->getData();
  float dot;

  for (x=0; x<xSize(); x++) {
    for (y=0; y<maxdy; y++) {
      dot=0;
      for (yy=maxdy; yy<ySize()-maxdy; yy++) {
	dot+=d[x+(yy+y)*xSize()]*d[x+(yy-y)*xSize()];
      }
      dd[x+y*xSize()]=dot;
    }
  }
  doneData();
  ret->doneData();

  return ret;
}

// makes a square image containing the Radon transform of the current image
// input should also be square
EMData *EMData::doRadon() {
  EMData *ret = new EMData;
  EMData *c;
  float *d,*dd;

  if (xSize()!=ySize() || zSize()!=1) {
    error(ERR_WARN,"Radon transform only works on square images");
    return NULL;
  }

  ret->setSize(xSize(),xSize(),1);
  ret->zero();
  dd=ret->getData();
  if (!Parent()) c=copy(); else c=this;

  c->setTAlign(0,0);
  for (int i=0; i<xSize(); i++) {
    c->setRAlign(PI*2.0*i/xSize());
    c->rotateAndTranslate(1.0,0,0,0,xSize()/2-1);
    d=c->getDataRO();
    for (int y=0; y<xSize(); y++) {
      for (int x=0; x<xSize(); x++) {
	if (SQR(x-xSize()/2)+SQR(y-xSize()/2)>xSize()*xSize()/4) continue;
	dd[i+y*xSize()]+=d[x+y*xSize()];
      }
    }
    c->doneData();
  }

  ret->doneData();
  if (!Parent()) delete c;
  return ret;
}

EMData *EMData::doFFT()	{	// obvious, note that result is R,I not A,P
  EMData *dat;
  float *d,re,im,norm;
  int i,j,k,l,ii,jj,dim;
  char s[80],*t;
  mrcH *newh;
  rfftwnd_plan cplan;

  if (flags&EMDATA_COMPLEX) return this;	// already complex, just return

  if (nx!=npfa(nx) || (ny!=npfa(ny)&&ny!=1) || (nz!=npfa(nz)&&nz!=1)) { error("Invalid FFT size!",ERR_ERROR); return NULL; }

  getData();	// we open for writing so nobody else does a fft at the same time

  //flags|=EMDATA_NEWFFT;		// disable caching

  if (fft) {
    if (!(flags&EMDATA_NEWFFT)) { 
      i=flags; 
      doneData(); 
      flags=i&~EMDATA_BUSY;
      //		printf("old ok\n"); 
      return fft; 
    }
    fft->setSize(nx+2,ny,nz);
    dat=fft;
    //	printf("old fft\n");
  }
  else {
    //	printf("new fft\n");
    dat=copyHead();
    //	sprintf(s,"FFT %s",name);
    //	dat->setName(s);
    fft=dat;
    fft->setSize(nx+2,ny,nz);
  }


  if (ny==1) dim=1;
  else if (nz==1) dim=2;
  else dim=3;

  cplan=makeplan(dim,nx,ny,nz,1);

  // 1d
  if (ny==1) {
    j=0;
    k=1;
    d=dat->getData();
    rfftwnd_one_real_to_complex(cplan,(fftw_real *)rdata,(fftw_complex *)d);
    dat->doneData();
  }
  // 2d
  else if (nz==1) {
    d=dat->getData();

    rfftwnd_one_real_to_complex(cplan,(fftw_real *)rdata,(fftw_complex *)d);

    // This 'rotates' the data vertically by ny/2
    l=ny/2*(nx+2);
    for (i=0; i<ny/2; i++) {
      for (j=0; j<nx+2; j++) {
	k=j+i*(nx+2);
	re=d[k];
	d[k]=d[k+l];
	d[k+l]=re;
      }
    }
    dat->doneData();
  }
  // 3d
  else {
    d=dat->getData();
	
    rfftwnd_one_real_to_complex(cplan,(fftw_real *)rdata,(fftw_complex *)d);

    // in the y,z plane, swaps quadrants I,III and II,IV
    // this is the 'rotation' in y and z
    t=(char *)malloc(sizeof(float)*(nx+2));

    k=(nx+2)*ny*(nz+1)/2;
    l=(nx+2)*ny*(nz-1)/2;
    jj=(nx+2)*sizeof(float);
    for (j=ii=0; j<nz/2; j++) {
      for (i=0; i<ny; i++,ii+=nx+2) {
	memcpy(t,d+ii,jj);
	if (i<ny/2) {
	  memcpy(d+ii,d+ii+k,jj);
	  memcpy(d+ii+k,t,jj);
	}
	else {
	  memcpy(d+ii,d+ii+l,jj);
	  memcpy(d+ii+l,t,jj);
	}
      }
    }
    dat->doneData();
    free(t);
  }

  fft=dat;
  dat->multConst(1.0/(nx*ny*nz));
  dat->update();
  dat->flags|=EMDATA_COMPLEX|EMDATA_RI;
  i=flags;	// we didn't acutally change 'this', so we don't need an update
  doneData();
  flags=i&~(EMDATA_NEWFFT|EMDATA_BUSY);
  return dat;
}

EMData *EMData::doIFT()	{	// obvious, source must be R,I
  EMData *dat;
  float *d,re,im,norm;
  int i,j,k,l,ii,jj,fl,dim;
  char s[80],*t;
  mrcH *newh;
  rfftwnd_plan cplan;

  if (!(flags&EMDATA_COMPLEX)) return NULL;
  if (!(flags&EMDATA_RI)) error(ERR_WARN,"doIFT on AP data, only RI should be used. I will fix it.");

  //printf("%d,%d,%d %d,%d,%d\n",nx,ny,nz,npfa(nx-2),npfa(ny),npfa(nz));

  if ((nx-2)!=npfa(nx-2) || (ny!=npfa(ny)&&ny!=1) || (nz!=npfa(nz)&&nz!=1)) { error("Invalid IFT size !",ERR_ERROR); return NULL; }

  getData();

  if (fft) {
    if (!(flags&EMDATA_NEWFFT)) { doneData(); printf("Old fft\n"); return fft; }
    fft->setSize(nx,ny,nz);
    dat=fft;
    printf("old pointer\n");
  }
  else {
    dat=copyHead();
    if (strlen(name)>74) name[75]=0;
    //	sprintf(s,"IFT %s",name);
    //	dat->setName(s);
    fft=dat;
    fft->setSize(nx,ny,nz);
  }

  ap2ri();

  if (ny==1) dim=1;
  else if (nz==1) dim=2;
  else dim=3;

  cplan=makeplan(dim,nx-2,ny,nz,0);


  // 1d
  if (ny==1) {
    j=nx-2;
    k=1;
    d=dat->getData();
    rfftwnd_one_complex_to_real(cplan, (fftw_complex *)rdata,(fftw_real *)d);
    dat->doneData();
  }
  // 2d
  else if (nz==1) {
    d=dat->getData();

    memcpy((char *)d,(char *)rdata,nx*ny*sizeof(float));

    // This 'rotates' the data vertically by ny/2
    l=ny/2*(nx);
    for (i=0; i<ny/2; i++) {
      for (j=0; j<nx; j++) {
	k=j+i*(nx);
	re=d[k];
	d[k]=d[k+l];
	d[k+l]=re;
      }
    }

	
    rfftwnd_one_complex_to_real(cplan, (fftw_complex *)d,(fftw_real *)d);

    for (i=1; i<ny; i++) memcpy((char *)&d[i*(nx-2)],(char *)&d[i*nx],(nx-2)*sizeof(float));
    dat->doneData();


    //printf("size -> %d %d %d\n",nx-2,ny,nz);
  }
  // 3d
  else {
    d=dat->getData();

    memcpy((char *)d,(char *)rdata,nx*ny*nz*sizeof(float));

    // in the y,z plane, swaps quadrants I,III and II,IV
    // this is the 'rotation' in y and z
    t=(char *)malloc(sizeof(float)*(nx+2));

    k=nx*ny*(nz+1)/2;
    l=nx*ny*(nz-1)/2;
    jj=nx*sizeof(float);
    for (j=ii=0; j<nz/2; j++) {
      for (i=0; i<ny; i++,ii+=nx) {
	memcpy(t,d+ii,jj);
	if (i<ny/2) {
	  memcpy(d+ii,d+ii+k,jj);
	  memcpy(d+ii+k,t,jj);
	}
	else {
	  memcpy(d+ii,d+ii+l,jj);
	  memcpy(d+ii+l,t,jj);
	}
      }
    }
    free(t);
		
    rfftwnd_one_complex_to_real(cplan, (fftw_complex *)d,(fftw_real *)d);

    for (i=0; i<ny*nz; i++) {
      j=i*(nx-2);
      k=i*nx;
      for (l=j; l<j+nx-2; l++,k++) d[l]=d[k];
    }
    dat->doneData();
  }

  fft->setSize(nx-2,ny,nz);

  dat->update();
  dat->setComplex(0);

  i=flags;	// we didn't acutally change 'this', so we don't need an update
  doneData();
  flags=i&~(EMDATA_NEWFFT|EMDATA_BUSY);

  return dat;
}

// If a program wishes to modify the 
void EMData::gimmeFFT() {
  fft=NULL;
}

// Convolutes 2 data sets
EMData *EMData::convolute(EMData *with) {
  EMData *f1,*f2,*r;

  f1=doFFT()->copy();
  f2=with->doFFT();
  f1->mult(f2);

  r=f1->doIFT();
  f1->gimmeFFT();
  delete f1;
  return r;
}

// This calculates CCFs in the x direction and adds them up, result is 1d
// WARNING: this routine will modify the 'this' and 'with' to contain
//		1D fft's without setting some flags. This is an optimization
//		for rotational alignment.
// With nosum, returns an imate y1-y0+1 pixels high
EMData *EMData::calcCCFX(EMData *with,int y0,int y1,int nosum) {
  EMData *cf;
  float *d1,*d2,*f1,*f2,*cfd;
  int i,j,nx2,ny2;
  float re1,im1,re2,im2,norm;
  static rfftw_plan planf = NULL,planr = NULL;
  static int plann=-1;

  if (!with) { error(ERR_ERROR,"calcCCFX requires 'with'."); return NULL; }
  if (with->xSize()!=xSize() || with->ySize()!=ySize()) {
    error(ERR_ERROR,"calcCCFX must have 2 images the same size");
    return NULL;
  }
  if (zSize()>1||with->zSize()>1) {
    error(ERR_ERROR,"calcCCFX only works on 2D images");
    return NULL;
  }

  //the result
  cf=new EMData;
  if (nosum) cf->setSize(xSize(),y1-y0+1,1);
  else cf->setSize(xSize(),1,1);
  cf->zero();
  cf->setName("CCFx");
  cf->setPath("/tmp/eman.ccf");

  if (y1<=y0) y1=ySize();
  if (y0>=y1) y0=0;
  if (y0<0) y0=0;
  if (y1>ySize()) y1=ySize();
  //printf("ccfx %d %d\n",y0,y1);

  // real-space mode mainly for debugging
  if (nosum==2) {
    cfd=cf->getData();
    int x,y,i;
    for (y=y0; y<y1; y++) {
      for (x=0; x<xSize(); x++) {
	float dot=0;
	for (i=0; i<xSize(); i++) dot+=valueAt(i,y)*with->valueAt((i+x)%xSize(),y);
	cfd[x+(y-y0)*xSize()]=dot;
	if (x==0) printf("%d.\t%f\n",y,dot);
      }
    }
	
    cf->doneData();
    return cf;
  }

  // We cache the last-used plan
  if (plann != xSize()) {
    if (planf) rfftw_destroy_plan(planf);
    if (planr) rfftw_destroy_plan(planr);

    planf=rfftw_create_plan(xSize(), FFTW_REAL_TO_COMPLEX, FFTW_ESTIMATE);
    planr=rfftw_create_plan(xSize(), FFTW_COMPLEX_TO_REAL, FFTW_ESTIMATE);
    plann=xSize();
  }

  f1=(float *)calloc(xSize()*sizeof(float),1);
  f2=(float *)calloc(xSize()*sizeof(float),1);
  cfd=cf->getData();
  d1=getDataRO();
  d2=with->getDataRO();


  // We do 1D FFT's in place on the input images. This is probably a nasty
  // thing to do that will cause someone problems later, but it is necessary
  // for optimization. We can't afford the memory for another cache.
  if (!isComplexX()) {
    for (j=0; j<ySize(); j++) {
      rfftw_one(planf,d1+j*xSize(),f1);
      memcpy(d1+j*xSize(),f1,xSize()*sizeof(float));
    }
    setComplexX(1);
  }
  if (!with->isComplexX()) {
    for (j=0; j<with->ySize(); j++) {
      rfftw_one(planf,d2+j*xSize(),f2);
      memcpy(d2+j*xSize(),f2,xSize()*sizeof(float));	
    }
    with->setComplexX(1);
  }

  float *f1a,*f2a;
  // now we calculate the sum of the 1D CCF's
  for (j=y0; j<y1; j++) {
    f1a=d1+j*xSize();
    f2a=d2+j*xSize();
	
    // cross-correlation into f1
    f1[0]=f1a[0]*f2a[0];
    f1[xSize()/2]=f1a[xSize()/2]*f2a[xSize()/2];
    for (i=1; i<xSize()/2; i++) {
      re1=f1a[i];
      re2=f2a[i];
      im1=f1a[xSize()-i];
      im2=f2a[xSize()-i];
      f1[i]=re1*re2+im1*im2;
      f1[xSize()-i]=im1*re2-re1*im2;
    }

    // real results in f2
    rfftw_one(planr,f1,f2);

    // sum into cf
    if (nosum) { for (i=0; i<xSize(); i++) cfd[i+xSize()*(j-y0)]=f2[i]; }
    else { for (i=0; i<xSize(); i++) cfd[i]+=f2[i]; }
  }

  cf->doneData();
  doneData();
  with->doneData();

  free(f1);
  free(f2);

  return cf;
}


// Calculates a CCF. If with==NULL, does mirror ACF
EMData *EMData::calcCCF(EMData *with,EMData *filter) {
  EMData *f1,*f2,*cf;
  float *rdata1,*rdata2;
  int i,j,nx2,ny2,nz2;
  float re,im,norm;

  // get the ffts
  if (filter) f1=doFFT()->copy();
  else f1=doFFT();
  if (!f1) return NULL;
  f1->ap2ri();			// make sure image is real/imag

  if (with) {
    cf=with->doFFT();	// this is where the result will go
    if (!cf) return NULL;
    cf=cf->copy(0,0);		// get a copy so we can modify it freely
    cf->ap2ri();		//make sure image is real/imag
  }
  else cf=f1->copy(0,0);		// already r/i, so don't worry

  if (filter) {
    if (filter->xSize()!=cf->xSize() ||filter->xSize()!=cf->xSize() ||filter->xSize()!=cf->xSize()) {
      printf("calcCCF: cannot apply improperly sized filter!\n");
      exit(1);
    }

    cf->mult(filter);
    f1->mult(filter);
  }

  // make sure the images are the same size
  nx2=cf->xSize();
  ny2=cf->ySize();
  nz2=cf->zSize();
  if (with && (f1->xSize()!=nx2 || f1->ySize()!=ny2 || f1->zSize()!=nz2)) 
    { error("CCF: Images must be the same size.",ERR_ERROR); return NULL; }

  //f1->writeMRC("a1.img");
  //cf->writeMRC("a2.img");

  rdata1=f1->getDataRO();
  rdata2=cf->getData();

  if (with==this) {
    for (i=0; i<nx2*ny2*nz2; i+=2) {
      rdata2[i]=(rdata1[i]*rdata2[i]+rdata1[i+1]*rdata2[i+1]);
      im=0;
    }
  }
  else if (with) {    
    //	norm=nx2*ny2*nx2*ny2*Sigma()*with->Sigma();
    //	norm=nx2*ny2*nz2*nx2*ny2*nz2;
    norm=1.0;
    for (i=0; i<nx2*ny2*nz2; i+=2) {
      re=(rdata1[i]*rdata2[i]+rdata1[i+1]*rdata2[i+1])/norm;
      im=(rdata1[i+1]*rdata2[i]-rdata1[i]*rdata2[i+1])/norm;
      rdata2[i]=re;
      rdata2[i+1]=im;
    }
  }
  else {                           
    //	norm=nx2*ny2*nx2*ny2*(Mean()+Sigma())*(Mean()+Sigma());
    //	norm=nx2*ny2*nz2*nx2*ny2*nz2;
    norm=1.0;
    for (i=0; i<nx2*ny2*nz2; i+=2) {
      re=(rdata1[i]*rdata2[i]-rdata1[i+1]*rdata2[i+1])/norm;
      im=(rdata1[i+1]*rdata2[i]+rdata1[i]*rdata2[i+1])/norm;
      rdata2[i]=re;
      rdata2[i+1]=im;
    }
  }

  f1->doneData();		// ok done with the original data
  cf->doneData();
  f2=cf->doIFT();		// the result
  cf->gimmeFFT();		// we're keeping the ift, don't want it freed
  delete cf;			// cf is a copy, so get rid of it
  if (filter) delete f1;
  f2->setName("CCF");
  f2->setPath("/tmp/eman.ccf");
  //f2->writeMRC("af.img");

  //if (f2->Sigma()) f2->multConst(1.0/f2->Sigma());

  return f2;
}

// Calculates a MCF. If with==NULL, does mirror ACF
EMData *EMData::calcMCF(EMData *with,EMData *filter) {
  EMData *f1,*f2,*cf;
  float *rdata1,*rdata2;
  int i,j,nx2,ny2,nz2;
  float re,im,norm;

  if (with && (nx!=with->xSize() || ny!=with->ySize() || nz!=with->zSize()))
    { error("CCF: Images must be the same size.",ERR_ERROR); return NULL; }

  // get the ffts
  if (filter) f1=doFFT()->copy();
  else f1=doFFT();
  if (!f1) return NULL;
  f1->ap2ri();		// make sure image is real/imag

  // make sure the images are the same size
  nx2=f1->xSize();
  ny2=f1->ySize();
  nz2=f1->zSize();

  if (with) {
    cf=with->doFFT();	// this is where the result will go
    if (!cf) return NULL;
    cf=cf->copy(0,0);		// get a copy so we can modify it freely
    cf->ap2ri();			// make sure image is real/imag
  } else cf=f1->copy(0,0);	// already r/i, so don't worry

  if (filter) {
    if (filter->xSize()!=cf->xSize() ||filter->xSize()!=cf->xSize() ||filter->xSize()!=cf->xSize()) {
      printf("calcCCF: cannot apply improperly sized filter!\n");
      exit(1);
    }

    cf->mult(filter);
    f1->mult(filter);
  }

  rdata1=f1->getDataRO();
  rdata2=cf->getData();

  // for an autocorrelation, we can take some shortcuts to save time
  if (with==this) {

    for (i=0; i<nx2*ny2*nz2; i+=2) {
      rdata2[i]=sqrt((rdata1[i]*rdata2[i]+rdata1[i+1]*rdata2[i+1]));
      rdata2[i+1]=0;
    }

    f1->doneData();		// ok done with the original data
    cf->doneData();
  }
  else {
    //norm=nx2*ny2*nz2*nx2*ny2*nz2;
    //norm=nx*ny*nz*nx*ny*nz;
    norm=1.0;
    for (i=0; i<nx2*ny2*nz2; i+=2) {
      re=(rdata1[i]*rdata2[i]+rdata1[i+1]*rdata2[i+1]);
      im=(rdata1[i+1]*rdata2[i]-rdata1[i]*rdata2[i+1]);
      rdata2[i]=re;
      rdata2[i+1]=im;
    }

    f1->doneData();		// ok done with the original data
    cf->doneData();

    // This is much faster than the ri2ap() scheme below

    rdata1=cf->getData();
    for (i=0; i<nx2*ny2*nz2; i+=2) {
      float t=SQR(rdata1[i])+SQR(rdata1[i+1]);
      if (t==0) continue;
      t=pow(t,.25);
      rdata1[i]/=t;
      rdata1[i+1]/=t;
    }
    cf->doneData();
  }

  /*
    cf->ri2ap();
    rdata1=cf->getData();
    for (i=0; i<nx2*ny2*nz2; i+=2) {
    rdata1[i]=sqrt(rdata1[i]);
    }
    cf->doneData();
    cf->ap2ri();
  */
		
  f2=cf->doIFT();		// the result
  cf->gimmeFFT();		// we're keeping the ift, don't want it freed
  delete cf;			// cf is a copy, so get rid of it
  if (filter) delete f1;
  f2->setName("MCF");
  f2->setPath("/tmp/eman.mcf");
  return f2;
}

// move center to center of mass (more or less)
// eliminates parent if any, integer shifts only
void EMData::cmCenter(int intonly) {
  float xm=0,ym=0,zm=0,m=0;
  int i,j,k;

  normalize();

  getData();
  for (i=0; i<nx; i++) {
    for (j=0; j<ny; j++) {
      for (k=0; k<nz; k++) {
	//			if (hypot(i-nx/2.0,j-ny/2.0)>=nx/2) continue;
	if (rdata[i+nx*j+k*nx*ny]<Sigma()*.75+Mean()) continue; // may help since protein is denser
	xm+=(float)i*rdata[i+nx*j+k*nx*ny];
	ym+=(float)j*rdata[i+nx*j+k*nx*ny];
	zm+=(float)k*rdata[i+nx*j+k*nx*ny];
	m+=rdata[i+nx*j+k*nx*ny];
      }
    }
  }	
  doneData();
  xm/=m;
  ym/=m;
  zm/=m;

  //printf("%f\t%f\t%f\n",xm,ym,zm);

  if (intonly) {
    dx=-(floor(xm+.5)-nx/2);
    dy=-(floor(ym+.5)-ny/2);
    if (nz>1) dz=-(floor(zm+.5)-nz/2);
  }
  else {
    dx=-(xm-nx/2);
    dy=-(ym-ny/2);
    if (nz>1) dz=-(zm-nz/2);
  }
  parent=NULL;
  rotateAndTranslate();
}

// This will calculate the shift for a translational alignment, but doesn't do the rotateAndTranslate
// Intonly will only make whole integer translations to avoid interpolation
// maximum allowed translation +-nx/4
float EMData::transAlign(EMData *with,int useparent,int intonly,int maxshift, float *peak) {
  EMData *cf;
  float *rdata3;
  int i,j,ii,jj,nx2,flag=1,acf=0;
  float re,im,r2,f,to,am,ph,t;
  float tx,ty,ret;
  char *tmp;

  if (with && (with->xSize()!=nx || with->ySize()!=ny)) { error("ALIGN: images must be same size",ERR_ERROR); return -1; }
  //if (!with) printf("Warning: ACF\n");

  //if (useparent&&parent) { cf=parent->calcMCF(with); }
  //else cf=calcMCF(with);
  if (useparent&&parent) { cf=parent->calcCCF(with); }
  else cf=calcCCF(with);

  if (maxshift<=0) maxshift=ny/8;
  if (maxshift>nx/2-1) maxshift=nx/2-1;
  if (maxshift>ny/2-1) maxshift=ny/2-1;

  //writeIMAGIC("ccf.img",-1);
  //with->writeIMAGIC("ccf.img",-1);
  //cf->writeIMAGIC("ccf.img",-1);

  cf->toCorner();				// shift center to center from corner
  cf->applyMask(0,-1);
  rdata3=cf->getData();

  /*
    t=cf->Max()-cf->Sigma();
    tx=ty=f=0;
    for (j=ny/2-maxshift; j<ny/2+maxshift; j++) {
    for (i=nx/2-maxshift; i<nx/2+maxshift; i++) {
    if (rdata3[i+j*nx]>t) {
    tx+=rdata3[i+j*nx]*i;
    ty+=rdata3[i+j*nx]*j;
    f+=rdata3[i+j*nx];
    }
    }
    }

    ii=floor(tx/f+.5);
    jj=floor(ty/f+.5);
  */

  float neg=cf->Mean()-cf->Min(), pos=cf->Max()-cf->Mean();
  if (neg>pos) flag=-1;


  /*
    if (flag==-1) { 
    printf("Negative correlation! %f > %f \n",neg,pos); 
    writeIMAGIC("neg.hed",-1); 
    with->writeIMAGIC("neg.hed",-1); 
    cf->doneData();
    cf->writeIMAGIC("neg.hed",-1); 
    rdata3=cf->getData();
    }
  */

  // locate the peak
  ii=jj=nx/2;
  r2=-1.0e20;
  f=1.0e20;
  for (j=ny/2-maxshift; j<ny/2+maxshift; j++) {
    for (i=nx/2-maxshift; i<nx/2+maxshift; i++) {
      if (rdata3[i+j*nx]*flag>r2) { r2=rdata3[i+j*nx]*flag; ii=i; jj=j; }
      if (rdata3[i+j*nx]<f) f=rdata3[i+j*nx];
    }
  }

  if (peak) *peak=r2;

  // now find the peak to subpixel resolution
  /*
    tx=-p5peak(rdata3[ii-2+jj*nx],rdata3[ii-1+jj*nx],rdata3[ii+jj*nx],
    rdata3[ii+1+jj*nx],rdata3[ii+2+jj*nx])+(float)(nx/2-ii);

    ty=-p5peak(rdata3[ii+jj*nx-2*nx],rdata3[ii+jj*nx-nx],rdata3[ii+jj*nx],
    rdata3[ii+jj*nx+nx],rdata3[ii+jj*nx+nx*2])+(float)(ny/2-jj);

    if (intonly) { tx=floor(tx+.5); ty=floor(ty+.5); }
  */

  // no subpixel at the moment
  tx=nx/2-ii;
  ty=ny/2-jj;


  if (useparent&&parent) {
    ret=hypot(tx-dx,ty-dy);
    //printf("\t%7.3f \t%7.3f \t%5.3g\n",tx-dx,ty-dy,r2);
  }
  else {
    ret=hypot(tx,ty);
    //printf("\t%7.3f \t%7.3f \t%5.3g\n",tx,ty,r2);
  }

  /*if (!parent||useparent) {
    dx=tx;
    dy=ty;
    euler.phi()=0;
    }
    else {
    dx+=tx*cos(euler.phi())+ty*sin(euler.phi());
    dy+=-tx*sin(euler.phi())+ty*cos(euler.phi());
    }*/
  dx=tx;
  dy=ty;
  setRAlign(0,0,0);

  if (!with) { dx/=2.0; dy/=2.0; }	// aligning to self, only 1/2 shift

  if (intonly) { dx=floor(dx+.5); dy=floor(dy+.5); }

  cf->doneData();
  delete cf;

  return ret;
}

// Intonly will only make whole integer translations to avoid interpolation
// maximum allowed translation +-nx/4
float EMData::transAlign3d(EMData *with,int useparent,int intonly) {
  EMData *cf;
  float *rdata3;
  int i,j,k,ii,jj,kk,nx2,flag,acf=0;
  float re,im,r2,f,to,am,ph;
  float tx,ty,tz,ret;
  char *tmp;

  if (with && (with->xSize()!=nx || with->ySize()!=ny || with->zSize()!=nz)) { error("ALIGN: images must be same size",ERR_ERROR); return -1; }
  if (!with) printf("Warning: ACF\n");

  if (useparent&&parent) { cf=parent->calcCCF(with); }
  else cf=calcCCF(with);

  cf->toCorner();				// shift center to center from corner
  rdata3=cf->getData();

  if (cf->Mean()-cf->Min()<cf->Max()-cf->Mean()) { flag=1; /*printf("positive CCF ");*/ }
  else { flag=-1; /*printf("negative CCF ");*/ }

  // locate the peak
  ii=jj=nx/2;
  r2=-1.0e20;
  f=1.0e20;
  for (k=nx/4; k<3*nx/4; k++) {
    for (j=ny/4; j<3*ny/4; j++) {
      for (i=nx/4; i<3*nx/4; i++) {
	if (rdata3[i+j*nx+k*nx*ny]*flag>r2) { r2=rdata3[i+j*nx+k*nx*ny]*flag; ii=i; jj=j; kk=k; }
	if (rdata3[i+j*nx+k*nx*ny]<f) f=rdata3[i+j*nx+k*nx*ny];
      }
    }
  }

  // no subpixel at the moment
  tx=nx/2-ii;
  ty=ny/2-jj;
  tz=nz/2-kk;

  if (useparent&&parent) {
    ret=hypot3(tx-dx,ty-dy,tz-dz);
    //printf("\t%7.3f \t%7.3f \t%5.3g\n",tx-dx,ty-dy,r2);
  }
  else {
    ret=hypot3(tx,ty,tz);
    //printf("\t%7.3f \t%7.3f \t%5.3g\n",tx,ty,r2);
  }

  dx=tx;
  dy=ty;
  dz=tz;

  if (!with) { dx/=2.0; dy/=2.0; dz/=2.0; }	// aligning to self, only 1/2 shift

  cf->doneData();
  delete cf;

  return ret;
}


// shrinks a factor by an integral factor using a local median filter
// note that sf MUST be a factor of the image size
void EMData::medianShrink(int sf) {
  int i,j,l,i2,j2,l2,k,nx2,ny2;
  float *old,*mbuf,f;

  if (sf<=1) return;
  if ((nx%sf) || (ny%sf)) error(ERR_WARN,"Image size not divisible by shrink factor");
  if (nz>1) { 
    if (nz%sf) error(ERR_WARN,"Image size not divisible by shrink size\n"); 
    mbuf=(float *)malloc(sf*sf*sf*sizeof(float));
    old=(float *)malloc(nx*ny*nz*sizeof(float));
    memcpy(old,getDataRO(),nx*ny*nz*sizeof(float));
    doneData();	
    nx2=nx;
    ny2=ny;
    setSize(nx/sf,ny/sf,nz/sf);
    getData();

    // note that nx,ny,nz are now /sf
    for (l=0; l<nz; l++) {
      for (j=0; j<ny; j++) {
	for (i=0; i<nx; i++) {
	  // read out the data for 1 result pixel median filter
	  k=0;
	  for (l2=l*sf; l2<(l+1)*sf; l2++) {
	    for (j2=j*sf; j2<(j+1)*sf; j2++) {
	      for (i2=i*sf; i2<(i+1)*sf; i2++,k++) {
		mbuf[k]=old[i2+j2*nx2+l2*nx2*ny2];
	      }
	    }
	  }
		
	  // sort into the 1st half of the array
	  for (k=0; k<sf*sf*sf/2+1; k++) {
	    for (i2=k+1; i2<sf*sf*sf; i2++) {
	      if (mbuf[i2]<mbuf[k]) { f=mbuf[i2]; mbuf[i2]=mbuf[k]; mbuf[k]=f; }
	    }
	  }
		
	  rdata[i+j*nx+l*nx*ny]=mbuf[sf*sf*sf/2];
	}
      }
    }
    doneData();

    free(old);
    free(mbuf);

    return;
  }

  mbuf=(float *)malloc(sf*sf*sizeof(float));
  old=(float *)malloc(nx*ny*sizeof(float));
  memcpy(old,getDataRO(),nx*ny*sizeof(float));
  doneData();
  nx2=nx;
  setSize(nx/sf,ny/sf,1);
  getData();

  // note that nx is now /sf
  for (i=0; i<nx; i++) {
    for (j=0; j<ny; j++) {
      // read out the data for 1 result pixel median filter
      k=0;
      for (i2=i*sf; i2<(i+1)*sf; i2++) {
	for (j2=j*sf; j2<(j+1)*sf; j2++,k++) {
	  mbuf[k]=old[i2+j2*nx2];
	}
      }
		
      // sort into the 1st half of the array
      for (k=0; k<sf*sf/2+1; k++) {
	for (i2=k+1; i2<sf*sf; i2++) {
	  if (mbuf[i2]<mbuf[k]) { f=mbuf[i2]; mbuf[i2]=mbuf[k]; mbuf[k]=f; }
	}
      }
		
      rdata[i+j*nx]=mbuf[sf*sf/2];
    }
  }
  doneData();

  free(old);
  free(mbuf);

  return;
}

// Like medianShrink, but uses average instead of median
// This one is done in-place
void EMData::meanShrink(int sf) {
  int i,j,k,ii,jj,kk;

  if (sf<=1) return;

  int nx2=nx/sf, ny2=ny/sf, nz2=nz/sf;

  if ((nx%sf) || (ny%sf)) error(ERR_WARN,"Image size not divisible by shrink factor");
  float *d=getData();
  if (nz>1) { 
    if (nz%sf) error(ERR_WARN,"Image size not divisible by shrink size\n"); 
	
    // outer loops iterate over new pixel positions
    for (k=0; k<nz2; k++) {
      for (j=0; j<ny2; j++) {
	for (i=0; i<nx2; i++) {
	  // inner loops iterate over old pixel positions
	  float sum=0;
	  for (kk=k*sf; kk<k*sf+sf; kk++) {
	    for (jj=j*sf; jj<j*sf+sf; jj++) {
	      for (ii=i*sf; ii<i*sf+sf; ii++) {
		sum+=d[ii+jj*nx+kk*nx*ny];
	      }
	    }
	  }
	  d[i+j*nx2+k*nx2*ny2]=sum/(sf*sf*sf);
	}
      }
    }
	
    doneData();
    setSize(nx2,ny2,nz2);
    return;
  }
  // outer loops iterate over new pixel positions
  for (j=0; j<ny2; j++) {
    for (i=0; i<nx2; i++) {
      // inner loops iterate over old pixel positions
      float sum=0;
      for (jj=j*sf; jj<j*sf+sf; jj++) {
	for (ii=i*sf; ii<i*sf+sf; ii++) {
	  sum+=d[ii+jj*nx];
	}
      }
      d[i+j*nx2]=sum/(sf*sf);
    }
  }
	
  doneData();
  setSize(nx2,ny2,1);
  return;

	
}



// Generates an autocorrelation based image used for rotational alignment
EMData *EMData::makeRFP(int premasked,int unwrap) {
  static EMData *filt=NULL;
  EMData *tmp,*tmp2;
  int cs;


  if (!(flags&EMDATA_NEWRFP) && rfp) { return rfp; }
  if (nx&1) return NULL;
  if (rfp) delete rfp;

  if (!filt) {
    filt=new EMData;
    filt->setComplex(1);
  }

  cs=(((nx*7/4)&0xfffff8)-nx)/2;	// this pads the image to 1 3/4 * size with result divis. by 8
  //cs=((nx/4)&0xfffff8);
  //while ((nx+2*cs)%8!=0) cs+=2;
  if (nz==1) tmp2=clip(-cs,-cs,nx+2*cs,ny+2*cs);
  else tmp2=clip(-cs,-cs,-cs,nx+2*cs,ny+2*cs,nz+2*cs);
  if (!premasked) tmp2->applyMask(nx/2,4);

  if (filt->xSize()!=tmp2->xSize()+2 || filt->ySize()!=tmp2->ySize() ||
      filt->zSize()!=tmp2->zSize()) {
    filt->setSize(tmp2->xSize()+2,tmp2->ySize(),tmp2->zSize());
    filt->one();
    //	filt->filter(3,999,1);
    filt->filter(3,9999,1);
  }
  //tmp=tmp2->calcCCF(tmp2,filt);
  tmp=tmp2->calcMCF(tmp2,filt);
  //tmp=tmp2->calcMCF(tmp2);
  delete tmp2;

  //tmp=calcCCF(this);
  tmp->toCorner();


  if (nz==1) {
    tmp2=tmp->clip(cs-nx/4,cs-ny/4,nx*3/2,ny*3/2);
  }
  else {
    tmp2=tmp->clip(cs-nx/4,cs-ny/4,cs-ny/4,nx*3/2,ny*3/2,nz*3/2);
  }

  //if (nz==1) tmp2->radialSubtract();
  //tmp2->applyMask(-1,4);

  rfp=tmp2;
  flags&=~EMDATA_NEWRFP;
  delete tmp;

  if (nz==1) {
    if (!unwrap) { tmp2->applyMask(-1,4); rfp=NULL; return tmp2; }
    rfp=tmp2->unwrap();
    delete tmp2;
  }
  //if (nz==1 && isvalidfft(nx/2)) rfp->medianShrink(2);

  return rfp;
}

/*
  EMData *EMData::makeRFP() {
  EMData *ret,*tmp,*tmpf;
  float *dat,*ddat,sig;
  int i,j,k,s,o,nnx,nny;

  if (!(flags&EMDATA_NEWRFP) && rfp) { return rfp; }
  if (rfp) delete rfp;

  // volume RFP
  if (nz>1) {
  s=ny/2;

  ret=new EMData;
  ret->setSize(s,s,s);
  ret->zero();
  ddat=ret->getData();

  // pads the image to twice its size for better fft
  //	tmp=clip(-ny/2,-ny/2,-ny/2,ny*2,ny*2,ny*2);
  tmp=copy();
  tmp->normalize();
  tmp->applyMask(ny/2,0);
  tmp->toCorner();
  tmpf=tmp->doFFT();
  tmpf->ri2ap();
  nnx=tmpf->xSize();
  nny=tmpf->ySize();
  dat=tmpf->getData();
  dat[ny*(ny+2)/2+ny*ny*(ny+2)/2]=0;	// complex origin ->0


  // right half
  for (i=s/2; i<s; i++) {
  for (j=1; j<s; j++) {
  for (k=1; k<s; k++) {
  ddat[i+j*s+k*s*s]=dat[(i-s/2)*2+(j+ny/4)*nnx+(k+ny/4)*nnx*nny];
  }
  }
  }

  // left half is point mirrored
  for (i=1; i<s/2; i++) {
  for (j=1; j<s; j++) {
  for (k=1; k<s; k++) {
  ddat[i+j*s+k*s*s]=ddat[(s-i)+(s-j)*s+(s-k)*s*s];
  }
  }
  }

  tmpf->doneData();
  delete tmp; //also gets tmpf

  ret->doneData();
  ret->applyMask(-1,0);
  rfp=ret;
  flags&=~EMDATA_NEWRFP;
	
  //	ret->writeMRC("rfp.mrc");
  return ret;
  }

  // 2d images
  s=ny/2;
  o=ny/4*(ny+2);

  ret=new EMData;
  ret->setSize(s,s,1);
  ret->zero();
  ddat=ret->getData();

  // pads the image to twice its size for better fft
  //tmp=clip(-ny/2,-ny/2,ny*2,ny*2);
  tmp=copy();
  tmp->normalize();
  //tmp->applyMask(ny/2,0);
  tmp->radialSubtract();
  tmp->toCorner();
  tmpf=tmp->doFFT();
  tmpf->ri2ap();
  dat=tmpf->getData();
  dat[(ny+2)*ny/2]=0;	// complex origin ->0


  // right lower half
  for (i=s/2; i<s; i++) {
  for (j=1; j<s; j++) {
  ddat[i+j*s]=dat[(i-s/2)*2+j*(ny+2)+o];
  }
  }

  // left half is point mirrored
  for (i=1; i<s/2; i++) {
  for (j=1; j<s; j++) {
  ddat[i+j*s]=ddat[(s-i)+(s-j)*s];
  }
  }


  tmpf->doneData();
  delete tmp; //also gets tmpf

  ret->doneData();
  //ret->applyMask(-1,4);
  rfp=ret;
  flags&=~EMDATA_NEWRFP;

  //ret->multConst(1.0/sig);
  //ret->writeIMAGIC("rfp.hed",-1);

  //ret->normalize();
  //ret->applyMask(-1,4);
  //ret->radialSubtract();
  return ret;
  }
*/


#define RALRAD	5
#define RALANG	8

// This does a rotational alignment using circular harmonics
float EMData::rotAlignCH(EMData *with,int irad,int orad) {
  int i,j,x,y,na;
  float cen,wid,a,r;
  EMData *d1,*d2;
  float *dat1,*dat2;

  if (nx!=ny) return(-1.0);

  if (irad<=0) irad=6;
  if (orad<=2 || orad>nx/2) orad=nx/2-2;

  //printf("%%%%%%%%%%%%%%%%%\n");

  // here we regenerate circular harmonics if necessary
  if (nx!=rals || irad!=rali || orad!=ralo) {
    //	printf("Recalc basis %d %d %d\n",rals,ralo,rali);
    for (i=0; i<ralfp->count(); i++) 
      delete (EMData *)(ralfp->objectAt(i));
    ralfp->empty();
	
    rals=nx;
    ralo=orad;
    rali=irad;
	
    wid=(ralo-rali)/(RALRAD+1);
    for (i=0; i<RALANG; i++) {
      for (j=0; j<RALRAD; j++) {
	cen=(j+i)*wid+rali;
	na=1<<(i+1);
	if (cen*PI<na) continue;
		
	d1=new EMData;
	d1->setSize(nx,ny,1);
	d2=new EMData;
	d2->setSize(nx,ny,1);
			
	d1->ufloat=cen;
	d1->unum2=na;
			
	dat1=d1->getData();
	dat2=d2->getData();

	for (x=0; x<nx; x++) {
	  for (y=0; y<ny; y++) {
	    if (y==ny/2 && x==nx/2) a=0;
	    else a=atan2((float)(y-ny/2),(float)(x-nx/2));
	    r=hypot((float)(y-ny/2),(float)(x-nx/2))-cen;
	    dat1[x+y*nx]=sin(a*na)*exp(-r*r/(wid*wid/2.4));
	    dat2[x+y*nx]=cos(a*na)*exp(-r*r/(wid*wid/2.4));
	  }
	}

	d1->doneData();
	d2->doneData();
			
	ralfp->addObject(d1);
	ralfp->addObject(d2);

      }
    }
  }

  // OK now do the alignment
  float ta,tb,a1,p1,a2,p2;
  float ca,aa=0,ndot=0;
  float dl,ep;

  for (i=0; i<ralfp->count(); i+=2) {
    d1=(EMData *)ralfp->objectAt(i);
    d2=(EMData *)ralfp->objectAt(i+1);

    ta=dot(d1);
    tb=dot(d2);
    a1=hypot(ta,tb);
    if (ta==0 && tb==0) p1=0;
    else p1=atan2(ta,tb);
	
    ta=with->dot(d1);
    tb=with->dot(d2);
    a2=hypot(ta,tb);
    if (ta==0 && tb==0) p2=0;
    else p2=atan2(ta,tb);
	
    // this complicated little mess could probably be simplified  :^)
    ca=p2-p1;							// relative phase of 2 images
    ca=ca-floor(ca/(2.0*PI))*2.0*PI;	// range 0 - 2 PI
    if (ca>PI) ca-=2.0*PI;				// range -PI to PI
    if (ndot>0) {
      dl=PI*2.0/(float)d1->unum2;			// size of one oscillation in radians
      ep=aa/ndot;
      ep=(ep-floor(ep/dl)*dl)*2.0*PI/dl;	// phase predicted by current average
      ca-=ep;								// phase relative to current average
      ca=ca-floor(ca/(2.0*PI))*2.0*PI;	// range 0 - 2 PI
      if (ca>PI) ca-=2.0*PI;				// range -PI to PI
      ca=aa/ndot+ca/(float)d1->unum2;
      //		printf("%6.2f  %4d\t%4.1f  %4.1f  %4.1f\n",d1->ufloat,d1->unum2,ca*180.0/PI,aa/ndot*180.0/PI,a2);
    }
    //	else printf("%6.2f  %4d\t%4.1f\n",d1->ufloat,d1->unum2,ca*180.0/PI);
    //	printf("%f\t%d\n",ca*180.0/PI,i);

    aa+=ca*d1->ufloat*a2;
    ndot+=d1->ufloat*a2;

    printf("%f\t%d\n",ca*180.0/PI,i);
  }
  printf("%f\t%d\n",aa/ndot*180.0/PI,i+5);
  setRAlign(aa/ndot,0,0);
  return aa/ndot;
}


// Rotational alignment with 180 degree ambiguity
// WARNING: This assumes the particles have been premasked
float EMData::rotAlign(EMData *with) {
  EMData *r1,*r2,*cf;
  int size;

  r1=makeRFP(1);
  r2=with->makeRFP(1);
  size=r1->xSize();

  // Calculate 1D CCF (MCF) of each row of rotational footprint and sum them
  cf=r1->calcCCFX(r2,0,ySize());

  //cf->writeIMAGIC("cf.hed",-1);

  // Find the peak in cf which indicates the best alignment angle
  float *d = cf->getDataRO();
  float pv = -9.e20;
  int i,px=0;
  //for (int i=0; i<size; i++) if (fabs(d[i])>pv) { pv=fabs(d[i]); px=i; }
  for (i=0; i<size; i++) {
    if (d[i]>pv) { pv=d[i]; px=i; }
  }
  ufloat4=pv;
  setRAlign(-px*PI/size);

  cf->doneData();
  delete cf;

  //printf("R Peak %d\t%g\t%1.1f\n",px,pv,px*180.0/size);


  return pv;
}

// Rotational alignment of precentered particles
float EMData::rotAlignPrecen(EMData *with) {
  EMData *r1,*r2,*cf;
  int size=bestfftsize(PI*ySize()*1.5);

  r1=unwrap(4,ySize()*7/16,size,0,0,1);
  r2=with->unwrap(4,ySize()*7/16,size,0,0,1);

  // Calculate 1D CCF (MCF) of each row of rotational footprint and sum them
  cf=r1->calcCCFX(r2,0,ySize());

  //cf->writeIMAGIC("cf.hed",-1);

  // Find the peak in cf which indicates the best alignment angle
  float *d = cf->getDataRO();
  float pv = -9.e20;
  int i,px=0;
  //for (int i=0; i<size; i++) if (fabs(d[i])>pv) { pv=fabs(d[i]); px=i; }
  for (i=0; i<size; i++) {
    if (d[i]>pv) { pv=d[i]; px=i; }
  }
  ufloat4=pv;
  setRAlign((1.0-px/(float)size)*PI*2);


  cf->doneData();
  delete cf;
  delete r1;
  delete r2;

  //printf("R Peak %d\t%g\t%1.1f\n",px,pv,px*180.0/size);


  return pv;
}


void EMData::calcRCF(EMData *with,float *sum,int NS) {
  int i,j,k,l,nx2;
  float *dat,*dat2;
  float DA=(2.0*PI/NS);
  EMData *dd;
  List *list;
  float min,max,max2,norm,lim,ret;
  char s[80];

  rfftwnd_plan fplan=rfftwnd_create_plan(1, &NS,FFTW_REAL_TO_COMPLEX,
					 FFTW_MEASURE|FFTW_IN_PLACE|FFTW_USE_WISDOM);
  rfftwnd_plan rplan=rfftwnd_create_plan(1, &NS,FFTW_COMPLEX_TO_REAL,
					 FFTW_MEASURE|FFTW_IN_PLACE|FFTW_USE_WISDOM);

  dat=(float *)malloc((NS+2)*sizeof(float));
  dat2=(float*)malloc((NS+2)*sizeof(float));

  // maximum radius to use
  nx2=nx*9/20;	// 90% rad
  if (fabs(dx)<fabs(dy)) lim=fabs(dy); else lim=fabs(dx);
  nx2-=(int)floor(lim);

  memset((char *)sum,0,sizeof(float)*NS);		// clear the sum array before we start

  // split into radial 'shells' 6 pixels wide
  for (i=8; i<nx2; i+=6) {
    calcAzDist(NS,0,DA,dat,i,i+6);			// azimuthal distributions in this slice
    with->calcAzDist(NS,0,DA,dat2,i,i+6);
	
    rfftwnd_one_real_to_complex(fplan, dat,(fftw_complex *)dat);
    rfftwnd_one_real_to_complex(fplan, dat2,(fftw_complex *)dat2);
	
    // CCF in Fourier space
    for (j=0; j<NS+2; j+=2) {		
      max=dat[j]*dat2[j]+dat[j+1]*dat2[j+1];
      max2=dat[j+1]*dat2[j]-dat2[j+1]*dat[j];
      dat[j]=max;
      dat[j+1]=max2;
    }

    // iFFT to get the CCF
    rfftwnd_one_complex_to_real(rplan,(fftw_complex *)dat, dat);
	
    norm=NS*NS*(4.0*Sigma())*(4.0*with->Sigma());
    for (j=0; j<NS; j++)
      sum[j]+=dat[j]*(float)i/norm;
  }

  //save_data(0,ROTDA,sum,ROTSMP,"test.rcf.dat");
  //writeMRC("x1.img");
  //with->writeMRC("x2.img");

  fftwnd_destroy_plan(fplan);
  fftwnd_destroy_plan(rplan);

  free(dat);
  free(dat2);
  return;
}

// 2D only. polar->rectangular coords, 0-180 degrees, radially weighted
// When used with RFP, this provides 1 pixel accuracy at 75% radius
EMData *EMData::unwrap(int r1,int r2,int xs,int dx,int dy,int do360) {
  int x,y,k,r,a;
  float *d,*dd;
  float t,u,xx,yy;

  if (do360) do360=2; else do360=1;

  EMData *ret = new EMData;

  if (xs<1) {
    xs = (int) floor(do360*PI*ySize()/4);
    xs=bestfftsize(xs);
  }
  //printf("fftsize %d -> %d\n",xs,bestfftsize(xs));

  if (r1<0) r1=4;
  int rr=ySize()/2-2-floor(hypot((float)dx,(float)dy));
  if (r2<=r1||r2>rr) r2=rr;


  //ret->setSize(xs,ySize()/2-5,1);
  ret->setSize(xs,r2-r1,1);
  d=getDataRO();
  dd=ret->getData();

  for (x=0; x<xs; x++) {
    float si=sin(x*PI*do360/xs);
    float co=cos(x*PI*do360/xs);
    for (y=0; y<r2-r1; y++) {
      xx=(y+r1)*co+xSize()/2+dx;
      yy=(y+r1)*si+ySize()/2+dy;
      t=xx-floor(xx);
      u=yy-floor(yy);
      k=floor(xx)+floor(yy)*xSize();
      dd[x+y*xs]=bilin(d[k],d[k+1],d[k+xSize()+1],d[k+xSize()],t,u)*(y+r1);
    }
  }
  doneData();
  ret->doneData();

  return ret;
}

// Rotational alignment with possible flip, particles must be centered already
// (2d only)
EMData *EMData::RFAlign(EMData *with,EMData *flip,int imask) {
  EMData *cop,*cop2;
  float d1,d2;

  //with->writeIMAGIC("tst.hed",-1);
  cop=copy();
  cop->rotAlign(with);
  cop->rotateAndTranslate();
  d1=cop->dot(with);

  //printf("------\n");

  if (flip) {
    cop2=copy();
    cop2->rotAlign(with);
    cop2->rotateAndTranslate();
    d2=cop2->dot(with);
  }
  else {
    cop2=copy();
    cop2->vFlip();
    cop2->rotAlign(with);
    cop2->rotateAndTranslate();
    d2=cop2->dot(with);
  }

  if (cop==NULL) return cop2;
  if (cop2==NULL) return cop;

  //with->writeIMAGIC("abc",-1);
  //cop->writeIMAGIC("abc",-1);
  //cop2->writeIMAGIC("abc",-1);

  if (d1>d2) { cop->setFlipped(0); delete cop2; return cop; }
  else { cop2->setFlipped(1); delete cop; return cop2; }

}

// WARNING: This routine does not work properly, it is experimental only
// This one allows for flips
// If flipped data is not provided and flipped alignment is best, 'this' will be flipped on output
EMData *EMData::RTFAlignRadon(EMData *with,int maxshift,EMData *thisf,EMData *radonwith,EMData *radonthis,EMData *radonthisf) {
  int Drw=0;
  EMData *r1,*r2;

  if (!radonwith) { Drw=1; radonwith=with->doRadon(); }

  r1=RTAlignRadon(with,maxshift,radonwith,radonthis);
  if (!thisf) {
    hFlip();
    r2=RTAlignRadon(with,maxshift,radonwith,radonthisf);
    hFlip();
  }
  else r2=thisf->RTAlignRadon(with,maxshift,radonwith,radonthisf);

  r1->ufloat4=r1->dot(with);
  r2->ufloat4=r2->dot(with);

  if (Drw) delete radonwith;

  if (r1->ufloat4<r2->ufloat4) { delete r1; if (!thisf) hFlip(); return r2; }

  delete r2;
  return r1;
}


// WARNING: This routine does NOT work properly, it is experimental only
// Rotational & translational alignment using Radon transform
// optional inputs give starting radon transforms for speed
// generally 'with' will be the less noisy image if there is a choice
// note that maxshift is radial, not x&y like the other routines
EMData *EMData::RTAlignRadon(EMData *with,int maxshift,EMData *radonwith,EMData *radonthis) {
  int i,j,x,x2,y,size,w1,w2,Drw=0,Drt=0;
  float *d,*d2,*vert;
  EMData *t1;
  EMData *ccf;
  int px,py;
  float pv;
  int lda=0,ldr=0,ldra=0;

  size=xSize();
  w1=size/4;
  w2=size/8;
  if (with->xSize()!=size || with->ySize()!=size || ySize()!=size) {
    error(ERR_ERROR,"RTFAlignRadon, images must be square and the same size!\n");
    return NULL;
  }

  vert=(float *)malloc(sizeof(float)*size);

  if (!radonwith) { Drw=1; radonwith=with->doRadon(); }
  if (!radonthis) { Drt=1; radonthis=doRadon(); }
  if (maxshift<=0) maxshift=size/8;

  // Mask out stripes from this
  //d=radonthis->getData();
  //memset(d,0,sizeof(float)*maxshift*size);
  //memset(d+(size-maxshift)*size,0,sizeof(float)*maxshift*size);
  //radonthis->doneData();

  // Now we start with a rotational alignment (cross correlation in x)
  t1=radonthis->copy(0,0);
  radonthis->writeIMAGIC("radon.hed",0);

  float si=0,co=0,max=0,ta=0;

  // 3 alignment iterations
  for (j=0; j<3; j++) {
    // We use the previous translation to set up for the rotation alignment
    if (j) {
      d=radonthis->getDataRO();
      d2=t1->getData();
      for (x=0; x<size; x++) {
	for (y=maxshift; y<size-maxshift+1; y++) {
	  d2[x+y*size]=d[x+(y+(int)floor(max*sin(x*2.0*PI/size+ta)))*size];
	}
      }
      radonthis->doneData();
      t1->doneData();
    }

    t1->writeIMAGIC("radon.hed",j+1);

    // Now try to locate the Radon image in the reference
    // This is actually silly, since we're only using one line
    // of the result, but there's a bug in ccfx
    //	ccf=t1->calcCCFX(radonwith,maxshift,size-maxshift+1);

    EMData *r1 = t1->vertACF(maxshift);
    EMData *r2 = radonwith->vertACF(maxshift);
    ccf=r1->calcCCFX(r2,0,maxshift);
    r1->writeIMAGIC("racf.hed",0);
    r2->writeIMAGIC("racf.hed",1);
    delete r1;
    delete r2;

    d=ccf->getDataRO();
    pv=0;
    for (i=0; i<size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; }
    ccf->doneData();
    delete ccf;

    if (px>size/2) lda=size-px;
    else lda=px;
    printf("R Peak %d\t%g\t%1.2f\n",lda,pv,lda*360.0/size);
	
    //lda=0;
    d=radonthis->getDataRO();
    d2=radonwith->getDataRO();
	
    // Now we calculate vertical offsets at each horizontal position
    // this should be a sine wave in x, with the amplitude representing
    // the magnitude of the translation and the phase representing the
    // direction. We do this in real-space
    for (x=0,x2=lda%size; x<size/2; x++,x2=(x2+1)%size) {
      float best=0;
      int besti=0;
      for (i=-maxshift; i<maxshift; i++) {
	float dot=0;
	for (y=maxshift; y<size-maxshift; y++) {
	  dot+=d2[x2+y*size]*d[x+(y+i)*size];
	}
	if (dot>best) { best=dot; besti=i; }
	//printf("%1.2g  ",dot);
      }
      vert[x]=besti;
      printf("%d\t%d\n",x,besti);
    }
	
    // Now compute the phase and amplitude to decide the direction and
    // magnitude of the translation
    si=co=max=ta=0;
    for (x=0; x<size/2; x++) { 
      si+=sin(x*2.0*PI/size)*vert[x]; 
      co+=cos(x*2*PI/size)*vert[x]; 
      if (abs(vert[x])>max) max=abs(vert[x]);
    }
    float inten=(si*si+co*co)/(size*PI);
    ta=atan2(co,si);
    printf("x,y = %g,%g\ta,p=%g / %g,%g\n",co,si,sqrt(inten),max,180.0/PI*ta); 
    max=floor(sqrt(inten)+.5);	//let's try this...
	
    t1->doneData();
    radonwith->doneData();
  }

  delete t1;

  t1=copy();
  t1->setRAlign(-lda*PI*2.0/size);
  t1->setTAlign(-max*cos(ta),-max*sin(ta));
  t1->rotateAndTranslate();

  if (Drt) delete radonthis;
  if (Drw) delete radonwith;
  return t1;

  /*
  // now the flipped version
  f1=radonthis->copy(0,0);
  f1->hFlip();
  f1->writeIMAGIC("radon.hed",2);

  // Now try to locate the Radon image in the reference
  ccf=f1->calcCCF(radonwith);
  //ccf->toCorner();
  ccf->writeIMAGIC("radon.hed",0);

  d=ccf->getDataRO();
  pv=0;
  for (i=0; i<size*size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; py=i/size; }
  printf("Peak %d,%d\t%g\n",px,py,pv);
  ccf->doneData();
  delete ccf;

  return copy(0,0);
  */
  /*
  // Now generate flipped and unflipped segments to use for a search
  // The first is the left 1/4 of the Radon transform
  t1=radonthis->copy(0,0);
  d=t1->getData()+w1;
  for (y=maxshift; y<size-maxshift; y++) memset(d+y*size,0,(size-w1)*sizeof(float));
  t1->doneData();

  // The second is shifted 1/8 to the right
  t2=radonthis->copy(0,0);
  d=t2->getData();
  for (y=maxshift; y<size-maxshift; y++) {
  memset(d+y*size+w1+w2,0,(size-w1-w2)*sizeof(float));
  memset(d+y*size,0,w2*sizeof(float));
  }
  t2->doneData();

  // horizontally flipped versions represent handedness change in real-space
  f1=t1->copy(0,0);
  f1->hFlip();
  f2=t2->copy(0,0);
  f2->hFlip();

  // Now try to locate the 1/4 image in the reference
  ccf=t1->calcCCF(radonwith);
  ccf->toCorner();
  ccf->writeIMAGIC("radon.hed",0);

  d=ccf->getDataRO();
  pv=0;
  for (i=0; i<size*size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; py=i/size; }
  printf("Peak %d,%d\t%g\n",px,py,pv);
  ccf->doneData();
  delete ccf;

  ccf=t2->calcCCF(radonwith);
  ccf->toCorner();
  ccf->writeIMAGIC("radon.hed",1);

  d=ccf->getDataRO();
  pv=0;
  for (i=0; i<size*size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; py=i/size; }
  printf("Peak %d,%d\t%g\n",px,py,pv);
  ccf->doneData();
  delete ccf;

  ccf=f1->calcCCF(radonwith);
  ccf->toCorner();
  ccf->writeIMAGIC("radon.hed",2);

  d=ccf->getDataRO();
  pv=0;
  for (i=0; i<size*size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; py=i/size; }
  printf("Peak %d,%d\t%g\n",px,py,pv);
  ccf->doneData();
  delete ccf;

  ccf=f2->calcCCF(radonwith);
  ccf->toCorner();
  ccf->writeIMAGIC("radon.hed",3);

  d=ccf->getDataRO();
  pv=0;
  for (i=0; i<size*size; i++) if (d[i]>pv) { pv=d[i]; px=i%size; py=i/size; }
  printf("Peak %d,%d\t%g\n",px,py,pv);
  ccf->doneData();
  delete ccf;

  return copy(0,0);
  */
  /*ccf=t1->calcCCFX(radonwith,maxshift,size-maxshift);
    d=ccf->getDataRO();
    pv=-1.0e30;
    for (i=0; i<size; i++) if (d[i]>pv) { pv=d[i]; px=i; }
    printf("Peak %d\t%g\n",px,pv);
    ccf->doneData();
    delete ccf;*/
}

// Rotational and translational alignment with possible flip
// if 'flip' is not provided, 'this' may be flipped on output
// (2d only)
EMData *EMData::RTFAlign(EMData *with,EMData *flip,int usedot,int maxshift) {
  EMData *cop,*cop2;
  float d1,d2;

  //with->writeIMAGIC("tst.hed",-1);
  cop=RTAlign(with,usedot,maxshift);

  //printf("------\n");

  if (flip) {
    cop2=flip->RTAlign(with,usedot,maxshift);
  }
  else {
    hFlip();
    cop2=RTAlign(with,usedot,maxshift);
  }

  if (cop==NULL) { error(ERR_WARN,"Align failed"); return cop2; }
  if (cop2==NULL) { error(ERR_WARN,"Align flip failed"); return cop; }

  //with->writeIMAGIC("abc",-1);
  //cop->writeIMAGIC("abc",-1);
  //cop2->writeIMAGIC("abc",-1);


  if (usedot) {
    d1=cop->dot(with);
    d2=cop2->dot(with);
    if (usedot==2) printf("%f vs %f  (%1.1f, %1.1f  %1.2f) (%1.1f, %1.1f  %1.2f)\n",
			  d1,d2,cop->Dx(),cop->Dy(),cop->alt()*180./PI,cop2->Dx(),cop2->Dy(),cop2->alt()*180./PI);
  }
  else {
#ifdef ALTCMP
    d1= cop->fscmp(with);
    d2=cop2->fscmp(with);
#else
    d1=2.0-cop->lcmp(with,1);
    d2=2.0-cop2->lcmp(with,1);
#endif
  }

  if (d1>d2) { 
    cop->setFlipped(0); 
    if (!flip) hFlip();
    delete cop2; 
    return cop; 
  }
  else { 
    cop2->setFlipped(1); 
    delete cop; 
    return cop2; 
  }

}

void EMData::rotateX(int DX) {
  float *tmp = (float *)malloc(xSize()*sizeof(float));
  getData();

  for (int y=0; y<ySize(); y++) {
    for (int x=0; x<xSize(); x++) {
      tmp[x]=rdata[y*xSize()+((x+DX)%xSize())];
    }
    memcpy(&rdata[y*xSize()],tmp,xSize()*sizeof(float));
  }

  doneData();
}

// Rotational, translational alignment with a possible flip, like above, but REALLY SLOW
// uses exhaustive search in x,y and correlation for angle determination
// if flip is not provided, the result will be parentless
EMData *EMData::RTFAlignSlow(EMData *with,EMData *flip,int maxshift) {
  if (maxshift<0) maxshift=xSize()/10;
  EMData *tmp=NULL,*df=NULL;
  EMData *ws=with->copy(0,0);

  int xst= (int) floor(2.0*PI*ySize());	// this is 1 pixel rotational accuracy at the edge
  //int xst= (int) floor(PI*ySize());	// this is 2 pixel rotational accuracy at the edge
  xst=bestfftsize(xst);

  ws->medianShrink(2);
  tmp=ws->unwrap(4,ws->ySize()/2-2-maxshift/2,xst/2,0,0,1);
  delete ws;
  ws=tmp;
  EMData *wsc=ws->copy(0,0);

  with=with->unwrap(4,with->ySize()/2-2-maxshift,xst,0,0,1);
  EMData *withc=with->copy(0,0);

  if (flip) df=flip->copy();
  else {
    df=copy(0,0);
    df->hFlip();
  }


  float astep=atan2(2.0,xSize());		// 1 pixel at the edge of the image accuracy

  // for the coarse search, we'll shrink the images by 2
  EMData *dns=copy(0,0);
  EMData *dfs=df->copy(0,0);
  dns->medianShrink(2);
  dfs->medianShrink(2);

  int bestflip=0,bestdx=0,bestdy=0;
  float bestang=0,bestval=1.0e32;
  int dflip;
  for (dflip=0; dflip<2; dflip++) {
    EMData *u;
    if (dflip) u=dfs; else u=dns;
    for (int DY=-maxshift/2; DY<=maxshift/2; DY++) {
      for (int DX=-maxshift/2; DX<=maxshift/2; DX++) {
	if (hypot((float)DX,(float)DY)>maxshift/2) continue;	// a little faster by igoring this area
	EMData *uw = u->unwrap(4,u->ySize()/2-2-maxshift/2,xst/2,DX,DY,1);
	EMData *uwc = uw->copy(0,0);
	EMData *a = uw->calcCCFX(ws);

	uwc->rotateX(a->MaxLoc());
#ifndef ALTCMP
	float cm = uwc->lcmp(wsc,1,1);
#else
	float cm = ySize()/2-uwc->fscmp(wsc);
#endif
	if (cm<bestval) {
	  bestval=cm; 
	  bestang=2.0*PI*a->MaxLoc()/(float)a->xSize(); 
	  bestdx=DX;
	  bestdy=DY; 
	  bestflip=dflip; 
	  //printf("%4s\t%1.4g\t%d,%d\t%1.5g\n",dflip?"dflip":"    ",bestang*180./PI,bestdx*2,bestdy*2,bestval);
	}
	delete a;
	delete uw;
	delete uwc;
      }
    }
  }
  delete dns;
  delete dfs;
  delete ws;
  delete wsc;
  bestdx*=2;
  bestdy*=2;
  bestval=1.0e32;

  //printf("--\n");

  // Now we do the final local search in the full-sized images
  int bestdx2=bestdx,bestdy2=bestdy;
  float bestang2=bestang;
  for (dflip=0; dflip<2; dflip++) {
    EMData *u;
    if (dflip) u=df; else u=this;
    if (dflip) { bestdx2=-bestdx2; bestang=-bestang; }
    for (int DY=bestdy2-3; DY<=bestdy2+3; DY++) {
      for (int DX=bestdx2-3; DX<=bestdx2+3; DX++) {
	if (hypot((float)DX,(float)DY)>maxshift) continue;	// a little faster by igoring this area
	EMData *uw = u->unwrap(4,u->ySize()/2-2-maxshift,xst,DX,DY,1);
	EMData *uwc = uw->copy(0,0);
	EMData *a = uw->calcCCFX(with);

	uwc->rotateX(a->MaxLoc());
#ifndef ALTCMP
	float cm = uwc->lcmp(withc,1,1);
#else
	float cm = ySize()/2-uwc->fscmp(withc);
#endif
	if (cm<bestval) {
	  bestval=cm; 
	  bestang=2.0*PI*a->MaxLoc()/(float)a->xSize(); 
	  bestdx=DX;
	  bestdy=DY; 
	  bestflip=dflip;
	}
	//printf("%4s\t%1.4g\t%d,%d\t%1.5g\n",dflip?"dflip":"    ",360.0*a->MaxLoc()/(float)a->xSize(),DX,DY,cm);
	delete a;
	delete uw;
	delete uwc;
      }
    }
  }

  delete with;
  delete withc;


  if (bestflip) {	
    df->setRAlign(bestang,0,0);
    df->setTAlign(-bestdx,-bestdy,0);
    df->rotateAndTranslate();
    df->setFlipped(1);
    return df;
  }

  delete df;
  EMData *dn;
  if (dflip) dn=copy(); else dn=copy(0,0);
  dn->setRAlign(bestang,0,0);
  dn->setTAlign(-bestdx,-bestdy,0);
  dn->rotateAndTranslate();

  return dn;
}

// Rotational, translational alignment with a possible flip, like above, but REALLY SLOW
// uses exhaustive search 
// if flip is not provided, the result will be parentless
EMData *EMData::RTFAlignSlowest(EMData *with,EMData *flip,int maxshift) {
  EMData *dn,*df;
  dn=copy();
  if (flip) df=flip->copy();
  else {
    df=copy();
    df->hFlip();
    df=df->copy();	// don't forget to delete df's parent
  }

  if (maxshift<0) maxshift=xSize()/10;

  float astep=atan2(2.0,xSize());		// 1 pixel at the edge of the image accuracy

  // for the coarse search, we'll shrink the images by 2
  EMData *dns=dn->copy(0,0);
  EMData *dfs=df->copy(0,0);
  EMData *ws=with->copy(0,0);
  dns->medianShrink(2);
  dfs->medianShrink(2);
  ws->medianShrink(2);
  dns=dns->copy();
  dfs=dfs->copy();

  int bestflip=0,bestdx=0,bestdy=0;
  float bestang=0,bestval=FLT_MAX;  // is 1.0e50 too big?
  int dflip;
  for (dflip=0; dflip<2; dflip++) {
    EMData *u;
    if (dflip) u=dfs; else u=dns;
    for (int dy=-maxshift/2; dy<=maxshift/2; dy++) {
      for (int dx=-maxshift/2; dx<=maxshift/2; dx++) {
	if (hypot((float)dx,(float)dy)>maxshift) continue;	// a little faster by igoring this area
	for (float ang=-astep*2.0; ang<=2*PI; ang+=astep*4.0) {
	  u->setRAlign(ang);
	  u->setTAlign(dx,dy);
	  u->rotateAndTranslate();
#ifndef ALTCMP
	  float lc=u->lcmp(ws,1,1);
#else
	  float lc=ySize()/2-u->fscmp(ws);
#endif				
	  if (lc<bestval) { 
	    bestval=lc; bestang=ang; bestdx=dx; bestdy=dy; bestflip=dflip; 
	    //					printf("%4s %3d,%3d   %1.4g\t%1.4g\n",bestflip?"flip":"    ",bestdx*2,bestdy*2,bestang*180./PI,bestval);
	  }
	}
      }
    }
  }
  delete dns->Parent();
  delete dfs->Parent();
  delete dns;
  delete dfs;
  delete ws;
  bestdx*=2;
  bestdy*=2;
  bestval=1.0e32;

  int bestdx2=bestdx,bestdy2=bestdy;
  float bestang2=bestang;
  for (dflip=0; dflip<2; dflip++) {
    EMData *u;
    if (dflip) u=df; else u=dn;
    if (dflip) bestdx2=-bestdx2;
    for (int dy=bestdy2-3; dy<=bestdy2+3; dy++) {
      for (int dx=bestdx2-3; dx<=bestdx2+3; dx++) {
	if (hypot((float)dx,(float)dy)>maxshift) continue;	// a little faster by igoring this area
	for (float ang=bestang2-astep*6.0; ang<=bestang2+astep*6.0; ang+=astep) {
	  u->setRAlign(ang);
	  u->setTAlign(dx,dy);
	  u->rotateAndTranslate();
#ifndef ALTCMP
	  float lc=u->lcmp(with,1,1);
#else
	  float lc=ySize()/2-u->fscmp(ws);
#endif
				
	  if (lc<bestval) { 
	    bestval=lc; bestang=ang; bestdx=dx; bestdy=dy; bestflip=dflip; 
	    //					printf("%4s %3d,%3d   %1.4g\t%1.4g\n",bestflip?"flip":"    ",bestdx,bestdy,bestang,bestval);
	  }
	}
      }
    }
  }

  if (bestflip) {
    delete dn;
    df->setRAlign(bestang,0,0);
    df->setTAlign(bestdx,bestdy,0);
    df->rotateAndTranslate();
    if (!dflip) { delete df->Parent(); df->setParent(NULL); }
    df->setFlipped(1);
    return df;
  }

  dn->setRAlign(bestang,0,0);
  dn->setTAlign(bestdx,bestdy,0);
  dn->rotateAndTranslate();
  //dn->setParent(NULL);

  if (!dflip) delete df->Parent();
  delete df;
  return dn;
}


// This will take 2 images that are aligned pretty well and do the
// rest of the job (mode = -1 is a debugging flag)
// mode = 1 is normal
// This is basically accurate to 1/2 pixel
void EMData::refineAlign(EMData *with,int mode) {
  float sda=euler.alt(),sdx=dx,sdy=dy,dda=atan(2.0/ny),best,f;
  int i,j=1;
  float daz;

#ifndef ALTCMP
  best=2.0-lcmp(with,1);
#else
  best=fscmp(with);
#endif

  if (!parent) printf("Warning: no parent in refineAlign\n");
  if (mode<0) printf("Start %f  %f,%f\n",sda*180.0/PI,sdx,sdy);
  for (i=0; i<5 && j; i++) {
    j=0;	// break out if nothing changes
	
    for (daz=sda-dda; daz<=sda+dda; daz+=dda/2.0) {
      if (daz==sda) continue;
      setRAlign(daz,0,0);
      rotateAndTranslate();
#ifndef ALTCMP
      f=2.0-lcmp(with,1);
#else
      f=fscmp(with);
#endif
      if (f>best) { best=f; sda=daz; j=1; break; }
    }
	
    for (dx=sdx-1.0; dx<=sdx+1.0; dx+=.5) {
      for (dy=sdy-1.0; dy<=sdy+1.0; dy+=.5) {
	rotateAndTranslate();
#ifndef ALTCMP
	f=2.0-lcmp(with,1);
#else
	f=fscmp(with);
#endif
	if (f>best) { best=f; sdx=dx; sdy=dy; j=1; break; }
      }
    }			
    if (mode<0) printf("-- %f  %f,%f (%1.3g) \n",sda*180.0/PI,sdx,sdy,best);
  }
  setRAlign(sda,0,0);
  dx=sdx;
  dy=sdy;
  rotateAndTranslate();

  ufloat4=best;

  return;
}

// Used by RTFAlign
EMData *EMData::RTAlign(EMData *with,int usedot,int maxshift) {
  EMData *cop,*cop2,*rfp,*rfpw;
  int i;
  float cda=0,cda2=0,cdx=0,cdy=0;

  cop=copy();

  rotAlign(with);
  cda=cda2=alt();

  // first align
  cop->setRAlign(cda,0,0);
  cop->setTAlign(0,0,0);
  cop->rotateAndTranslate();
  cop2=cop->copy();
  cop2->setParent(cop->Parent());

  cop->transAlign(with,0,1,maxshift);
  cdx= cop->dx*cos(cda)+cop->dy*sin(cda);
  cdy=-cop->dx*sin(cda)+cop->dy*cos(cda);
  cop->fastTranslate();
  cop->setRAlign(cda,0,0);
  cop->setTAlign(cdx,cdy,0);
  //cop->rotateAndTranslate();


  //cop->writeIMAGIC("abc.hed",-1);

  // now try 180 deg rot
  cda2+=PI;
  cop2->setRAlign(cda2,0,0);
  cop2->setTAlign(0,0,0);
  cop2->rot180();
  //cop2->rotateAndTranslate();

  cop2->transAlign(with,0,1,maxshift);
  cdx= cop2->dx*cos(cda2)+cop2->dy*sin(cda2);
  cdy=-cop2->dx*sin(cda2)+cop2->dy*cos(cda2);
  cop2->fastTranslate();
  cop2->setRAlign(cda2,0,0);
  cop2->setTAlign(cdx,cdy,0);
  //cop2->rotateAndTranslate();

  //cop2->writeIMAGIC("abc.hed",-1);

  float dot1,dot2;
  if (usedot) {
    dot1=cop->dot(with);
    dot2=cop2->dot(with);
  }
  else {
#ifndef ALTCMP
    dot1=2.0-cop->lcmp(with,1);
    dot2=2.0-cop2->lcmp(with,1);
#else
    dot1=cop->fscmp(with);
    dot2=cop2->fscmp(with);
#endif
  }
  if (dot1>dot2) {
    /*	cop->rotAlign(with);
	cda+=cop->alt();
	cda=cda-floor(cda/(2.0*PI))*2.0*PI;
	cop->setRAlign(cda,0,0);
	cop->setTAlign(cdx,cdy,0);
	cop->rotateAndTranslate();*/

    cop->ufloat4=dot1;
    delete cop2;
    return cop;
  }

  /*cop2->rotAlign(with);
    cda2+=cop2->alt();
    cda2=cda2-floor(cda2/(2.0*PI))*2.0*PI;
    cop2->setRAlign(cda2,0,0);
    cop2->setTAlign(cdx,cdy,0);
    cop2->rotateAndTranslate();*/

  cop2->ufloat4=dot2;
  delete cop;
  return cop2;

  //printf("%5.1f\n",cda*180.0/PI);
  /*
  // iterate translate/rotate 3 times
  for (i=0; i<5; i++) {
  cop->transAlign(with,0,1,0);
  cdx+= cop->dx*cos(cda)+cop->dy*sin(cda);
  cdy+=-cop->dx*sin(cda)+cop->dy*cos(cda);
  cop->setRAlign(cda,0,0);
  cop->setTAlign(cdx,cdy,0);
  cop->rotateAndTranslate();
  cop->writeIMAGIC("abc.hed",-1);

  cop->rotAlign(with);
  printf("%5.1f\n",cop->alt()*180.0/PI);
  cda+=cop->alt();
  cda=cda-floor(cda/(2.0*PI))*2.0*PI;
  cop->setRAlign(cda,0,0);
  cop->setTAlign(cdx,cdy,0);
  cop->rotateAndTranslate();
  cop->writeIMAGIC("abc.hed",-1);
  printf("%5.1f\t%5.1f\t%5.2f\n",cdx,cdy,cda*180.0/PI);
  }
  */
  //return cop;
}

// Used by RTFAlign
/*
  EMData *EMData::RTAlign(EMData *with,int usedot) {
  EMData *cop,*cop2;
  float v,a;
  float tx,ty;
  float dot,dot2;

  //with->writeIMAGIC("tst.hed",-1);

  cop=copy();
  cop->setRAlign(0,0,0);
  cop->setTAlign(0,0,0);
  cop->rotAlignTI(with,usedot);
  a=cop->euler.alt();

  cop->rotateAndTranslate();
  cop->transAlign(with,0,0,0);
  cop->setRAlign(a,0,0);
  tx=cop->DX()*cos(a)+cop->Dy()*sin(a);
  ty=-cop->DX()*sin(a)+cop->Dy()*cos(a);
  cop->setTAlign(tx,ty,0);
  cop->rotateAndTranslate();
  if (usedot) dot2=cop->dot(with);
  else dot2=-cop->lcmp(with);
  cop2=cop;
  //cop->writeIMAGIC("tst.hed",-1);

  cop=copy();
  cop->setTAlign(0,0,0);
  if (a>PI) a=a-PI,0,0; else a=a+PI;
  cop->setRAlign(a,0,0);
  cop->rotateAndTranslate();
  cop->transAlign(with,0,0,0);
  cop->setRAlign(a,0,0);
  tx=cop->DX()*cos(a)+cop->Dy()*sin(a);
  ty=-cop->DX()*sin(a)+cop->Dy()*cos(a);
  cop->setTAlign(tx,ty,0);
  cop->rotateAndTranslate();
  if (usedot) dot=cop->dot(with);
  else dot=-cop->lcmp(with);
  //cop->writeIMAGIC("tst.hed",-1);

  //printf("%f\t%f\n",dot2,dot);

  if (dot>dot2) { delete cop2; return cop; }

  delete cop;
  return cop2;
  }
*/

// This does a very fast rotation by 180 degrees
// ignores the leftmost column and the bottom row completely
void EMData::rot180() {
  if (xSize()!=ySize()) error(ERR_WARN,"Warning, Rot180 on non-square image poorly defined.");

  float *d = getData();
  float t;
  for (int x=1; x<xSize(); x++) {
    int y;
    for (y=1; y<ySize(); y++) {
      if (x==xSize()/2 && y==ySize()/2) break;
      t=d[x+y*xSize()];
      d[x+y*xSize()]=d[(xSize()-x)+(ySize()-y)*xSize()];
      d[(xSize()-x)+(ySize()-y)*xSize()]=t;
    }
    if (x==xSize()/2 && y==ySize()/2) break;
  }
  doneData();

}

void EMData::zero() {
  getData();
  memset(rdata,0,nx*ny*nz*sizeof(float));
  doneData();
  update();
}

void EMData::one() {
  if (isComplex()) {
    setRI(1);
    getData();
    //	for (int i=0; i<nx*ny*nz; i+=2) { rdata[i]=1.0; rdata[i+1]=0; }
    for (int i=0; i<nx*ny*nz; i++) rdata[i]=1.0;
  }
  else {
    getData();
    for (int i=0; i<nx*ny*nz; i++) rdata[i]=1.0;
  }
  doneData();
  update();
}

// check to see if 2 images have identical CTF parameters
// 0 if same, <0 or >0 indicate differences
// in an unspecified sortable way, 1 if either has no ctf
int EMData::CTFCmp(EMData *with) {
  if (!(hasCTF()&&with->hasCTF()) && !(hasCTFF()&&with->hasCTFF())) return 1;

  if (hasCTF()) return memcmp(getCTF(),with->getCTF(),11*sizeof(float));

  int n1,n2;
  char s1[80],s2[80];
  sscanf(      Name(),"!$ %d,%s",&n1,s1);
  sscanf(with->Name(),"!$ %d,%s",&n2,s2);

  int r;
  r=strcmp(s1,s2);
  if (!r) return n1-n2;
  return r;
}

// This attempts to automatically mask out a par
void EMData::autoMask(float thresh,float filter) {
  int j,k,l;

  EMData *d=copy(0,0);

  if (thresh>0) {
    thresh= -thresh;
    d->multConst(-1.0);
  }

  d->filter(0,filter*ySize()/2.0,1);
  d->normalize();

  float *dn = (float *) calloc(d->xSize()*d->ySize()*sizeof(float),1);
  float *dc = (float *) calloc(d->xSize()*d->ySize()*sizeof(float),1);
  float *dd = d->getData();
		
  // first we find all dark areas that are likely to be part of a particle
  int nx=d->xSize(),ny=d->ySize();
  for (l=0,k=0; l<ny; l++) {
    for (j=0; j<nx; j++,k++) {
      if (hypot(l-ny/2,j-nx/2)>=ny/2) dn[k]=-10;
      else if (dd[k]<thresh*d->Sigma()) {
	dn[k]=10;
      }
      else {
	dn[k]=0;
      }
    }
  }
		
  // now we iteratively mask out the areas which contact the edge of the image
  int ch=1;
  while (ch) {
    ch=0;
    memcpy(dc,dn,nx*ny*sizeof(float));
    for (l=1; l<ny-1; l++) {
      for (j=1; j<nx-1; j++) {
	k=j+l*nx;
	if (dn[k]==-10) continue;
	if (dn[k]==10) {
	  if (dn[k-1]==-10 ||dn[k+1]==-10 ||dn[k-nx]==-10 ||dn[k+nx]==-10) { dn[k]=-10; ch=1; }
	}					
      }
    }
  }

  // Now we clean up some of the less probable areas that weren't masked out
  for (l=0,k=0; l<ny; l++) {
    for (j=0; j<nx; j++,k++) {
      if (l==0 || l==ny-1 || j==0 || j==nx-1) dn[k]=-10;
      else if (dd[k]>thresh*d->Sigma()*2.0 && dn[k]==10) dn[k]=0;
    }
  }
		
  // Now we iteratively try to find a good region to mask based on nearness to masked areas
  float v,v2,t;
  ch=1;
  int chl=0;
  while (ch) {
    ch=0;
    memcpy(dc,dn,nx*ny*sizeof(float));
    for (l=1; l<ny-1; l++) {
      for (j=1; j<nx-1; j++) {
	k=j+l*nx;
	if (dn[k]!=-10.0 && dn[k]!=10.0) {
	  v=maxf(dc[k-1],dc[k+1],dc[k-nx],dc[k+nx]);
	  v2=minf(dc[k-1],dc[k+1],dc[k-nx],dc[k+nx]);
					
	  if (v2>=0 && v>.25) dn[k]=v-.25;
	  else if (v<=0 && v2<-.25) dn[k]=v2+.25;
	  else if (v>.25 && v2<-.25) dn[k]=(v+v2)/2.0;
						
	  if (dn[k]!=dc[k]) ch++;
	}
					
      }
    }
  }
		
  // make a real mask
  for (k=0; k<nx*ny; k++) {
    if (dn[k]>=-5.0) dn[k]=1.0; else dn[k]=0;
  }

  // eliminate isolated nonmasked regions
  dn[nx*ny/2+nx/2]=2.0;
  ch=1;
  while (ch) {
    ch=0;
    for (l=1; l<ny-1; l++) {
      for (j=1; j<nx-1; j++) {
	k=j+l*nx;
	if (dn[k]==1.0) {
	  v=maxf(dn[k-1],dn[k+1],dn[k-nx],dn[k+nx]);
	  if (v==2.0) { dn[k]=2.0; ch=1.0; }
	}					
      }
    }
  }
		
  // Expand the edges a little to mmake it unlikely that we're chopping off something we shouldn't
  for (ch=0; ch<4; ch++) {
    memcpy(dc,dn,nx*ny*sizeof(float));
    for (l=1; l<ny-1; l++) {
      for (j=1; j<nx-1; j++) {
	k=j+l*nx;
	if (dc[k]!=2.0) {
	  v=maxf(dc[k-1],dc[k+1],dc[k-nx],dc[k+nx]);
	  if (v==2.0) dn[k]=2.0;
	}
      }
    }
  }
		
  // now make it into a mask again
  for (k=0; k<nx*ny; k++) {
    if (dn[k]==2.0) dn[k]=1.0; else dn[k]=0;
  }

  memcpy(dd,dn,nx*ny*sizeof(float));
  free(dn);
  free(dc);
  d->doneData();

  mult(d);
  delete d;
}
